# bf.tcl---base frame menu for eg-session

#    21 Mar 1996 VP  restructure to use include files
#       now only tk version  file is bf.tk.tcl
#       -  previous version kept for xv use as bf.xv.tcl
#    15 Aug 1995 VP  modify for conversion to TK


set debug 0

set frame BF
set frametitle "Midas Base Frame"
set icontitle "Midas"
set iconimage "@$env(ICONHOME)/midas"

set preferences(QuitOption1) 1
set preferences(QuitOption2) 1


proc show_logo {logo} {
  global env

  set file ""

  if {$logo == "midas"} then {
     set f [file join $env(ICONHOME) midas.gif]
     if {[file exists $f]} then {set file $f}
  } else {
     set f1 [file join $env(ICONHOME) $logo.gif]
     set f2 [file join $env(ICONHOME) midas.gif]
     if {[file exists $f1]} then {set file $f1} \
       else {if {[file exists $f2]} then {set file $f2}}

     if {[info exists env(NOLOGO)]} then {set file ""}
  }

  if {$file == ""} then {return}

  if {$logo == "midas"} then {
      :frame LOGOFRAME :label "Midas logo" :toplevel T :colours H
  } else {
      :frame LOGOFRAME :label "logo" :toplevel T :colours H
  }

  if {[winfo depth [:path LOGOFRAME]] > 8} then {
     image create photo logo -format gif -file $file
  } else {
     image create photo logo -format gif -palette 3/3/3 -file $file
  }

  :canvas LOGO :w 308 :h 308
  [:path LOGO] create image 154 154  -image logo

  :show T
}

proc show_motto {} {
  global env

  set file [file join $env(ICONHOME) motto.gif]
  :frame MOTTOFRAME :label "Midas motto" :toplevel T :colours H

  if {[winfo depth [:path MOTTOFRAME]] > 8} then {
     image create photo motto -format gif -file $file
  } else {
     image create photo motto -format gif -palette 3/3/3 -file $file
  }

  :canvas MOTTO :w 900 :h 300
  [:path MOTTO] create image 450 150  -image motto

  :show T
}

proc delete_logo {} {
    catch  {destroy-frame [:path LOGOFRAME]}
    raise .
}

# N.B the # in the list below is MANDATORY
# The following loop looks for it to determine the end of the list element
#

set tasklist {
  {testtask                     #   test task}
}

#    define following for the inform procedure

set informlocation "bf.tcl"
set mytaskname     bf

#    include optional components

set menu {}

proc include {option} {
   global menu MENU
   loadsource bf.$option
   foreach item [array names MENU] {
      if {[lsearch -exact $menu $item] < 0} then {lappend menu $item}
   }
}

set OPTION(ACCESS) 1
set OPTION(DIRECTORY) 1
set OPTION(XM) 1
set OPTION(PREFERENCES) 1
set OPTION(HARDCOPY) 1
set OPTION(EXTRAS) 1
set OPTION(HELP) 2
set OPTION(SL) 2
set OPTION(BF) 1

proc exclude {option} {
  global OPTION

   if {$option == "ALL"} then {
      set OPTION(ACCESS) 0
      set OPTION(DIRECTORY) 0
      set OPTION(XM) 0
      set OPTION(PREFERENCES) 0
      set OPTION(HARDCOPY) 0
      set OPTION(EXTRAS) 0
      set OPTION(HELP) 0
      set OPTION(SL) 0
      set OPTION(BF) 0
   } else {
      set OPTION($option) 0
   }
}

loadsource bf.includes

#    Note - the following code MUST occur after the tasklist is complete

#    we put the base frame task list into the env array because it does not
#    appear that the send command will pass it correctly back

set env(bf_tasklist) $tasklist

bf_tasks

proc click-access m {
   if {$m=="Experiment Access"} then {inform ex start; return}
   if {$m=="Apparatus Access"}  then {inform ap start; return}
   if {$m=="Security"}          then {inform security start; return}
}

proc click-preferences m {
   if {$m=="Save Preferences"} then {click_update_preferences; return}
   if {$m=="Edit Preferences"} then {inform pedit start; return}
   if {$m=="Edit Options"}     then {inform oedit start; return}
   if {$m=="Printing"}         then {inform opt-misc start; return}
   if {$m=="Colours"}          then {inform opt-colours start; return}
   if {$m=="Fonts"}            then {inform opt-fonts start; return}
}

set ExtrasMenu [list "Text Editor" "Terminal" "HTML Viewer" "Demos"]
proc click-extras m {
   if {$m=="Text Editor"}        then {inform editor start; return}
   if {$m=="HTML Viewer"}        then {inform htmlviewer start; return}
   if {$m=="Terminal"}           then {inform terminal start; return}
   if {$m=="Demos"}              then {inform opt-demos start; return}
}

proc click_update_preferences {} {

   foreach app [Get_Task_Names] {
      if {$app != "bf"} then {catch {inform $app update_preferences}}
   }
   update_preferences
}

proc click-hardcopy m {
   if {$m=="Screen Dump"} then {midas-capture screen; return}
   if {$m=="Window Dump"} then {midas-capture window; return}
}

proc click-help m {
   if {$m=="About Midas"}         then {show_logo midas}
   if {$m=="Online Manual"}       then {inform help show-manual}
   if {$m=="Midas Documents"}     then {inform help show-document}
   if {$m=="manual pages"}        then {inform help show-manpages contents}
}

proc recency f {
   set fday [expr [file mtime $f]/3600/24]
   set today [expr [first [clock]]/3600/24]
   set d [expr $today-$fday]
   if {$d==0} then {return "today!!!"} \
   elseif {$d==1} then {return "yesterday!!"} \
   elseif {$d<15} then {return "[expr $d] days ago"} \
   elseif {$d<90} then {return "[expr round($d/7)] weeks ago"} \
   else {return "[expr round ($d/30)] months ago"}
   }

proc lookup {k ps} {
   foreach p $ps {
      if {[first $p]==$k} then {return [second $p]}
   }
   error "keyword $k not in dictionary \{$ps\}"
}

proc click arg {
   global debug
   if {$debug} then {puts stdout $arg}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$c==":quit"} then {
       if {$f == "LOGOFRAME" || $f == "MOTTOFRAME"} then {
          destroy-frame $f
       } else {
          click-quit $f
       }
       return
   }
   set-busy
   switch $w  {
      ACCESS      {click-access $v}
      DIRECTORY   {inform sb start}
      XM          {inform xm start}
      PREFERENCES {click-preferences $v}
      HELP        {click-help $v}
      HARDCOPY    {click-hardcopy $v}
      EXTRAS      {click-extras $v}
      QUIT        {click-quit}
      default     {
            global ACTION
            set names [array names ACTION]
            foreach name $names {
               if {$w == [first $ACTION($name)]} then {eval [second $ACTION($name)]; break}
            }
      }
   }
   clear-busy
}


proc start args {
   global env OPTION

#     task creates et al from init.tcl

#    following lines for backwards compatibility
  if {[info exists env(EXPERIMENTDIR)]} then {
     if {![info exists env(EXPERIMENTBASE)]} then {set env(EXPERIMENTBASE) $env(EXPERIMENTDIR)}
  }

  if {![info exists env(EXPERIMENTBASE)]} then {
     set z [catch {
        set env(EXPERIMENTBASE) [get-eg-option experiment-directory]
     } m]
     if {$z != 0} then {
       if {[info exists env(EXPERIMENTBASE)]} then {unset env(EXPERIMENTBASE)}
       if {[info exists env(EXPERIMENTDIR)]}  then {unset env(EXPERIMENTDIR)}
     }
  }

  if {$args != ""} then {set OPTION(BF) 0}

  if {$OPTION(BF) > 0} then {show_logo logo}

  load_preferences

  make_frame
 
  if {[info exists env(EXPERIMENTBASE)]} then {
      if {$env(EXPERIMENTBASE) != ""} then {inform ex auto-join}
  }

  if {$OPTION(BF) > 0} then {delete_logo}

  if {$args != ""} then {inform [first $args] start}
}


proc make_frame {} {
  global env iconimage OPTION ExtrasMenu

#    icon windows are not supported by windows
  if {$env(platform) == "unix"} then {midas_icon}

  :frame [self frame-name] :label "[self frame-title]" :show-footer T :win-ctrl N \
        :icon-name "[self icon-title]"  :icon-image $iconimage :resize F

   if {$env(platform) == "unix"} then {wm iconwindow . [:path MIDASICON]}

       :panel MENU :layout V

         if {$OPTION(ACCESS) > 0} then {
            :menubutton ACCESS :label "Access Control" \
                :strings [list "Experiment Access" "Apparatus Access" Security]
         }

         if {$OPTION(DIRECTORY) > 0} then {
            :button DIRECTORY :label "Spectrum Viewer"
         }

            global MENU menu
            foreach name $menu {eval $MENU($name)}

         if {$OPTION(XM) > 0} then {
            :button XM :label "Exception Monitor"
            send xm start_closed
         }

         if {$OPTION(PREFERENCES) > 0} then {
            :menubutton PREFERENCES :label "Preferences & Options" \
                :strings [list "Save Preferences" "Edit Preferences" "Edit Options" "Printing" "Colours" "Fonts"]
         }

         if {$OPTION(HARDCOPY) > 0} then {
            :menubutton HARDCOPY :label "Hard Copy" \
                :strings [list "Screen Dump" "Window Dump"]
         }

         if {$OPTION(EXTRAS) > 0} then {
            :menubutton EXTRAS :label "Extras" :strings $ExtrasMenu
         }

         if {$OPTION(HELP) > 0} then {
            :menubutton HELP :label "Help" \
               :strings [list "Online Manual" "Midas Documents" "manual pages" "About Midas"]
         }

         if {$OPTION(HELP) == 2} then {inform help start}

         if {$OPTION(SL) == 2} then {inform sl start}

#    the width of the QUIT button defines the width of the base frame
        :button QUIT :label "Quit" :w 24

      if {$env(platform) == "unix"} then {
        bind [:path QUIT] <ButtonPress-3>  "inform rmt start"
      } else {
        bind [:path QUIT] <ButtonPress-3>  "inform rmtcntrl start"
      }
      bind [:path QUIT] <ButtonPress-2>          "inform rmtcntrl start"
      bind [:path QUIT] <Control-ButtonPress-1>  "inform rmtcntrl start"
      bind [:path QUIT] <Control-ButtonPress-3>  "inform rmtcntrl start"
      bind [:path QUIT] <Shift-ButtonPress-3>      "show_motto"

         if {$OPTION(BF) > 0} then {:show T; position-frame 8 24}

}

proc do-quit {opt} {
   global env preferences
   
   destroy-frame [self frame-name].quit
   
   if {$opt == "N"} {return}
   
   update_preferences

   catch {:file delete [glob [file join $env(MIDASTempDir) tcl[pid]*]]}
   catch {:file delete [file join $env(MIDASTempDir) tcl[pid]]}
   
   if {$preferences(QuitOption1) == 1} {click_update_preferences}
   
   if {$preferences(QuitOption2) == 1} {

#
#    send "finish" command to all known interpreters
#
       foreach app [Get_Task_Names] {if {$app != "bf"} then {catch {inform $app finish}}}
   }

   exit
}

proc click-quit {args} {
   global preferences

  :frame [self frame-name].quit :label "Quit Session Confirmation" \
       :show-footer F :resize F :toplevel T :transient T

    :panel PANEL1 :layout T :padx 10 :pady 10
      :layout V; :row-gap 5; :col-gap 10
      
      :message M1   :label "Select the session terminate options you require"  :bold T
      
      :checkbox OPT1  :label "Save all task preferences"   :layout L  :value $preferences(QuitOption1) \
           :command "SelectOption 1"
      :checkbox OPT2  :label "Send \"finish\" to all tasks"  :layout L  :value $preferences(QuitOption2) \
           :command "SelectOption 2"
      
      :vskip 10
      :message M2 :label "Are you sure you wish to end this MIDAS session?" :bold T

      :next-row T
      :layout H
      :hskip 25
      :button YES  :label "Yes"  :w 16 :command "do-quit Y"
      focus [:path YES]
      :button NO   :label "No"   :w 8  :command "do-quit N"

   :show T
}
   
proc SelectOption {opt w v} {global preferences; set preferences(QuitOption$opt) $v}
            
proc finish args {inform root restart-bf}
