# x2vbdiag.tcl-- F2VB & D2VB module diagnostic frame

#    source common to both f2vb and d2vb modules

set debug 0
set started 0

set Doption 0
set meminc 0

set dumpopt 2
set dumpdata ""
set dumplist ""
set dumpstart 0
set dumpwords 64

set x2VBmemOffset 0
set memdisplaysize 16

proc update-widget {widget value} {
   :modify $widget :value $value
}

proc write-register {name value} {
   global resource mname
   set register /$resource/$mname.$name
   set-footer "writing $value to $register"
   EG "register :pattern '$register' :value '$value'"
   clear-footer
}

proc read-register {name} {
   global resource mname
   set register /$resource/$mname.$name
   set-footer "reading $register"
   set value [EG "register :name '$register' ?value"]
   clear-footer
   return $value
}


proc update_x2vbregisters {} {
#    the x2VB 16 bit registers
   update-widget CSR [read-register DCSR]
   update-widget CCR [read-register DCCR]
   update-widget ICR [read-register DICR]
   update-widget IVR [read-register DIVR]
   update-widget LRAR [read-register DLRAR]
   update-widget DRAR [read-register DDRAR]
#    the x2VB 32 bit registers
   update-widget CBRA [read-register DCBRA]
   update-widget CBWA [read-register DCBWA]
}

proc update_x2vbcsr {} {
   set csr [read-register DCSR]
   foreach i {0 1 2 3 4 5 6 7 8 9 10} {update-widget CSRB[set i] [expr $csr >> $i & 1]}
}

proc update_x2vbccr {} {
   set ccr [read-register DCCR]
   foreach i {8 9 10 11 13} {update-widget CCRB[set i] [expr $ccr >> $i & 1]}
}


proc do_readRAM {} {
  global resource mname DataWidth
  global x2VBmemOffset meminc Doption
  global dumpdata dumplist dumpwords
  if {$Doption == 0} then {
     set register /$resource/$mname.Dbram
     set reg Dbram
     set len $dumpwords
  } else {
     set register /$resource/$mname.Dlram
     set reg Dlram
     if {$Doption == 2} {set len 8} else {set len 96}
  }
  EG "register :name '$register' :object '-o $x2VBmemOffset -l $len'"
  :modify MEMDISPLAY :strings "\n  Memory Offset $x2VBmemOffset\n"
  switch $DataWidth {
     32   {
        if {$Doption == 0} then {
            set data [string trimleft [read-register $reg]]
        } else {
           set data ""
           set i 0
           foreach word [string trimleft [read-register $reg]] {
              append data "$word "
              incr i
              if {$i == 8} then {append data "\n"; set i 0}
           }
        }
     }
     16   {
        if {$Doption == 0} then {
           set data ""
           foreach word [string trimleft [read-register $reg]] {
              append data "0x[string range $word 2 5] 0x[string range $word 6 9] "
           }
        } else {
           set data ""
           set i 0
           foreach word [string trimleft [read-register $reg]] {
              append data "$word "
              incr i
              if {$i == 8} then {append data "\n"; set i 0}
           }
        }
     }
  }
  :modify MEMDISPLAY :strings "$data"
  if {$meminc != 0} then {
     if {$Doption == 0} then {
        set x2VBmemOffset [expr $x2VBmemOffset + 256]
     } else {
        set x2VBmemOffset [expr $x2VBmemOffset + 128]
     }
     :modify MEMOFFSET :value [format "0x%X" $x2VBmemOffset]
  }
  if {$Doption == 0} then {set dumpdata $data} else {set dumplist $data}
}

proc click-redisplay {} {update_x2vbregisters; update_x2vbcsr; update_x2vbccr}

proc show_current  {}  {
    global x2VBmemOffset Doption dumplist
    set x2VBmemOffset [read-register DLRAR]
    :modify MEMOFFSET :value [format "0x%X" $x2VBmemOffset]
    set Doption 2
    do_readRAM
    set x2VBmemOffset  [format "0x%x" [expr [first $dumplist] << 5]]
    :modify MEMOFFSET :value $x2VBmemOffset
    set Doption 0
    do_readRAM
}

proc click arg {
   global debug x2VBmemOffset meminc Doption Dctrdata dumpopt
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$c==":quit"} then {click-quit; return}
   switch $w {

      CSR     {write-register DCSR $v}
      CCR     {write-register DCCR $v}
      ICR     {write-register DICR $v}
      IVR     {write-register DIVR $v}
      DRAR    {write-register DDRAR $v}

      CSRB0   {}
      CSRB1   {}
      CSRB2   {}
      CSRB3   {}

      CCRB8   {}
      CCRB9   {}
      CCRB10  {}
      CCRB11  {}
      CCRB13  {}

      MEMOFFSET {set x2VBmemOffset $v}

      RDRAM     {do_readRAM}
      EMPTY     {:modify MEMDISPLAY :empty T}

      INCR      {set meminc $v}
      RDOPT     {set Doption $v}
      DCTR      {set Dctrdata $v}

      FORMATOPT {set dumpopt $v}

      REDISPLAY {click-redisplay}
      default   {puts stdout $arg}
   }
}




proc make-frame {} {
  global x2VBmemOffset memdisplaysize

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-name "[self icon-title]" :resize F

   :panel P1 :layout L :padx 10 :pady 10
      :layout V; :row-gap 10; :col-gap 10

      :text CSR  :label "Card status register:" :w 10 :value "0x0000" :valuex 28c
      :text CCR  :label "Card control register:" :w 10 :value "0x0000" :valuex 28c
      :text ICR  :label "Interrupt control register:" :w 10 :value "0x0000" :valuex 28c
      :text IVR  :label "Interrupt vector register:" :w 10 :value "0x0000" :valuex 28c
      :text LRAR :label "Current List RAM address:" :w 10 :value "0x0000" :valuex 28c :deaf T
      :text DRAR :label "Buffer RAM base address:" :w 10 :value "0x0000" :valuex 28c

      :text CBRA :label "Current buffer RAM read address:" :w 12 :value "0x000000" :valuex 34c :deaf T
      :text CBWA :label "Current buffer RAM write address:" :w 12 :value "0x000000" :valuex 34c :deaf T

   :panel P2 :layout L :padx 10 :pady 10
      :layout H; :row-gap 0; :col-gap 10
      :message P2M1 :label "Status Register" :bold T :valuex 32c
      :next-row T
      :checkbox CSRB0 :label "List ended" :valuex 12c
      :checkbox CSRB1 :label "Memory full" :valuex 20c
      :checkbox CSRB2 :label "Block change" :valuex 12c
      :next-row T
      :checkbox CSRB3 :label "reserved" :valuex 12c
      :checkbox CSRB4 :label "event storage active" :valuex 20c :deaf T
      :checkbox CSRB5 :label "PASS" :valuex 12c :deaf T
      :next-row T
      :checkbox CSRB6 :label "REN" :valuex 12c :deaf T
      :checkbox CSRB7 :label "PAUSE" :valuex 20c :deaf T
      :checkbox CSRB8 :label "STOP" :valuex 12c :deaf T
      :next-row T
      :checkbox CSRB9 :label "FIFO empty" :valuex 12c :deaf T
      :checkbox CSRB10 :label "FIFO full" :valuex 20c :deaf T
      :next-row T
      :next-row T
      :message P2M2 :label "Control Register" :bold T :valuex 32c
      :next-row T
      :checkbox CCRB8 :label "Enable event storage" :valuex 20c
      :checkbox CCRB9 :label "Software reset" :valuex 14c
      :checkbox CCRB10 :label "FIFO reset" :valuex 12c
      :next-row T
      :checkbox CCRB11 :label "Abort event storage" :valuex 20c
      :checkbox CCRB13 :label "A32 enable" :valuex 14c


   :panel DISPLAY :layout B :padx 10 :after FOOTER
      :layout H; :row-gap 10; :col-gap 10
      :text MEMOFFSET :label "RAM Offset:" :w 8 :value "$x2VBmemOffset" :valuex 18c
      :button RDRAM :label "read"
      :optionbutton RDOPT  :strings [list "display data RAM" "display list RAM"] :value 0
      :checkbox INCR :label "increment RAM address" :value 0
      :button CURRENT :label "Dump using current addresses"  :command show_current
        bind [:path CURRENT] <ButtonPress-3>  format_a_block
      :next-row T
      :galley MEMDISPLAY :rows $memdisplaysize :cols 115 :font fixed

   :panel CLICK :layout B :padx 10 :pady 10 :after FOOTER
      :layout H; :row-gap 15; :col-gap 15
      :button REDISPLAY :label "Redisplay"
      :button EMPTY     :label "empty memory display"
      :button FORMAT    :label "format data RAM"  :command format_the_dump
      add-format-option
      :button DEFINE    :label "define registers"  :command define-x2vb-registers

   do-button-appends

   :show T

}

proc start args {
   global started
   if {$started == 0} then {make-frame} else {open-frame}
   set started 1
  define-x2vb-registers
   click-redisplay
}


#    following procs available for patching
proc add-format-option {} {
     global dumpopt
     :choice-stack FORMATOPT :label "format option" :strings [list 0 1 2] :value $dumpopt
}

proc do-button-appends {} {}
proc format_the_dump {} {}
proc define-x2vb-registers {} {}
