
#    common.tk.tcl

#    procedures to enable tk windows to be generated using commandmidas-save-window
#    format similar to that used with xv

set debug 0
set started 0

set framename ""
set frame [string toupper [Get_My_Name]]
set frametitle ""
set title ""
set icontitle ""
set iconname ""

if {![info exists env(EGsilent)]} {set env(EGsilent) 0}
if {![info exists env(FooterText)]} {set env(FooterText) ""}
if {![info exists env(ErrorText)]} {set env(ErrorText) ""}
if {![info exists env(WarningText)]} {set env(WarningText) ""}
if {![info exists env(ReportText)]} {set env(ReportText) ""}

set _xincadjust 0
set _yincadjust 0
if {[info exists env(tkxincadjust)]} then {set _xincadjust $env(tkxincadjust)}
if {[info exists env(tkyincadjust)]} then {set _yincadjust $env(tkyincadjust)}

#    various commonally used procedures

proc click arg {
   global debug WIDGET COMMAND FRAME VALUE ITEM
   if {$debug} then {insert-debug "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set FRAME $f
   set c [lindex $arg 1]
   set COMMAND $c
   set w [lindex $arg 2]
   set WIDGET $w
   set v [lindex $arg 3]
   set VALUE $v
   set i [lindex $arg 4]
   set ITEM $i
   if {$c==":quit"} then {return [click-quit $f]}

   set p "click-[lowercase $w]"
   if {[info procs $p] == "$p"} then {
      set argnum [llength [info args $p]]
      if {$argnum == 0} then {$p}
      if {$argnum == 1} then {$p "$v"}
      if {$argnum == 2} then {$p "$v" "$i"}
      if {$argnum > 2} then {insert-debug "click: $p expects too many args ($argnum)"}
   } else {
      set p "click-default"
      if {[info procs $p] == "$p"} then {
         set argnum [llength [info args $p]]
         if {$argnum == 0} then {$p}
         if {$argnum == 1} then {$p $w}
         if {$argnum == 2} then {$p $w "$v"}
         if {$argnum == 3} then {$p $w "$v" "$i"}
         if {$argnum > 3} then {insert-debug "click: $p expects too many args ($argnum)"}
      } else {
         insert-debug "click with argument \{$arg\}"
      }
   }
}

proc ::enable-event {win name} {
  global frame
#      insert-debug "::enable-event $win $name"
  switch $name {
     Map        {:click-Map $win}
     Unmap      {:click-UnMap $win}
     Visibility {:click-Visibility $win}
     Configure  {:click-Configure $win}
     Enter      {:click-Enter $win}
     default    {}
  }
}

#    procedures to handle Map and UnMap events

set WMapState -100

proc :click-UnMap args {
   global WMapState
   set WMapState 0
   set p "click-unmap"
   if {[info procs $p] == "$p"} then {$p $args; return}
}

proc :click-Map   args {
   global WMapState
   incr WMapState 1
   if {$WMapState <= 0} then {return}
   set p "click-map"
   if {[info procs $p] == "$p"} then {$p $args; return}
}

proc :click-Visibility args {
   set p "click-visibility"
   if {[info procs $p] == "$p"} then {$p $args; return}
}

proc :click-Configure args {
   set p "click-configure"
   if {[info procs $p] == "$p"} then {$p $args; return}
}

proc :click-Enter args {
   set p "click-enter"
   if {[info procs $p] == "$p"} then {$p $args; return}
}

proc start args {
   global started
   if {$started == 0} then {make-frame; EGbindings} else {open-frame}
   set started 1
   set p "click-redisplay"
   if {[info procs $p] == "$p"} then $p
}

proc click-quit args  {close-frame [first $args]}

proc frame-close win {
   catch {wm iconify [winfo toplevel $win]}
}

proc frame-quit win  {
   catch {wm iconify [winfo toplevel $win]}
   wm withdraw [winfo toplevel $win]
}

proc finish args {global started; destroy-frame; set started 0}

proc self op {
  global env framename frame frametitle title icontitle iconname helpname mytaskname
  switch $op {
     frame-name {
       if {$frame != ""} then {set name $frame} else {set name $framename}
       if {[string match .* $name] == 0} then {set name .[string tolower $name]}
       return $name
     }
     frame-title {
       if {$frametitle != ""} then {return $frametitle} else {return $title}
     }
     icon-title {
       if {$icontitle != ""} then {return $icontitle} else {return $iconname}
     }
     help-name {
       if {[info exists helpname]} then {return $helpname} \
          else {return $mytaskname}
     }
  }
}

set _footer_output 0
set _footer_width 100
set _footer_only 0

proc set-footer {args} {
   global _footer_width _footer_only _footer_output _footercharw
   global env

     if {[second $args] == ""} then {
       set f [self frame-name]
       set text [first $args]
     } else {
       set f [:path [first $args]]
       set text [second $args]
     }

     set env(FooterText) "$text"
     if {$env(EGsilent) >= 2} {puts stdout "[Get_My_Name]: set-footer: (silent) $env(FooterText)"; return}

     update
     if {[info commands $f.footer.footer] != ""} then {

        set w [winfo width $f]
        set _footer_width [expr $w / $_footercharw]
        set len [string length "$text"]

        if {$len <= $_footer_width} then {
           $f.footer.footer config -text "$text"
        } else {
           $f.footer.footer config -text "[string range $text 0 $_footer_width]"
           footer-report "$text\n\n($len:$_footer_width)"
        }
        update idletasks

        if {$_footer_output != 0} then {puts stdout "[Get_My_Name]: set-footer: $text"}
     } else {
        puts stdout "[Get_My_Name]: set-footer: $text"
     }
}

proc footer-report msg {
   global _footer_only
   if {$_footer_only != 0} then {return}

    set z [catch {midas_dialog [self frame-name].footerDialog "Footer for [self frame-title]" \
           "$msg" "" 0 "Ok" "Do not show dialog box in future"} m]

    if {$z != 0} then {
       puts stderr "midas-report: [self frame-name].footerDialog: $m"
       catch {destroy [self frame-name].footerDialog}
    } else {
       set _footer_only $m
    }
}

proc clear-footer {args} {
   global env

    if {$env(EGsilent) >= 2} {return}

     if {[first $args] == ""} then {
       set f [self frame-name]
     } else {
       set f [:path [first $args]]
     }
     if {[info commands $f.footer.footer] != ""} then {
       $f.footer.footer config -text ""
       update idletasks
     }
}

proc clear-busy {} {global env; if {$env(EGsilent) >= 2} {return}; :modify [self frame-name] :busy F}
proc set-busy   {} {global env; if {$env(EGsilent) >= 2} {return}; :modify [self frame-name] :busy T}
proc query-busy {} {:query [self frame-name] :busy}
proc frame-busy {} {global env; if {$env(EGsilent) >= 2} {return}; :modify [self frame-name] :busy T}
proc frame-idle {} {global env; if {$env(EGsilent) >= 2} {return}; :modify [self frame-name] :busy F}

proc :cursor {win c} {
    global env
    if {$env(os) == "Linux"} {return}     ;#  avoid busy in X server
    set z [catch {$win configure -cursor $c}]
# Use the default cursor if the requested one doesn't exist
    if {$z != 0} then {$win configure -cursor ""}
}

proc cleanup-frame {} {

   remove-eg-error-report self
   clear-footer
   frame-idle
}

proc destroy-frame {args} {

   catch {
            if {$args == ""} then {set f [self frame-name]} else {set f [first $args]}
            if {[string match .* $f] == 0} then {set f .$f}
            set f [string tolower $f]
            wm withdraw [winfo toplevel $f]
            destroy $f
   }
}

proc close-frame {args} {

            if {$args == ""} then {set f [self frame-name]} else {set f [first $args]}
            if {[string match .* $f] == 0} then {set f .$f}
            set f [string tolower $f]
            catch {wm iconify [winfo toplevel $f]}
            wm withdraw [winfo toplevel $f]
}

proc iconize-frame {args} {

            if {$args == ""} then {set f [self frame-name]} else {set f [first $args]}
            if {[string match .* $f] == 0} then {set f .$f}
            set f [string tolower $f]
            catch {wm iconify [winfo toplevel $f]}
}

proc :query_Mapped {args} {
   if {$args == ""} then {set f [self frame-name]} else {set f [first $args]}
   if {$f == "."} then {set f [self frame-name]}
   if {[string match .* $f] == 0} then {set f .$f}
   set f [string tolower $f]
   return [winfo ismapped $f]
}

proc open-frame {args} {
    global env

   if {$args == ""} then {set f [self frame-name]} else {set f [first $args]}
   if {$f == "."} then {set f [self frame-name]}
   if {[string match .* $f] == 0} then {set f .$f}
   set f [string tolower $f]

   set w [winfo toplevel [string tolower $f]]

#    wm withdraw $w
#
#    switch $env(os) {
#       Linux   {after 100}
#       default {update}
#    }

    wm deiconify $w
    raise $w
}

proc bring-to-front {f} {open-frame $f}

proc show-frame {args} {open-frame $args}

proc centre-frame {args} {
   global env

   if {$args == ""} then {set f [self frame-name]} else {set f [first $args]}
   if {$f == "."} then {set f [self frame-name]}
   if {[string match .* $f] == 0} then {set f .$f}
   set f [string tolower $f]

   set w [winfo toplevel [string tolower $f]]

    wm withdraw $w
    update

    set x [expr [winfo screenwidth $w]/2 - [winfo reqwidth $w]/2]
    set y [expr [winfo screenheight $w]/2 - [winfo reqheight $w]/2]

    # position the window on the screen.

    wm geom $w +$x+$y

    switch $env(os) {
       Linux   {after 100}
       default {update}
    }

    wm deiconify $w
}

proc position-frame {x y args} {
   global env

   if {$args == ""} then {set f [self frame-name]} else {set f [first $args]}
   if {$f == "."} then {set f [self frame-name]}
   if {[string match .* $f] == 0} then {set f .$f}
   set f [string tolower $f]

   set w [winfo toplevel [string tolower $f]]

    wm withdraw $w
    update

    # position the window on the screen.

    wm geom $w +$x+$y

    switch $env(os) {
       Linux   {after 100}
       default {update}
    }

    wm deiconify $w
}

proc :modify {name args} {
  global debug _classname _winname

#    :modify name options

  set name [lowercase $name]
  if {[string match .* $name] != 0} then {set name [string range $name 1 end]}
  set class $_classname($name)
  set win   $_winname($name)

  if {$debug} then {insert-debug ":modify  $name is type $class  args- $args"}

  :modify_$class $win $name $args

}

proc :query {name args} {
  global debug _classname _winname

#    :query name options

  set name [lowercase $name]
  if {[string match .* $name] != 0} then {set name [string range $name 1 end]}
  set class $_classname($name)
  set win   $_winname($name)

  if {$debug} then {insert-debug ":query  $name is type $class  args- $args"}

  :query_$class $win $name $args
}

proc :query_widget {name} {
  global  _classname _winname

  set name [lowercase $name]
   if {![info exists _classname($name)]} then {return 0}
   if {![info exists _winname($name)]} then {return 0}
  return [list $_classname($name) $_winname($name)]
}

set __coloursave1 ""; set __coloursave2 ""; set __textsave ""

proc xv arg {set f [self frame-name]; puts stdout "$f  xv: $arg"}

proc uppercase s {string toupper "$s"}
proc lowercase s {string tolower "$s"}

set frame_type "frame"

proc midas_icon {} {
  global env

  :frame MIDASICON :toplevel T :win-ctrl N :show-footer F
#  image create photo midas -file $env(ICONHOME)/midas.golden
  image create photo midas -format gif -file $env(ICONHOME)/midasicon.gif
# image create bitmap midas -background goldenrod -file $env(ICONHOME)/midas
  :canvas MIDASICON_C :w 64 :h 48
  [:path MIDASICON_C] create image 24 26  -image midas
  bind [:path MIDASICON_C] <Double-ButtonRelease-1> "wm deiconify ."
  :show T
}

proc midas_default_frame {} {
  global env

  :frame [self frame-name] :label Midas
# image create photo midas -file $env(ICONHOME)/midas.golden
  image create photo midas -format gif -file $env(ICONHOME)/midas.gif
# image create bitmap midas -background goldenrod -file $env(ICONHOME)/midas
  :canvas MIDASDEFAULT :w 64 :h 48
  [:path MIDASDEFAULT] create image 36 28  -image midas
  tk_focusFollowsMouse
}

proc :initialise {} {
  global env

#    procedure which can be called to initialise control variables
#    should be called if toplevel window is destroyed

  global _frame _panel _container _xposn _yposn _rowgap _colgap  _placed
  global _xinc _yinc
  global _charh _charw _footercharw
  global _chkbtsize _chcesize
  global _listch _listcw
  global __skipupdate 0
  global _autosize _hreq _wreq _information_font

:_reset

set _frame ""
set _panel ""
set _container ""
set _xposn 0
set _yposn 0
set _xinc 0
set _yinc 0
set _rowgap 0
set _colgap 0
set _charh 25
set _charw 7
set _footercharw 6
set _listch 17
set _listcw 7
set _chkbtsize 24
set _chcesize 24
set _placed 0
set __skipupdate 0
set _autosize 0
set _hreq 0
set _wreq 0
set _information_font variable

   image create bitmap inc -file $env(ICONHOME)/incr \
        -maskfile $env(ICONHOME)/incr
   image create bitmap dec -file $env(ICONHOME)/decr \
        -maskfile $env(ICONHOME)/decr

   image create bitmap lefter -file $env(ICONHOME)/lefter \
        -maskfile $env(ICONHOME)/lefter
   image create bitmap righter -file $env(ICONHOME)/righter \
        -maskfile $env(ICONHOME)/righter

   image create bitmap tickon -file $env(ICONHOME)/tick \
        -maskfile $env(ICONHOME)/tick
   image create bitmap tickoff -file $env(ICONHOME)/notick \
        -maskfile $env(ICONHOME)/notick

   image create bitmap Izoomin -file $env(ICONHOME)/expand \
        -maskfile $env(ICONHOME)/expand
   image create bitmap Izoomout -file $env(ICONHOME)/contract \
        -maskfile $env(ICONHOME)/contract
   image create bitmap Itimes2 -file $env(ICONHOME)/times2 \
        -maskfile $env(ICONHOME)/times2
   image create bitmap Idivide2 -file $env(ICONHOME)/divide2 \
        -maskfile $env(ICONHOME)/divide2
   image create bitmap Ileft -file $env(ICONHOME)/leftarrow \
        -maskfile $env(ICONHOME)/leftarrow
   image create bitmap Iright -file $env(ICONHOME)/rightarrow \
        -maskfile $env(ICONHOME)/rightarrow
   image create bitmap Iup -file $env(ICONHOME)/uparrow \
        -maskfile $env(ICONHOME)/uparrow
   image create bitmap Idown -file $env(ICONHOME)/downarrow \
        -maskfile $env(ICONHOME)/downarrow
   image create bitmap Isquare -file $env(ICONHOME)/square \
        -maskfile $env(ICONHOME)/square

   image create bitmap hourglass -file $env(ICONHOME)/hourglass \
        -maskfile $env(ICONHOME)/hourglass
   image create bitmap Iquit -file $env(ICONHOME)/quit \
        -maskfile $env(ICONHOME)/quit
   image create bitmap Iclose -file $env(ICONHOME)/close \
        -maskfile $env(ICONHOME)/close
}

set _tk_standard 0
proc :tk_standard {} {global _tk_standard; set _tk_standard 1}

set _default_autoreturn "T"

proc :_reset {} {
  global _layout _anchor _expand _fill _ipadx _ipady _padx _pady _pack
  global _tk_standard
  global _footer_position

set _pack ""
if {$_tk_standard == 1} then {
   set _layout "top"
   set _anchor "center"
   set _expand "yes"
   set _fill "none"
} else {
   set _layout "left"
   set _anchor "nw"
   set _expand "yes"
   set _fill "both"
}
set _footer_position left
set _ipadx 0
set _ipady 0
set _padx 0
set _pady 0
}

proc :set {item name} {
  set v _$item
  global $v
  set $v $name
}

proc :path  widget {
  global _winname

  if {[string match .* $widget] == 0} then {set name $widget} else \
     {set name [string range $widget 1 end]}

  return $_winname([lowercase $name])
}

proc :widget path {
   global _winname _classname
   set p $path
   set names [array names _winname]
#    first type full path name
   foreach name $names {
      if {$_winname($name) == $p} then {return [uppercase $name]}
   }
#    no match - so try again dropping final component of path
   set l [last [split $p .]]
   set p [string range $p 0 [expr [string length $p] - [string length $l] - 2]]
   foreach name $names {
      if {$_winname($name) == $p} then {return [uppercase $name]}
   }
#    no match - so try again dropping final component of path
   set l [last [split $p .]]
   set p [string range $p 0 [expr [string length $p] - [string length $l] - 2]]
   foreach name $names {
      if {$_winname($name) == $p} then {return [uppercase $name]}
   }
   return "UNKNOWN"
}

proc :show {opt} {
  global _frame

#    :show T/F

  if {$opt == "F"} then {
    if {[winfo exists $_frame]} then {wm iconify [winfo toplevel $_frame]} else {wm iconify .}
  } else {
    if {[winfo exists $_frame]} then {centre-frame [winfo toplevel $_frame]} else {centre-frame}
    if {[winfo exists $_frame]} then {wm deiconify [winfo toplevel $_frame]} else {wm deiconify .}
    tk_focusFollowsMouse
  }
}

proc :autosize {opt} {
  global debug
  global _frame _panel _container _hreq _wreq
  global _xincadjust _yincadjust

  if {$opt == "T"} then {:modify $_panel :h [expr $_hreq - $_yincadjust] :w [expr $_wreq - $_xincadjust]}
}

proc :layout {v args} {
  global _layout _placed _anchor

   set _placed 0

   switch $v {
     L  {set _layout "left"}
     H  {set _layout "left"; set _placed 1}
     T  {set _layout "top"}
     V  {set _layout "top"; set _placed 1}
     R  {set _layout "right"}
     B  {set _layout "bottom"}
     default {puts stderr ":layout - unknown option $v"}
   }

  set i 0
  while {$i < [llength $args]} {
    set p [lindex $args $i]
    set v [lindex $args [expr $i + 1]]
    switch $p {

      :pack {
        switch $v {
          T {set _placed 0}
          F {set _placed 1}
          default {puts stderr ":layout :pack - unknown option $v"}
        }
      }
      :place {
        switch $v {
          T {set _placed 1}
          F {set _placed 0}
          default {puts stderr ":layout - unknown option $v"}
        }
      }
      :anchor {set _anchor [string tolower $v]}

      default {puts stderr ":layout - unknown arg $p"}
    }
    set i [expr $i + 2]
  }
}


proc :enable {w args} {
#    enable "special" events

  set WIN $w
  if {[string match .* $w] != 0} then {set WIN [uppercase [string range $w 1 end]]}

  set i 0
  while {$i < [llength $args]} {
    set p [lindex $args $i]

    bind [:path $WIN] <$p> ":enable-event $WIN $p"

    set i [expr $i + 2]
  }
}

proc :enable-event {win name} {
  global frame
#      insert-debug ":enable-event $win $name"
  click [list $frame :$name $win]
}

proc :row-gap v {
  global _rowgap _placed

#    :row-gap ?

  set _rowgap $v
  set _placed 1
}

proc :col-gap v {
  global _colgap _placed

#    :col-gap ?

  set _colgap $v
  set _placed 1
}

proc :next-row args {
  global _xposn _yposn _rowgap _yinc
  global __skipupdate _placed

  set _yposn [expr $_yposn + $_yinc + $_rowgap]
  set _xposn  0

  set __skipupdate 1
  set _placed 1
}

proc :next-col args {
  global _xposn _yposn _colgap _xinc
  global __skipupdate _placed

  set _xposn [expr $_xposn + $_xinc + $_colgap]
  set _yposn 0

  set __skipupdate 1
  set _placed 1
}

proc :hskip vl {
  global _xposn _yposn _placed

   set _xposn [expr $_xposn + $vl]

#   set __skipupdate 1
   set _placed 1
}

proc :vskip vl {
  global _xposn _yposn _placed

   set _yposn [expr $_yposn + $vl]

#   set __skipupdate 1
   set _placed 1
}

proc :_post-updateposn {placed win xp yp} {
  global debug
  global _xposn _yposn _xinc _yinc
  global _xincadjust _yincadjust
  global _autosize _hreq _wreq
  global _panel _container

#   layout left is H;  layout right is V

    set _xposn $xp
    set _yposn $yp
    update
    set _xinc [expr [winfo width $win]  + $_xincadjust]
    set _yinc [expr [winfo height $win] + $_yincadjust]

    if {[expr $_xposn + $_xinc] > $_wreq} then {set _wreq [expr $_xposn + $_xinc]}
    if {[expr $_yposn + $_yinc] > $_hreq} then {set _hreq [expr $_yposn + $_yinc]}

    if {$placed == 1 && $_autosize == 0 && $_panel != ""} then {
       if {$debug >=2} then {insert-debug ":modify :h [expr $_hreq - $_yincadjust] :w [expr $_wreq - $_xincadjust]"}
       :modify $_panel :h [expr $_hreq - $_yincadjust] :w [expr $_wreq - $_xincadjust]
    }
}


proc :_pre-updateposn {} {
  global _layout _xposn _yposn _colgap _rowgap _xinc _yinc
  global __skipupdate

#   layout left is H;  layout right is V

  if {$__skipupdate > 0} then {set __skipupdate 0; return}

  if {$_layout == "left"} then {
    set _xposn [expr $_xposn + $_xinc + $_colgap]
  } else {
    set _yposn [expr $_yposn + $_yinc + $_rowgap]
  }
}

proc :size {win args} {
  global debug

#    options -   :h widget :w widget

  if {$debug} then {insert-debug ":size called with  $win $args"}

  set i 0
  while {$i < [llength $args]} {
    set p [lindex $args $i]
    set w [lindex $args [expr $i + 1]]
    switch $p {

      :h  {
   :modify $win :h [expr [winfo y [:path $w]] + [winfo height [:path $w]]]
      }

      :w  {
   :modify $win :w [expr [winfo x [:path $w]] + [winfo width [:path $w]]]
      }

      default {puts stderr ":size - unknown arg $p"}
    }
    set i [expr $i + 2]
  }

}

proc :_position-widget {placed win xp yp} {
  global debug
  global _layout _anchor _expand _fill _ipadx _ipady _padx _pady _pack

#    if no :x or :y supplied use the pack command

#  insert-debug "$placed $win -side $_layout -anchor $_anchor -expand $_expand -fill $_fill \
#      -ipadx $_ipadx -ipady $_ipady -padx $_padx -pady $_pady $_pack"


  if {$placed == 0} then {
    eval pack configure $win -side $_layout -anchor $_anchor -expand $_expand -fill $_fill \
      -ipadx $_ipadx -ipady $_ipady -padx $_padx -pady $_pady $_pack
  }
  if {$placed == 1} then {
    place $win -x $xp -y $yp  -anchor $_anchor
    if {$debug >= 2} then {puts stderr "place $win -x $xp -y $yp  -anchor $_anchor"}
  }
  set _pack ""
}

proc :colour args {
  global env

  set mode   [first $args]
  set widget [second $args]
  set option [third $args]
  set value  [fourth $args]

  global _colour_[set widget]_[set option]
  if {$value == ""} then {
     unset _colour_[set widget]_[set option]
  } else {
     set _colour_[set widget]_[set option] $value
  }

  if {$mode == "global"} then {
     set env(tk$widget$option) [set _colour_[set widget]_[set option]]
     set env(USERCOLORS) 1
  }
}

set __options [list activebackground activeforeground background disabledforeground foreground \
               hightlightbackground highlightcolor insertbackground selectcolor \
               selectbackground selectforeground troughcolor tagcolour gatecolour polycolour pointercolour boxcolour activeboxcolour textcolour activetextcolour]

proc :_set-color {window widget} {
  global debug env __options

  if {[winfo depth .] <= 1} then {return}
  if {[info exists env(tkwindepth)]} then {if {$env(tkwindepth) <= 1} then {return}}
  if {$env(USERCOLORS) == 0} then {return}

#    if the second argument is not a widget name then it is a colour name

  if {"[info commands $widget]" == "$widget"} then {
#    widget is indeed a widget name

       set cmd ""
       foreach option $__options {
         global _colour_[set widget]_[set option]
         if { [info exists  _colour_[set widget]_[set option]]} then {
           set colour [set _colour_[set widget]_[set option]]
           if {[lindex [$window configure -$option] 4] != $colour} then {
             append cmd " -$option $colour"
           }
         } else {
            if { [info exists env(tk$widget$option)]} then {
              if {[lindex [$window configure -$option] 4] != $env(tk$widget$option)} then {
                append cmd " -$option $env(tk$widget$option)"
              }
            }
         }
       }
       if {$debug == 1} then {puts stdout "$window configure $cmd"}
       if {$cmd != ""} then          {eval $window configure $cmd}
  } else {
       eval $window configure $widget
  }
}

#    define footer and dialog as procedures so that they look like widgets
proc footer {} {}
proc topper {} {}
proc dialog {} {}

proc :font args {
  global env

  set mode   [first $args]
  set widget [second $args]
  set weight [third $args]

  set fndry adobe
  set fmly  helvetica
  set pxlsz 12
  set ptsz  120
  if {$weight == "b"} then {set wght bold} else {set wght medium}
  set slant r
  set swdth normal
  set rgstry iso8859

  set i 3
  while {$i < [llength $args]} {
    set p [lindex $args $i]
    set v [lindex $args [expr $i + 1]]
    switch $p {

      :fndry  {set fndry $v}
      :fmly   {set fmly "$v"}
      :wght   {set wght $v}
      :pxlsz  {set pxlsz $v}
      :ptSz   {set ptsz $v}
      :ptsz   {set ptsz $v}
      :slant  {set slant $v}
      :sWdth  {set swdth $v}
      :swdth  {set swdth $v}
      :rgstry {set rgstry $v}

      default {puts stderr ":panel - unknown arg $p"}
    }
    set i [expr $i + 2]
  }

  global _font_[set widget]_[set weight]
  set _font_[set widget]_[set weight]  \
      "-$fndry-$fmly-$wght-$slant-$swdth-*-$pxlsz-$ptsz-*-*-*-*-$rgstry-*"

  if {$mode == "global"} then {set env(tkfont$widget$weight) $_font_[set widget]_[set weight]}
}

proc :_set-font {window widget weight size} {
   global env

#    if the second argument is not a widget name then it is a font string
#    weight = 0   bold=F;    weight = 1   bold=T;   weight = -1  undefined
#    size is the font size require; has values S (small); L (large) and default N (normal)

 set w $widget
 if {$w == "Vtext"} then {set w "text"}
 if {$w == "Ftext"} then {set w "text"}
 if {$w == "Vlistbox"} then {set w "listbox"}
 if {$w == "Flistbox"} then {set w "listbox"}

  if {[info commands $w] == $w} then {
#    widget is indeed a widget name
     if {$weight == 1} then {set option "b"} else {set option "n"}

     global _font_[set widget]_[set option]
     if { [info exists  _font_[set widget]_[set option]] } then {
       set font "[set _font_[set widget]_[set option]]"
     } else {
       if { [info exists env(tkfont$widget$option)] } then {
          set font "$env(tkfont$widget$option)"
       } else {
          set font ""
       }
     }
  } else {
     set font $widget
  }
  set z [catch {$window configure -font $font} m]
  if {$z != 0} then {puts stderr "$m"}
}

proc :_deaf {widget class} {
   set list [bind $class]
   foreach item $list {
      bind $widget $item "break"
   }
   :_globalbindings $widget
}

#    support for resize where not automatic

#    this procedure should be called at the end of the creation of
#    a frame containing a galley (and after the frame is opened)
#    to establish initial sizes of things
#    the galley would have been created with something like
#              :galley $W :rows $rows :cols $cols

proc :galley_size {W rows cols} {
   upvar #0 _[set W]galley _galley

   set _galley(W) $W
   set _galley(rows) $rows
   set _galley(cols) $cols
   update
   set _galley(frameheight) [winfo height [winfo toplevel [self frame-name]]]
   set _galley(framewidth)  [winfo width  [winfo toplevel [self frame-name]]]

   set _galley(rowsize) 0
   set _galley(rowoverhead) 0
   if {$rows > 0} then {
      while {$_galley(rowsize) == 0} {
         update
         set lh [winfo height [:path $W]]
         set _galley(rowsize) [expr $lh/$rows]
      }

      while {$_galley(rowoverhead) <= 0} {
         update
         set wh [winfo height [winfo toplevel [self frame-name]]]
         set _galley(rowoverhead) [expr $wh - $lh]
      }
   }
   set _galley(colsize) 0
   set _galley(coloverhead) 0
   if {$cols > 0} then {
      while {$_galley(colsize) == 0} {
         update
         set lw [winfo width [:path $W]]
         set _galley(colsize) [expr $lw/$cols]
      }

      while {$_galley(coloverhead) <= 0} {
         update
         set ww [winfo width [winfo toplevel [self frame-name]]]
         set _galley(coloverhead) [expr $ww - $lw]
      }
   }
#    can now handle galley resize
   set _galley(busy) 0
}

#    must have called :galley_size first
#    call whenever you would like to check if galley can be
#    resized to fill the window

proc :do_galley_resize {W} {
  upvar #0 _[set W]galley _galley

#insert-debug ":do_galley_resize $W - called"

  if {![info exists _galley(busy)]} then {return 0}
  if {$_galley(busy) != 0} then {return 0}

  if {![winfo exists [:path $W].text]} then {:galley_resize $W; return 0}

#insert-debug ":do_galley_resize $W - checking"

  set _galley(busy) 1
  update
  set h [winfo height [winfo toplevel [self frame-name]]]
  set w [winfo width  [winfo toplevel [self frame-name]]]
  if {$h == $_galley(frameheight) && $w == $_galley(framewidth)} then {set _galley(busy) 0; return 0}

#insert-debug ":do_galley_resize $W - resize h=$h w=$w"

  if {$_galley(rows) > 0} then {
     set size [expr [expr $h - $_galley(rowoverhead)] / $_galley(rowsize)]
     set _galley(rows) $size
  }
  if {$_galley(cols) > 0} then {
     set size [expr [expr $w - $_galley(coloverhead)] / $_galley(colsize)]
     set _galley(cols) $size
  }
  :modify $_galley(W)  :rows $_galley(rows)  :cols $_galley(cols)

#insert-debug ":modify $_galley(W)  :rows $_galley(rows)  :cols $_galley(cols)"

  update
  set h [winfo height [winfo toplevel [self frame-name]]]
  set w [winfo width  [winfo toplevel [self frame-name]]]

#insert-debug ":do_galley_resize $W - size h=$h w=$w"

  set _galley(frameheight) $h
  set _galley(framewidth)  $w
  set _galley(busy) 0
  return 1
}

#    new versions  -  much simpler but seem to work better !

proc :galley_size {W rows cols} {
   upvar #0 _[set W]galley _galley

   set _galley(W) $W
   update
   set _galley(frameheight) [winfo height [winfo toplevel [self frame-name]]]
   set _galley(framewidth)  [winfo width  [winfo toplevel [self frame-name]]]

#    can now handle galley resize
   set _galley(busy) 0
}

proc :do_galley_resize {W} {
  upvar #0 _[set W]galley _galley

  if {![info exists _galley(busy)]} then {return 0}
  if {$_galley(busy) != 0} then {return 0}

  set _galley(busy) 1
  update
  set h [winfo height [winfo toplevel [self frame-name]]]
  if {$h != $_galley(frameheight)} then {
      :modify $_galley(W)  :rows 50     ;# seems like setting to a large value is good once the frame has been sized
  }
  set _galley(busy) 0
}


proc :galley_resize {W} {after 1000 :do_galley_resize $W}

#    various procedures for support of interactive help

set __helpexists 0
set _helpactive 1
set _helpitem "unknown"
set _helpwindow "unknown"
set _suppresshelp 0

proc :_help {itemname widget class} {
   global _helpactive
   if {$_helpactive == 0} then {return}
   bind $widget <Enter>            ":_help_action1 $itemname %W"
   bind $widget <Leave>            ":_help_action0"
}

#    called on leaving a widget
proc :_help_action0 {} {
# insert-debug ":_help_action0 called"
      global __helpexists
      if {$__helpexists == 1} then {clear-footer; set __helpexists 0}
}

#    called on entering a widget
proc :_help_action1 {itemname W} {
# insert-debug ":_help_action1 called with $itemname $W"
      global __helpexists _helpitem _suppresshelp _helpwindow
      upvar #0 :help help
      set _helpitem [uppercase $itemname]
      set _helpwindow $W
      if {$_suppresshelp != 0} then {return}
      if {[info exists help($itemname)]} then {
         set text [first [split $help($itemname) \n]]
         set f [self frame-name]
         $f.footer.footer config -text "$text"
         set __helpexists 1
         return
      }
      if {[info exists help(DEFAULTHELP)]} then {
         set text [first [split $help(DEFAULTHELP) \n]]
         set f [self frame-name]
         $f.footer.footer config -text "$text"
         set __helpexists 1
         return
      }
}

#    called by button F1
proc :_help_action {opt path} {
#   opt = 0 is just F1;   opt = 1 is shift+F1
#  global tkPriv
# insert-debug ":_help_action called with $opt $path $tkPriv(window)"
  global _helpitem
  upvar #0 :help help
  if {$path == "."} then {
     set p [split [self frame-name] .]
  } else {
     set p [split $path .]
  }

  set taskname [self help-name]

  switch $opt {
  1  {
        if {[llength $p] <= 3} then {
#   path is a frame and a panel or just a frame
           set name [uppercase $taskname]
           set f [inform help "help-on-task" $taskname]
        } else {
           set name $_helpitem
           if {$taskname == "bf"} then {
              global HELP
              if {[info exists HELP($name)]} then {set taskname $HELP($name)}
           }
           set f [inform help "help-on-widget" $taskname $name]
        }
        if {$f == 0} then {
           if {[info exists help($name)]} then {midas-help $name "" $help($name); return}
           if {[info exists help(DEFAULTHELP)]} then {midas-help $name "" $help(DEFAULTHELP); return}
        }
     }
  0  {
        if {[llength $p] <= 3} then {
#   path is a frame and a panel or just a frame
           set name $taskname
        } else {
           set name $_helpitem
        }
        if {[info exists help($name)]} then {
           midas-help $name "" $help($name)
           return
        }
        if {[info exists help(DEFAULTHELP)]} then {
           midas-help $name "" $help(DEFAULTHELP)
           return
        }
        if {[llength $p] <= 3} then {
           inform help "help-on-task" $taskname
        } else {
           inform help "help-on-widget" $taskname $_helpitem
        }
     }
  }
}

proc :help {widget text} {
   upvar #0 :help help
   set help($widget) "$text\n"
}

proc :_help_widget widget {
   upvar #0 :help help
   return $help($widget)
}

proc click-help {} {
      set taskname [self help-name]
      if {[string range $taskname 0 0] == "."} then {set taskname [string range $taskname 1 end]}
      inform help "help-on-task" $taskname
}

proc :load-help {} {
   global env
   upvar #0 :help help
   set taskname [self help-name]
   if {[string range $taskname 0 0] == "."} then {set taskname [string range $taskname 1 end]}

   if {$env(platform) == "unix"} then {set J ":"} else {set J ";"}

   set path [split $env(TCLPATH) $J]

   foreach dir $path {
        set file [file join $dir $taskname.txt]
        if { [file exists $file] } {

           set f [open $file]
           set text [split [read $f] \n]
           close $f

           set widget "DEFAULT"
           set help($widget) ""
           foreach line $text {
              set l [first [split $line]]
              if {$l == "+widget"} then {
                 set widget [second [split $line]]
                 set help($widget) ""
              } else {
                 append help($widget) $line\n
              }
           }
        }
   }
}

# ---common error notices

set tkskiperror 0

if {[info tclversion] == 7.4} then {

proc tkerror err {
    global errorInfo tkerrorInfo tkskiperror

    # Be sure to release any grabs that might be present on the
    # screen, since they could make it impossible for the user
    # to dismiss the error dialog box.

    #  look for skip flag set by proc display-midas-error-report et al
    #    which have already made a call to midas_dialog
    #  if set return  after unsetting flag

    if {$tkskiperror == 1} then {set tkskiperror 0; return}
    set tkerrorInfo $errorInfo

    if {[grab current .] != ""} then {grab release [grab current .]}

    catch {:modify [self frame-name] :busy T}
    set z [catch {midas_dialog [self frame-name].tkerrorDialog "Error Detected in Tcl Script" \
            "Error: $err" "error" 0 "continue" "Stack Trace"} button]
    catch {:modify [self frame-name] :busy F}

    if {$z != 0} then {
       puts stderr "midas_dialog: [self frame-name].tkerrorDialog: $button"
       destroy [self frame-name].tkerrorDialog
       return
    }

    if {$button != 0} then {tkStackTrace}
}

} else {

proc bgerror err {
    global errorInfo tkerrorInfo tkskiperror

    # Be sure to release any grabs that might be present on the
    # screen, since they could make it impossible for the user
    # to dismiss the error dialog box.

    #  look for skip flag set by proc display-midas-error-report et al
    #    which have already made a call to midas_dialog
    #  if set return  after unsetting flag

    if {$tkskiperror == 1} then {set tkskiperror 0; return}
    set tkerrorInfo $errorInfo

    if {[grab current .] != ""} then {grab release [grab current .]}

    catch {:modify [self frame-name] :busy T}
    set z [catch {midas_dialog [self frame-name].tkerrorDialog "Error Detected in Tcl Script" \
            "Task '[Get_My_Name]' detected error in Tcl script\n\n$err" "error" 0 "continue" "Stack Trace"} button]
    catch {:modify [self frame-name] :busy F}

    if {$z != 0} then {
       puts stderr "midas_dialog: [self frame-name].tkerrorDialog: $button"
       destroy [self frame-name].tkerrorDialog
       return
    }

    if {$button != 0} then {tkStackTrace}
}

}

proc display-eg-error-report {p e args} {set m ""; foreach item $args {append m " $item"}; display-midas-error-report $p $e $m}

proc remove-eg-error-report args {remove-midas-error-report}

proc display-error-report {p args} {set m ""; foreach item $args {append m " $item"}; display-midas-error-report $p "" $m}
proc remove-error-report args {remove-midas-error-report}

proc display-midas-error-report {p e args} {
   global tkerrorInfo tkskiperror
   global env

   set tkskiperror 1
   set tkerrorInfo "Called from display-midas-error-report"

   if {$env(EGsilent) >= 1} {
       if {[grab current .] != ""} then {grab release [grab current .]}
   }

   set errmsg ""
   foreach item $args {append errmsg " $item"}

   if {$e == ""} then {
     set m "$errmsg"
   } else {
     if {[llength $e] == 1} {set m1 "Midas error $e"; set m2 [eg "(sys :error $e ?message)"]} else {set m1 "Midas error"; set m2 $e}
     set m "$m1\n$m2\n$errmsg"
   }

     set env(ErrorText) "$m"
     if {$env(EGsilent) >= 1} {puts stdout "[Get_My_Name]: error-report: (silent) $env(ErrorText)"; return}   

   catch {:modify [self frame-name] :busy T}
   set z [catch {midas_dialog [self frame-name].midasDialog "Error Report for [self frame-title]" \
           "$m" "warning" 0 "continue" "Stack Trace"} button]
   catch {:modify [self frame-name] :busy F}

    if {$z != 0} then {
       puts stderr "display-midas-error-report: [self frame-name].midasDialog: $button"
       puts stderr "display-midas-error-report: $m"
       catch {destroy [self frame-name].midasDialog}
       return "$m"
    }

   if {$button != 0} then {tkStackTrace}

   return "$m"
}

proc remove-midas-error-report args {}

proc tkStackTrace {} {
   global tkerrorInfo

#    generate Stack Trace window

    set w [self frame-name].stacktrace
    catch {destroy $w}
    toplevel $w -class ErrorTrace
    wm minsize $w 1 1
    wm title $w "Stack Trace for Error"
    wm iconname $w "Stack Trace"
#    bind $w <Visibility>  "raise $w"
    button $w.ok -text OK -command "destroy $w"
    :_set-font $w.ok button 1 L
    text $w.text -relief sunken -bd 2 -yscrollcommand "$w.scroll set" \
            -setgrid true -width 60 -height 20
    :_set-font $w.text Vtext 0 N
    scrollbar $w.scroll -relief sunken -command "$w.text yview"
    pack $w.ok -side bottom -padx 3m -pady 2m
    pack $w.scroll -side right -fill y
    pack $w.text -side left -expand yes -fill both
    $w.text insert 0.0 $tkerrorInfo
    $w.text mark set insert 0.0

    # Center the window on the screen.

    __centre-frame $w
}

set dialog_wraplength 6i
set dialog_buttonlayout left

proc midas_dialog {w title text bitmap default args} {
    global tkPriv env
    global dialog_wraplength
    upvar #0 dialog_buttonlayout side

    # -1.  if called when the parent window is not mapped then seen that
    #      CDE does not map the dialog window and so the user does not acknowledge
    #      Real dumb!!

    set wmflag1 0
    set wmflag2 0
    if {[wm state .] != "normal"} then {set wmflag1 1}
    set parent [join [lreplace [split $w .] end end] .]
    if {[winfo exists $parent]} then {
       if {[wm state [winfo toplevel $parent]] != "normal"} then {
          set wmflag2 1
          wm deiconify [winfo toplevel $parent]
       }
    } else {
       set w .midasDialog
    }

    # 1. Create the top-level window and divide it into top
    # and bottom parts.

    catch {destroy $w}
    toplevel $w -class Dialog
    wm title $w $title
    wm iconname $w Dialog
    wm protocol $w WM_DELETE_WINDOW { }
    wm transient $w [winfo toplevel [winfo parent $w]]
#    bind $w <Visibility>  "raise $w"
#    bind $w <Unmap>       "wm deiconify ."
    frame $w.top -relief raised -bd 1
    pack $w.top -side top -fill both
    frame $w.bot -relief raised -bd 1
    pack $w.bot -side bottom -fill both

    # 2. Fill the top part with bitmap and message.

    label $w.msg -wraplength $dialog_wraplength -justify left -text $text
    :_set-font $w.msg dialog 0 L
    pack $w.msg -in $w.top -side right -expand 1 -fill both -padx 3m -pady 3m
    if {$bitmap != ""} {
        label $w.bitmap -bitmap $bitmap
        pack $w.bitmap -in $w.top -side left -padx 3m -pady 3m
    }

    # 3. Create a row of buttons at the bottom of the dialog.

    set i 0
    foreach but $args {
        button $w.button$i -text $but -command "set tkPriv(button) $i"
        :_set-font $w.button$i button 1 L
        if {$i == $default} {
            frame $w.default -relief sunken -bd 1
            raise $w.button$i $w.default
            pack $w.default -in $w.bot -side $side -expand 1 -padx 3m -pady 2m
            pack $w.button$i -in $w.default -padx 2m -pady 2m
            bind $w <Return> "$w.button$i flash; set tkPriv(button) $i"
        } else {
            pack $w.button$i -in $w.bot -side $side -expand 1 \
                    -padx 3m -pady 2m
        }
        incr i
    }

    # 4. centre the window on the screen

    __centre-frame $w

    # 5. Set a grab and claim the focus too.

    set oldFocus [focus]
    catch {grab $w}
    tkwait visibility $w
    if {$default >= 0} {
        focus $w.button$default
    } else {
        focus $w
    }

    # 6. Wait for the user to respond, then restore the focus and
    # return the index of the selected button.  Restore the focus
    # before deleting the window, since otherwise the window manager
    # may take the focus away so we can't redirect it.
    tkwait variable tkPriv(button)
    catch {focus $oldFocus}
    destroy $w

    #   if we initially opened the parent return to original state

    if {$wmflag2 == 1} then {wm iconify [winfo toplevel $parent]}
    if {$wmflag1 == 1} then {wm withdraw .}
    return $tkPriv(button)
}

proc __centre-frame w {
    # Withdraw the window, then update all the geometry information
    # so we know how big it wants to be, then center the window in the
    # display and de-iconify it.

    wm withdraw $w
    update idletasks

    set x [expr [winfo screenwidth $w]/2 - [winfo reqwidth $w]/2 \
            - [winfo vrootx [winfo parent $w]]]
    set y [expr [winfo screenheight $w]/2 - [winfo reqheight $w]/2 \
            - [winfo vrooty [winfo parent $w]]]

    # position the window on the screen.

    wm geom $w +$x+$y
    wm deiconify $w
}

#
# midas_dialog:
#
# This procedure displays a dialog box, waits for a button in the dialog
# to be invoked, then returns the index of the selected button.  If the
# dialog somehow gets destroyed, -1 is returned.
#
# Arguments:
# w -        Window to use for dialog top-level.
# title -    Title to display in dialog's decorative frame.
# text -    Message to display in dialog.
# bitmap -    Bitmap to display in dialog (empty string means none).
# default -    Index of button that is to display the default ring
#        (-1 means none).
# args -    One or more strings to display in buttons across the
#        bottom of the dialog box.

proc midas_dialog {w title text bitmap default args} {
    global env tkPriv tcl_platform
    upvar #0 dialog_wraplength wraplength
    upvar #0 dialog_buttonlayout side

    # Check that $default was properly given
if {$env(TCLVERSION) >= 8.3} {
    if {[string is int $default]} {
    if {$default >= [llength $args]} {
        return -code error "default button index greater than number of\
            buttons specified for tk_dialog"
    }
    } elseif {[string equal {} $default]} {
    set default -1
    } else {
    set default [lsearch -exact $args $default]
    }
}

    # 1. Create the top-level window and divide it into top
    # and bottom parts.

    catch {destroy $w}
    toplevel $w -class Dialog
    wm title $w $title
    wm iconname $w Dialog
    wm protocol $w WM_DELETE_WINDOW { }

    # Dialog boxes should be transient with respect to their parent,
    # so that they will always stay on top of their parent window.  However,
    # some window managers will create the window as withdrawn if the parent
    # window is withdrawn or iconified.  Combined with the grab we put on the
    # window, this can hang the entire application.  Therefore we only make
    # the dialog transient if the parent is viewable.
    #
    if { [winfo viewable [winfo toplevel [winfo parent $w]]] } {
    wm transient $w [winfo toplevel [winfo parent $w]]
    }

#      Ensure that the dialog cannot be overlayed and thus hidden from the user
#      This seems to cause problems with Linux X client

#    bind $w <Visibility> "raise $w"

    frame $w.bot
    frame $w.top
if {$env(TCLVERSION) >= 8.3} {
    if {[string equal $tcl_platform(platform) "unix"]} {
    $w.bot configure -relief raised -bd 1
    $w.top configure -relief raised -bd 1
    }
}
    pack $w.bot -side bottom -fill both
    pack $w.top -side top -fill both -expand 1

    # 2. Fill the top part with bitmap and message (use the option
    # database for -wraplength and -font so that they can be
    # overridden by the caller).

    option add *Dialog.msg.wrapLength 3i widgetDefault
    option add *Dialog.msg.font {Times 12} widgetDefault

    label $w.msg -wraplength $wraplength -justify left -text $text
    :_set-font $w.msg dialog 0 L
    pack $w.msg -in $w.top -side right -expand 1 -fill both -padx 3m -pady 3m
    if {[string compare $bitmap ""]} {
    label $w.bitmap -bitmap $bitmap
    pack $w.bitmap -in $w.top -side left -padx 3m -pady 3m
    }

    # 3. Create a row of buttons at the bottom of the dialog.

    set i 0
    foreach but $args {
    button $w.button$i -text $but -command [list set tkPriv(button) $i]
        :_set-font $w.msg dialog 0 L
    if {$i == $default} {
        $w.button$i configure -default active
    } else {
        $w.button$i configure -default normal
    }
    grid $w.button$i -in $w.bot -column $i -row 0 -sticky ew -padx 10
    grid columnconfigure $w.bot $i
    incr i
    }

    # 4. Create a binding for <Return> on the dialog if there is a
    # default button.

    if {$default >= 0} {
    bind $w <Return> "
    [list $w.button$default] configure -state active -relief sunken
    update idletasks
    after 100
    set tkPriv(button) $default
    "
    }

    # 5. Create a <Destroy> binding for the window that sets the
    # button variable to -1;  this is needed in case something happens
    # that destroys the window, such as its parent window being destroyed.

    bind $w <Destroy> {set tkPriv(button) -1}

    # 6. Withdraw the window, then update all the geometry information
    # so we know how big it wants to be, then center the window in the
    # display and de-iconify it.

    __centre-frame $w

    # 7. Set a grab and claim the focus too.

    set oldFocus [focus]
    set oldGrab [grab current $w]
    if {[string compare $oldGrab ""]} {
    set grabStatus [grab status $oldGrab]
    }
    grab $w
    if {$default >= 0} {
    focus $w.button$default
    } else {
    focus $w
    }

    # 8. Wait for the user to respond, then restore the focus and
    # return the index of the selected button.  Restore the focus
    # before deleting the window, since otherwise the window manager
    # may take the focus away so we can't redirect it.  Finally,
    # restore any grab that was in effect.
    tkwait variable tkPriv(button)
    catch {focus $oldFocus}
    catch {
    # It's possible that the window has already been destroyed,
    # hence this "catch".  Delete the Destroy handler so that
    # tkPriv(button) doesn't get reset by it.
    bind $w <Destroy> {}
    destroy $w
    }
    if {[string compare $oldGrab ""]} {
      if {[string compare $grabStatus "global"]} {
        grab $oldGrab
      } else {
          grab -global $oldGrab
    }
    }
    return $tkPriv(button)
}


proc midas-warning msg {
   global env

     set env(WarningText) "$msg"
     if {$env(EGsilent) >= 1} {puts stdout "[Get_My_Name]: warning: (silent) $env(WarningText)"; return}

    set z [catch {midas_dialog [self frame-name].warningDialog "Warning for [self frame-title]" \
           "$msg" "warning" 0 "continue"} m]

    if {$z != 0} then {
       puts stderr "midas-warning: [self frame-name].warningDialog: $m"
       catch {destroy [self frame-name].warningDialog}
    }
}

proc midas-report msg {
   global env

     set env(ReportText) "$msg"
     if {$env(EGsilent) >= 1} {puts stdout "[Get_My_Name]: report: (silent) $env(ReportText)"; return}

    set z [catch {midas_dialog [self frame-name].reportDialog "Report for [self frame-title]" \
           "$msg" "" 0 "Ok"} m]

    if {$z != 0} then {
       puts stderr "midas-report: [self frame-name].reportDialog: $m"
       catch {destroy [self frame-name].reportDialog}
    }
}

proc midas-confirmation {msg args} {

    if {[llength $args] == 2} then {
       set m2 "[lindex $args 1]"
    } else {
       set m2 "Yes - Please continue"
    }
    if {[llength $args] >= 1} then {
       set m1 "[lindex $args 0]"
    } else {
       set m1 "No - Abort the operation"
    }
    set z [catch {midas_dialog [self frame-name].confirmDialog "Confirmation Request for [self frame-title]" \
           "$msg" "" 1 "$m1" "$m2"} m]

    if {$z != 0} then {
       puts stderr "midas-warning: [self frame-name].confirmDialog: $m"
       destroy [self frame-name].confirmDialog
       return 0
    }
    return $m
}  

proc midas-error {e args} {
   global tkerrorInfo tkskiperror
   global env

   set tkskiperror 1
   set tkerrorInfo "Called from midas-error"

   if {$env(EGsilent) >= 1} {
       if {[grab current .] != ""} then {grab release [grab current .]}
   }

   if {$e == ""} then {
     set m "$args"
   } else {
     set m1 "Midas error $e"
     set m2 [eg "(sys :error $e ?message)"]
     set m "$m1\n$m2\n$args"
   }

     set env(ErrorText) "$m"
     if {$env(EGsilent) >= 1} {puts stdout "[Get_My_Name]: midas-report: (silent) $env(ErrorText)"; return} 

   catch {:modify [self frame-name] :busy T}
   set z [catch {midas_dialog [self frame-name].midasDialog "Error Report for [self frame-title]" \
           "$m" "warning" 0 "continue" "abort"} button]
   catch {:modify [self frame-name] :busy F}

    if {$z != 0} then {
       puts stderr "display-midas-error-report: [self frame-name].midasDialog: $button"
       destroy [self frame-name].midasDialog
    }
   return $button
}

proc midas-help {widget class msg} {

   set z [catch {midas_dialog [self frame-name].helpDialog "Help for $class $widget" \
           "$msg" "" 0 "Ok"} m]

   if {$z != 0} then {
       puts stderr "midas-report: [self frame-name].helpDialog: $m"
       destroy [self frame-name].helpDialog
   }
   if {$class == "Button"} then {after 1 [:path $widget] config -state normal}
}

proc midas-information {text} {
   global _information_font

    set frame [self frame-name].info

    :frame $frame :label  "Information from [self frame-title]" :toplevel T
         wm protocol $frame WM_DELETE_WINDOW "destroy $frame"

    :galley G :deaf T :strings $text :font $_information_font
    :show T
}


proc midas-show-file {file} {

#    read the file

   set z [catch {open $file} f]
   if {$z != 0} then {midas-warning $f; return}
   set text [read $f]
   close $f

   set frame [self frame-name].info

   :frame $frame :label  "Information for [self frame-title]" :show-footer T :toplevel T
         wm protocol $frame WM_DELETE_WINDOW "destroy $frame"

    :galley G :deaf T :strings $text
    :show T

   set-footer $frame "Information from file $file"
}

set sframenumber 0

proc midas-show-task-source {} {
   global autoloaded sframenumber
   global mytaskname

  set taskname $mytaskname
  if {[string range $mytaskname 0 0] == "."} then {set taskname [string range $mytaskname 1 end]} \
      else {set taskname $mytaskname}

   foreach file $autoloaded {
      if {[midas-confirmation "Show source file $file?"]} then {

#    read the file

         set z [catch {open $file} f]
         if {$z != 0} then {midas-warning $f; return}
         set text [read $f]
         close $f

         set win [self frame-name]$sframenumber

         :frame $win :label  "Source file $file for task $taskname" :toplevel T
         wm protocol $win WM_DELETE_WINDOW "destroy $win"
         :layout T
         :galley MIDASSOURCE :rows 40 :cols 100 :strings $text
#         :layout B; :set fill none; :set anchor center
#         :button MIDASSOURCEBUTTON :label "Dismiss" :command "destroy $win"

         open-frame  [self frame-name]$sframenumber

         incr sframenumber
      }
   }
}

proc midas-show-object {W} {
#    this procedure is a default dummy procedure
#    the real procedure is possibly task dependent
   global _helpwindow
   set W $_helpwindow
   set w [:widget $W]
   set i [$W configure]
   midas-report "This window has path $W which is a component of widget $w\n\nWindow configure is $i"
}

proc midas-print-object {W} {
#    Print current widget (selective rather than window print from Base Frame)
#    Use _helpwindow since that tracks current mouse location correctly
   global _helpwindow
   set W $_helpwindow
   set w [:widget $W]

   set-footer "printing $W"

   midas-capture $W
}

proc midas-save-object {W} {
#    save current widget (selective rather than window print from Base Frame)
#    Use _helpwindow since that tracks current mouse location correctly
   global _helpwindow print_stuff
   set W $_helpwindow
   set w [:widget $W]

   set-footer "saving $W"

   set print_stuff [midas-capture $W noprint]
   if {$print_stuff == ""} then {return}

   midas-browser W print-to-file
}

proc midas-save-window {window} {
   upvar #0 browserdirectory directory
   upvar #0 browserfile file

   set z [catch {open $directory/$file w} f]
   if {$z != 0} then {midas-warning "Unable to write $directory/$file\n$f"; return}
   if {[last [split $window .]] == "text"} then {
      puts $f [$window get 1.0 end]
   } else {
      foreach item [$window get 0 end] {puts $f $item}
   }
   close $f
}

proc  midas-print-window {window} {
   global env
   set tempfile [file join $env(MIDASTempDir) tcl[pid] print.txt]
   set f [open $tempfile w]
   if {[last [split $window .]] == "text"} then {
      puts $f [$window get 1.0 end]
   } else {
      foreach item [$window get 0 end] {puts $f $item}
   }
   close $f
   print-listing $tempfile
}

proc midas-capture {option args} {
   global env preferences

   if {[midas-check-for-temp] == 0} then {return ""}

   set z [catch {
      set tempfile  [file join $env(MIDASTempDir) tcl[pid] [set option]dump.ps]
      set xwdfile   [file join $env(MIDASTempDir) tcl[pid] [set option]dump.xwd]
      if {$option == "screen"} then {
         switch $env(platform) {
            unix   {
               if {$env(platform) == "Linux" && [winfo depth .] == 8} then {
                  set-footer "Capturing screen."
                  exec xgrabsc -root -s 0  -l  -cps > $tempfile
               } else {
                  set-footer "Capturing screen."
                  exec xwd -root > $xwdfile
                  set z [catch {exec xwd2ps < $xwdfile > $tempfile} m]
               }
            }
            windows {
               midas-warning "This function is not current supported by the Windows version but is coming soon"
               clear-footer
               return ""
            }
            default {midas-warning "$env(platform) not supported at present"}
         }
      } else {
         if {$option == "window"} then {
            switch $env(platform) {
            unix   {
               if {$env(platform) == "Linux" && [winfo depth .] == 8} then {
                  set-footer "Click on window to capture."
                  exec xgrabsc -click -s 0  -l  -cps > $tempfile
               } else {
                  set-footer "Click on window to capture."
                  exec xwd > $xwdfile
                  set z [catch {exec xwd2ps < $xwdfile > $tempfile} m]
               }
            }
              windows {
                  midas-warning "This function is not current supported by the Windows version but is coming soon"
                  clear-footer
                  return ""
              }
              default {midas-warning "$env(platform) not supported at present"}
            }
         } else {
            switch $env(platform) {
            unix   {
               if {$env(platform) == "Linux" && [winfo depth .] == 8} then {
                  set-footer "capturing $option."
                  exec xgrabsc -id [winfo id $option] -s 0  -l  -cps > $tempfile
               } else {
                  set-footer "capturing $option."
                  exec xwd  -id [winfo id $option] > $xwdfile
                  set z [catch {exec xwd2ps < $xwdfile > $tempfile} m]
               }
            }
               windows {
                  midas-warning "This function is not current supported by the Windows version but is coming soon"
                  clear-footer
                  return ""
               }
               default {midas-warning "$env(platform) not supported at present"}
            }
         }
      }
      clear-footer
      if {[first $args] != "noprint"} then {print-postscript $tempfile}
   } m]
   if {$z==0} then {
      set-footer "printing [set option]dump"
   } else {
      midas-warning "[set option] dump failed \n $m"
      set tempfile ""
   }
   after 2000 clear-footer
   return $tempfile
}

proc midas-edit_file {file}  {
    global env __texteditdone

  :frame "[self frame-name].textedit" :label "[self frame-title]" :show-footer T  :resize F :toplevel T \
      :icon-image "@$env(ICONHOME)/default" \
      :icon-name "[self icon-title]"

    set __texteditdone 0
  
    :panel TEXTEDITPANEL :layout T :padx 10 :pady 10
      :layout H; :row-gap 5; :col-gap 20

    :galley TEXTEDIT :rows 50 :cols 100 :font fixed
     :next-row T

    :button TEXTEDITDONE :label "Save file and finish"     :command edit_file_done
    :button TEXTEDITQUIT :label "Quit without saving file" :command edit_file_quit
    :show T

   set z [catch {open $file r} f]
   if {$z != 0} then {midas-warning "Unable to read $file\n$f"; destroy "[self frame-name].textedit"; return}
   set text [read $f]
   close $f

    :modify TEXTEDIT :strings "$text" 

   vwait __texteditdone

   if {$__texteditdone == 1}  {

       set text "[[:path TEXTEDIT].text get 1.0 end]"

       set z [catch {open $file w} f]
       if {$z != 0} then {midas-warning "Unable to write file $file\n$f"; destroy "[self frame-name].textedit"; return}
       puts -nonewline $f "$text"
       close $f
   }

   destroy "[self frame-name].textedit"

}

proc edit_file_done {} {global __texteditdone; set __texteditdone 1}
proc edit_file_quit {} {global __texteditdone; set __texteditdone 2}

proc midas-query {question}  {
    global env __queryresponse

  :frame "[self frame-name].query" :label "[self frame-title]" :show-footer T  :resize F :toplevel T \
      :icon-image "@$env(ICONHOME)/default" \
      :icon-name "[self icon-title]"

    set __querydone 0
  
    :panel QUERYPANEL :layout T :padx 10 :pady 10
      :layout H; :row-gap 5; :col-gap 20
      
     :text QUERY :w 40  :label "$question" :command query_done
               
     :next-row T

    :button QUERYQUIT :label "cancel"          :command query_quit
    :show T

   vwait __queryresponse

   destroy "[self frame-name].query"
   
   return $__queryresponse

}

proc query_done {args} {global __queryresponse; set __queryresponse "[second $args]"}
proc query_quit {}     {global __queryresponse; set __queryresponse ""}


proc :frame args {
###########
###########
  global env :__interp
  global debug env  _frame _container _charh _charw

#    :frame name :label ? :icon-name ? :icon-image ? :w ? :h ? :toplevel T|F
#         :show-footer T :resize F :closed T  :win-ctrl N|F|T

  global _panel  _charh _footer_position

  set xw 0; set xh 0; set footer T
  set closed 0; set resize 0; set transient 0; set placed 0; set depth 0
  set type "toplevel"

  switch $env(platform) {
     windows  {set winc N}
     unix     {set winc F}
     default  {set winc T}
  }

  set _xposn 0
  set _yposn 0

  set name [lindex $args 0]
  if {[string match .* $name] == 0} then {set name .[string tolower $name]}
  set win $name

#   some defaults

  set name [string range $name 1 end]
  set ll "$name Frame"
  set in "$name"
  set ii "@$env(ICONHOME)/default"

  global _classname _winname
  set _classname($name) "frame"
  set _winname($name)  $win

  set i 1
  while {$i < [llength $args]} {
    set p [lindex $args $i]
    set v [lindex $args [expr $i + 1]]
    switch $p {

      :w  {set xw $v; set placed 1}
      :h  {set xh $v; set placed 1}
      :label {set ll "$v"}
      :icon-name {set in "$v"}
      :icon-image {set ii "$v"}
      :toplevel {
        if {$v == "T"} then {set type "toplevel"}
        if {$v == "F"} then {set type "frame"}
      }
      :show-footer {
        if {$v == "T"} then {set footer $v}
        if {$v == "F"} then {set footer $v}
      }
      :win-ctrl {
        if {$v == "N"} then {set winc $v}
        if {$v == "F"} then {set winc $v}
        if {$v == "T"} then {set winc $v}
      }
      :resize {
        if {$v == "T"} then {set resize 0}
        if {$v == "F"} then {set resize 1}
      }
      :closed {
        if {$v == "T"} then {set closed 1}
        if {$v == "F"} then {set closed 0}
      }
      :pack {
        if {$v == "T"} then {set placed 0}
      }
      :place {
        if {$v == "T"} then {set placed 1}
      }
      :transient {
        if {$v == "T"} then {set transient 1}
      }
      :colours {
        if {$v == "H"} then {set depth 24}
        if {$v == "L"} then {set depth 8}
      }

      default {puts stderr ":frame - unknown arg $p"}
    }
    set i [expr $i + 2]
  }

   catch {destroy $win}

   set visuals [winfo visualsavailable .]
   if {$depth <= 8} then {
       if {[lsearch $visuals "pseudocolor 8"] >= 0} then {
           $type $win -visual "pseudocolor 8"
       } else {
           $type $win
       }
   } else {
       if {[lsearch $visuals "truecolor 24"] >= 0} then {
           $type $win -visual "truecolor 24"
       } else {
           $type $win
       }
   }
   if {$debug} then {insert-debug ":frame created $win of type $type"}

  set WIN [uppercase [string range $win 1 end]]
  if {[self frame-name] == "$win"} then {:_globalbindings all}

  if {$transient == 0} then {
    bind $win <Unmap>        "::enable-event $win Unmap"
    bind $win <Map>          "::enable-event $win Map"
    bind $win <Visibility>   "::enable-event $win Visibility"
    bind $win <Configure>    "::enable-event $win Configure"
    bind $win <Enter>        "::enable-event $win Enter"

    if {$type == "frame"} then {
       wm protocol .    WM_DELETE_WINDOW "click {$WIN :quit}"
       wm withdraw .
    } else {
       wm protocol $win WM_DELETE_WINDOW "click {$WIN :quit}"
       wm withdraw $win
    }
  } else {
      set flag [last [split $win .]]
      wm transient $win [winfo toplevel [winfo parent $win]]
      wm protocol $win WM_DELETE_WINDOW "global [set flag]visible; set [set flag]visible 0; destroy $win"
      if {$type == "frame"} then {wm withdraw .} else {wm withdraw $win}
  }

   if {$type == "toplevel"} then {set ww $win} else {set ww .}
   wm title $ww "$ll"
   wm iconname $ww "$in"
   wm iconbitmap $ww "$ii"

   if {$resize == 1} then {wm resizable $ww no no}

   $win configure -relief raised -borderwidth 1 -width $xw -height $xh
   if {$type == "frame"} then {pack $win -side top -fill x}

   :_set-color $win frame

   if {$footer == "T"} then {
     frame $win.footer -height 1c -width $xw
     message $win.footer.footer -aspect 10000 -width $xw
     if {$winc == "F"} then {
        button $win.footer.quit -relief raised -width 8 -height 8 \
          -bitmap "@$env(ICONHOME)/quit" -command "frame-quit $win"
        button $win.footer.close -relief raised -width 8 -height 8 \
          -bitmap "@$env(ICONHOME)/close" -command "frame-close $win"
     }

     bind $win.footer.footer <ButtonPress-1> "clear-footer"
     bind $win.footer.footer <ButtonPress-3> "clear-busy"

     set _classname(footer) "footer"
     set _winname(footer) $win.footer

     :_set-font $win.footer.footer footer 0 N

     if {$placed == 0} then {
       pack $win.footer -side bottom -anchor s -fill both
       if {$winc == "F"} then {
         pack $win.footer.quit  -side right -anchor e
         if {$transient == 0} then {pack $win.footer.close -side right -anchor e}
       }
     } else {
       place $win.footer -x 5 -rely 1.0 -y -20
     }

     $win.footer.footer configure -justify $_footer_position
     if {$_footer_position == "center"} then {
        pack $win.footer.footer -side bottom -fill both -anchor s
     } else {
        pack $win.footer.footer -side left   -fill both -anchor w
     }
   }

#    if footer is False and winc is Footer then switch winc to Topper
   if {$footer == "F" && $winc == "F"} then {set winc "T"}

   if {$winc == "T"} then {
     frame $win.topper 
     button $win.topper.quit -relief raised -width 8 -height 8 \
         -bitmap "@$env(ICONHOME)/quit" -command "frame-quit $win"
     button $win.topper.close -relief raised -width 8 -height 8 \
         -bitmap "@$env(ICONHOME)/close" -command "frame-close $win"

     set _classname(topper) "topper"
     set _winname(topper) $win.topper

     pack $win.topper -side top -anchor n -fill both
     pack $win.topper.quit  -side right -anchor e
     if {$transient == 0} then {pack $win.topper.close -side right -anchor e}
   }

   :initialise
   set _frame $win
   set _panel ""
   set _container ""

   if {$env(USERCOLORS) == 0} then {set env(tkfooterbusycolor) ""}
}

proc :_globalbindings {win} {
    bind $win <Control-KeyPress-Delete> "finish; DeleteTask"
    bind $win <Alt-KeyPress-Delete>     "finish; DeleteTask"

    bind $win <KeyPress-F1>          ":_help_action 0 %W"
    bind $win <Shift-KeyPress-F1>    ":_help_action 1 %W"
    bind $win <Alt-KeyPress-F1>      ":load-help"

    bind $win <KeyPress-F7>          ":_help_action 0 %W"
    bind $win <Shift-KeyPress-F7>    ":_help_action 1 %W"

    bind $win <Shift-KeyPress-F2>    midas-show-task-source
    bind $win <Shift-KeyPress-F3>    "midas-show-object %W"

    bind $win <Shift-Control-KeyPress-P> "midas-print-object %W"
    bind $win <Shift-Control-KeyPress-S> "midas-save-object %W"

    bind $win <Control-KeyPress-F5>  "wm geom . +0+0"
    bind $win <Alt-KeyPress-F5>      "wm geom . +0-0"
    bind $win <Control-KeyPress-F6>  "wm geom . -0-0"
    bind $win <Alt-KeyPress-F6>      "wm geom . -0+0"
}


proc :modify_frame {win name args} {
  global debug env
  global __coloursave1 __coloursave2

#    options -    :label ? :delete T|F :closed T|F :w ? :h ?

  set args [lindex $args 0]
  if {$debug} then {insert-debug ":modify_frame called with  $win $name $args"}

  set i 0
  while {$i < [llength $args]} {
    set p [lindex $args $i]
    set v [lindex $args [expr $i + 1]]
    switch $p {

      :w {$win configure -width $v}
      :h {$win configure -height $v}
      :label {wm title $win "$v"}
      :busy {
        switch $v {
          T {
             set footertext [$win.footer.footer cget -text]

   #         if {$footertext == ""} then {
   #            $win.footer.footer configure -text "Busy...."
   #         }

             if {$footertext == ""} then {
                 $win.footer.footer configure -text "Busy...."
             } else {
                 $win.footer.footer configure -text ""
                 update idletasks
                 $win.footer.footer configure -text "$footertext"
             }

             if {[$win.footer.footer cget -background] != $env(tkfooterbusycolor)} then {
                if {$env(USERCOLORS) == 0} then {set env(tkfooterbusycolor) [$win.footer.footer cget -background]}
                set __coloursave1 [$win.footer.footer cget -background]
                set __coloursave2 [$win.footer cget -background]
                $win.footer.footer configure -background $env(tkfooterbusycolor)
                $win.footer configure -background $env(tkfooterbusycolor)
             }
             :cursor $win watch
             update idletasks
          }
          F {

             set footertext [$win.footer.footer cget -text]

             if {$footertext == "Busy...."} then {
                $win.footer.footer configure -text ""
             } else {
                $win.footer.footer configure -text ""
                update idletasks
                $win.footer.footer configure -text "$footertext"
             }

             if {[$win.footer.footer cget -background] == $env(tkfooterbusycolor)} then {
                if {$__coloursave1 != ""} then {
                   $win.footer.footer configure -background $__coloursave1
                   $win.footer configure -background $__coloursave2
                }
             }
             :cursor $win left_ptr
             update idletasks
          }
          default {puts stderr ":modify  :busy  - unknown option $v"}
        }
      }
      :delete {
        switch $v {
          T {wm withdraw .; destroy $win}
          F {}
          default {puts stderr ":modify  :delete  - unknown option $v"}
        }
      }
      :transient {
        switch $v {
          T {
            set flag [last [split $win .]]
            bind $win <Unmap> "global [set flag]visible; set [set flag]visible 0; destroy $win"
            wm protocol $win WM_DELETE_WINDOW "global [set flag]visible; set [set flag]visible 0; destroy $win"
            }
          F {}
          default {puts stderr ":modify  :transient  - unknown option $v"}
        }
      }
      :closed {
        switch $v {
          T {wm withdraw .}
          F {wm deiconify .; raise .}
          default {puts stderr ":modify  :show  - unknown option $v"}
        }
      }

      default {puts stderr ":modify_frame - unknown arg $p"}
    }
    set i [expr $i + 2]
  }

}

proc :query_frame {win name args} {
  global debug

#    settings -    :w :h :label :footer :busy

  set args [lindex $args 0]
  if {$debug} then {insert-debug ":query_frame called with  $win $name $args"}

  switch [set p [lindex $args 0]] {
    :w {return [$win cget -width]}
    :h {return [$win cget -height]}
    :label {return [wm title .]}
    :footer {return [$win.footer.footer cget -text]}
    :busy {return [if {[$win cget -cursor] == "watch"} {set b "T"} {set b F}]}
    default {puts stderr ":query_frame $win - unknown arg $p"}
  }
}


proc :panel args {
###########
###########
  global debug _frame _panel _container _xposn _yposn _layout _rowgap _colgap _placed
  global _layout _anchor _expand _fill _ipadx _ipady _padx _pady _pack
  global __skipupdate _xinc _yinc
  global _autosize _hreq _wreq

#    :panel name :x ? :y ? :w ? :h ? :layout H|V :row-gap ? :col-gap ?

  :_reset

  set placed 0
  set _xposn 0
  set _yposn 0
  set _rowgap 0
  set _colgap 0
  set _xinc 0
  set _yinc 0
  set __skipupdate 1
  set _placed 0
  set xp 0; set yp 0
  set _autosize 0; set _hreq 0; set _wreq 0

  set name [string tolower [lindex $args 0]]


  if {$_container == ""} then {
        set win $_frame.$name
  } else {
        set win $_frame.$_container.$name
  }
  set _panel $name

  global _classname _winname
  set _classname($name) "panel"
  set _winname($name)  $win

  frame $win -relief flat
  if {$debug} then {insert-debug ":panel created $win"}
  if {$debug} then {$win configure -relief raised -bd 1}

  #$win configure -relief raised -bd 1

  set i 1
  while {$i < [llength $args]} {
    set p [lindex $args $i]
    set v [lindex $args [expr $i + 1]]
    switch $p {

      :x  {set xp $v; set placed 1}
      :y  {set yp $v; set placed 1}
      :w  {set _autosize 1; $win configure -width $v}
      :h  {set _autosize 1; $win configure -height $v}
      :relief {
         switch $v {
            raised -
            R   {$win configure -relief raised -bd 1}
            default {}
         }
      }
      :row-gap {set _rowgap $v; set _placed 1}
      :col-gap {set _colgap $v; set _placed 1}
      :layout {
        switch $v {
          L  {set _layout "left"}
          H  {set _layout "left"}
          T  {set _layout "top"}
          V  {set _layout "top"}
          R  {set _layout "right"}
          B  {set _layout "bottom"}
          default {puts stderr ":panel :layout - unknown option $v"}
        }
      }
      :anchor {set _anchor [string tolower $v]}
      :expand {set _expand [string tolower $v]}
      :fill   {set _fill [string tolower $v]}
      :ipadx  {set _ipadx $v}
      :ipady  {set _ipady $v}
      :padx   {set _padx  $v}
      :pady   {set _pady  $v}
      :after  {set _pack " -after  [:path $v] "}
      :before {set _pack " -before [:path $v] "}

      default {puts stderr ":panel - unknown arg $p"}
    }
    set i [expr $i + 2]
  }

  :_set-color $win frame

  :_position-widget $placed $win $xp $yp
}

proc :modify_panel {win name args} {
  global debug _frame  _panel _container

#    options -    :delete T|F :w ? :h ?

  set args [lindex $args 0]
  if {$debug} then {insert-debug ":modify_panel called with  $win $name $args"}

  set i 0
  while {$i < [llength $args]} {
    set p [lindex $args $i]
    set v [lindex $args [expr $i + 1]]
    switch $p {

      :w {$win configure -width $v}
      :h {$win configure -height $v}
      :delete {
        switch $v {
          T {destroy $win}
          F {}
          default {puts stderr ":modify  :delete  - unknown option $v"}
        }
      }
      :empty {
        switch $v {
          T {
             foreach w [winfo children $win] {destroy $w}

             :_reset
             global _xposn _yposn _xinc _yinc _skipupdate _autosize
             global _rowgap _colgap _placed _hreq _wreq
             set _xposn 0; set _yposn 0; set _xinc 0; set _yinc 0; set __skipupdate 1
             set _autosize 1
             set _rowgap 0; set _colgap 0
          #   set _placed 0; set _hreq 0; set _wreq 0

          }
          F {}
          default {puts stderr ":modify  :empty  - unknown option $v"}
        }
        set _panel [string tolower $name]
        set l [split $win .]
        set _frame .[second $l]
        if {[llength $l] == 4} then {set _container [third $l]} else {set _container ""}
      }
      default {puts stderr ":modify_panel - unknown arg $p"}
    }
    set i [expr $i + 2]
  }

}

proc :query_panel {win name args} {
  global debug

#    settings -    :w :h

  set args [lindex $args 0]
  if {$debug} then {insert-debug ":query_panel called with  $win $name $args"}

  switch [set p [lindex $args 0]] {
    :w {return [$win cget -width]}
    :h {return [$win cget -height]}
    default {puts stderr ":query_panel $win - unknown arg $p"}
  }
}


proc :container args {
###########
###########
  global debug _frame _panel _container _xposn _yposn _layout _rowgap _colgap _placed
  global _layout _anchor _expand _fill _ipadx _ipady _padx _pady _pack
  global __skipupdate _xinc _yinc
  global _autosize _hreq _wreq

#    :container name :x ? :y ? :w ? :h ? :layout H|V

  if {[first $args] == "end"} then {set _container ""; return}

  :_reset

  set placed 0
  set _xposn 0
  set _yposn 0
  set _rowgap 0
  set _colgap 0
  set _xinc 0
  set _yinc 0
  set __skipupdate 1
  set _placed 0
  set xp 0; set yp 0
  set _autosize 0; set _hreq 0; set _wreq 0

  set name [string tolower [lindex $args 0]]
  set win $_frame.$name
  set _container $name

  global _classname _winname
  set _classname($name) "container"
  set _winname($name)  $win

  frame $win -relief flat
  if {$debug} then {insert-debug ":container created $win"}
  if {$debug} then {$win configure -relief raised -bd 1}

  set i 1
  while {$i < [llength $args]} {
    set p [lindex $args $i]
    set v [lindex $args [expr $i + 1]]
    switch $p {

      :x  {set xp $v; set placed 1}
      :y  {set yp $v; set placed 1}
      :w  {set _autosize 1; $win configure -width $v}
      :h  {set _autosize 1; $win configure -height $v}
      :relief {
         switch $v {
            raised -
            R   {$win configure -relief raised -bd 1}
            default {}
         }
      }
      :layout {
        switch $v {
          L  {set _layout "left"}
          H  {set _layout "left"}
          T  {set _layout "top"}
          V  {set _layout "top"}
          R  {set _layout "right"}
          B  {set _layout "bottom"}
          default {puts stderr ":container :layout - unknown option $v"}
        }
      }
      :anchor {set _anchor [string tolower $v]}
      :expand {set _expand [string tolower $v]}
      :fill   {set _fill [string tolower $v]}
      :ipadx  {set _ipadx $v}
      :ipady  {set _ipady $v}
      :padx   {set _padx  $v}
      :pady   {set _pady  $v}
      :after  {set _pack " -after  [:path $v] "}
      :before {set _pack " -before [:path $v] "}

      default {puts stderr ":container - unknown arg $p"}
    }
    set i [expr $i + 2]
  }

  :_set-color $win frame

  :_position-widget $placed $win $xp $yp

  set _panel ""
}

proc :modify_container {win name args} {
  global debug _panel _container

#    options -    :delete T|F :w ? :h ?

  set args [lindex $args 0]
  if {$debug} then {insert-debug ":modify_container called with  $win $name $args"}

  set i 0
  while {$i < [llength $args]} {
    set p [lindex $args $i]
    set v [lindex $args [expr $i + 1]]
    switch $p {

      :w {$win configure -width $v}
      :h {$win configure -height $v}
      :delete {
        switch $v {
          T {destroy $win}
          F {}
          default {puts stderr ":modify  :delete  - unknown option $v"}
        }
      }
      :empty {
        switch $v {
          T {
             foreach w [winfo children $win] {destroy $w}

             :_reset
             global _padx _pady _xposn _yposn _xinc _yinc _skipupdate _autosize
             set _xposn 0; set _yposn 0; set _xinc 0; set _yinc 0; set __skipupdate 1
             set _autosize 1
             set _padx 0; set _pady 0
          }
          F {}
          default {puts stderr ":modify  :empty  - unknown option $v"}
        }
        set _container [string tolower $name]
        set l [split $win .]
        set _frame .[second $l]
        set _panel ""
      }
      default {puts stderr ":modify_container - unknown arg $p"}
    }
    set i [expr $i + 2]
  }

}

proc :query_container {win name args} {
  global debug

#    settings -    :w :h

  set args [lindex $args 0]
  if {$debug} then {insert-debug ":query_container called with  $win $name $args"}

  switch [set p [lindex $args 0]] {
    :w {return [$win cget -width]}
    :h {return [$win cget -height]}
    default {puts stderr ":query_container $win - unknown arg $p"}
  }
}


proc :button args {
############
############
  global debug env _frame _panel _container _layout _placed _anchor _pack
  global _xposn _yposn _colgap _rowgap _charw

#    :button name :x ? :y ? :w ? :h ? :label ?

  set placed $_placed
  set width 0
  set xw 0; set xh 0
  set padx 0; set pady 3; set hlt 0
  set bold 0; set deaf "F"; set help "T"; set colour ""

  set name [string tolower [lindex $args 0]]
  set helpitem [lindex $args 0]
  set ll [string toupper [string range $name 0 0]]
  append ll [string range $name 1 end]

  if {$_container == ""} then {
     if {$_panel == ""} then {
        set win $_frame.$name
     } else {
        set win $_frame.$_panel.$name
     }
  } else {
     if {$_panel == ""} then {
        set win $_frame.$_container.$name
     } else {
        set win $_frame.$_container.$_panel.$name
     }
  }

  global _classname _winname
  set _classname($name) "button"
  set _winname($name)  $win

  :_pre-updateposn;  set xp $_xposn; set yp $_yposn

  button $win -relief raised -command ":button-event $name"
  if {$debug} then {insert-debug ":button  created $win"}
  if {$debug >= 2} then {puts stderr "button $win -relief raised -command \":button-event $name\""}

#    setup special default processing for HELP widget
  if {$name == "help"} then {
     global _commandname
     set _commandname($name) "click-help"

     set taskname [self help-name]
     if {[string range $taskname 0 0] == "."} then {set taskname [string range $taskname 1 end]}
     set i [inform help "check-for-help-on-task" $taskname]
     if {$i == 0} then {
        $win configure -state disabled
     } else {
        if {$env(platform) == "unix"} then {
           image create photo help -format gif -palette 3/3/3 -file $env(ICONHOME)/help.gif
        } else {
           image create photo help -format gif -file $env(ICONHOME)/help.gif
        }
        $win configure -state normal -image help
     }
  }

  set i 1
  while {$i < [llength $args]} {
    set p [lindex $args $i]
    set v [lindex $args [expr $i + 1]]
    switch $p {

      :x  {set xp $v; set placed 1}
      :y  {set yp $v; set placed 1}
      :valuex -
      :w  {set xw $v}
      :h  {set xh $v}
      :label  {set ll "$v"}
      :image  {
         set ll ""
         set format [last [split $v .]]
         if {$env(platform) == "unix"} then {
            image create photo $name -format $format -palette 3/3/3 -file $v
         } else {
            image create photo $name -format $format -file $v
         }
         $win configure -image $name
      }
      :padx {set padx $v}
      :pady {set pady $v}
      :hlt  {set hlt $v}
      :show {}
      :deaf {
        switch $v {
          T    {set deaf "T"}
          F    {set deaf "F"}
          default {puts stderr ":button :deaf - unknown option $v"}
        }
      }
      :inactive {
        switch $v {
          T {$win configure -state disabled}
          F {$win configure -state normal}
          default {puts stderr ":button :inactive - unknown option $v"}
        }
      }
      :bold {
        switch $v {
          T    {set bold 1}
          F    {set bold 0}
          default {puts stderr ":button :bold - unknown option $v"}
        }
      }
      :colour {set colour $v}
      :help {
        switch $v {
          T    {set help "T"}
          F    {set help "F"}
          default {:help $helpitem $v}
        }
      }
      :helpitem {set helpitem $v}
      :double-click {}
      :after  {set _pack " -after  [:path $v] "}
      :before {set _pack " -before [:path $v] "}
      :command {global _commandname; set _commandname($name) "$v"}

      default {puts stderr ":button - unknown arg $p ($args)"}
    }
    set i [expr $i + 2]
  }

  $win configure -width $xw -height $xh -highlightthickness $hlt -pady $pady
  if {$xw > 0 && $xh > 0} then {$win configure -wraplength [expr $xw * $_charw]}
  if {$ll != ""} then {
     if {[string range $ll 0 0] == "@"} then {
        $win configure -bitmap "$ll"
     } else {
        $win configure -text "$ll"
     }
  }
  if {$padx != 0} then {$win configure -padx $padx}

  if {$deaf == "T"} then {:_deaf $win Button}
  if {$help == "T"} then {:_help $helpitem $win Button}

  :_set-font  $win button $bold N
  :_set-color $win button
  if {$colour != ""} then {$win configure -background $colour}

  :_position-widget $placed $win $xp $yp

  :_post-updateposn $placed $win $xp $yp

}

proc :button-event {name} {
  global frame _commandname _winname
  if {[info exists _winname($name)]} {$_winname($name) flash}
  if {[info exists _commandname($name)]} then {eval $_commandname($name)} else \
  {click [list $frame "" [string toupper $name]]}
}

proc :modify_button {win name args} {
  global debug

#    options -   :busy T|F :inactive T|F

  set args [lindex $args 0]
  if {$debug} then {insert-debug ":modify_button called with  $win $name $args"}

  set i 0
  while {$i < [llength $args]} {
    set p [lindex $args $i]
    set v [lindex $args [expr $i + 1]]
    switch $p {

      :deaf {}
      :label {$win configure -text $v}
      :colour {$win configure -background $v}
      :inactive  {
        switch $v {
          T {$win configure -state disabled}
          F {$win configure -state normal}
          default {puts stderr ":modify_button $win  :inactive  - unknown option $v"}
        }
      }
      :busy  {
        switch $v {
          T {$win configure -relief sunken}
          F {$win configure -relief raised}
          default {puts stderr ":modify_button $win  :busy  - unknown option $v"}
        }
      }

      default {puts stderr ":modify_button $win - unknown arg $p"}
    }
    set i [expr $i + 2]
  }

}

proc :query_button {win name args} {
  global debug

#    settings -    :label :busy

  set args [lindex $args 0]
  if {$debug} then {insert-debug ":query_button called with  $win $name $args"}

  switch [set p [lindex $args 0]] {
    :label {return [$win cget -text]}
    :colour {return [$win cget -background]}
    :busy  {if {[$win cget -relief]=="sunken"} {return "T"} {return "F"}}

    default {puts stderr ":query_button $win - unknown arg $p"}
  }
}


proc :text args {
##########
##########
  global debug env _frame _panel _container _layout _xposn _yposn _placed _anchor _pack _colgap _rowgap _charh _charw


#    :text name :x ? :y ? :w ? :h ? :valuex ? :label ? :value ? :deaf T/F :read-only T/F

  set placed $_placed
  set xp $_xposn; set yp $_yposn
  set xw 0; set xh 0; set vx 0; set vl ""
  set bold 0;set size N; set deaf "F"; set shifters "F"; set help "T"

  set name [string tolower [lindex $args 0]]
  set helpitem [lindex $args 0]
  set ll [string toupper [string range $name 0 0]]
  append ll [string range $name 1 end]

  if {$_container == ""} then {
     if {$_panel == ""} then {
        set win $_frame.$name
     } else {
        set win $_frame.$_panel.$name
     }
  } else {
     if {$_panel == ""} then {
        set win $_frame.$_container.$name
     } else {
        set win $_frame.$_container.$_panel.$name
     }
  }

  global _classname _winname _variable
  set _classname($name) "text"
  set _winname($name)  $win
  set _variable($name) ""

  :_pre-updateposn;  set xp $_xposn; set yp $_yposn

  frame $win -relief flat
  if {$debug} then {$win configure -relief raised -bd 1}

  label $win.label
  entry $win.entry -relief sunken -textvariable _variable($name)
  bind  $win.entry <Return>   ":text-event $win $name"

  global _default_autoreturn
  set _autoreturn $_default_autoreturn

  if {$debug} then {insert-debug ":text created $win $win.label and $win.entry"}

  set i 1
  while {$i < [llength $args]} {
    set p [lindex $args $i]
    set v [lindex $args [expr $i + 1]]
    switch $p {

      :x  {set xp $v; set placed 1}
      :y  {set yp $v; set placed 1}
      :h  {set xh $v}
      :w  {set xw $v}
      :valuex {set vx $v}
      :label  {set ll "$v"}
      :value  {set vl $v}
      :show {}
      :read-only -
      :deaf {
        switch $v {
          T    {set deaf "T"}
          F    {set deaf "F"}
          default {puts stderr ":text :deaf - unknown option $v"}
        }
      }
      :auto {
        switch $v {
          T    {set _autoreturn "T"}
          F    {set _autoreturn "F"}
          default {puts stderr ":text :auto - unknown option $v"}
        }
      }
      :shifters {
        switch $v {
          T    {set shifters "T"}
          F    {set shifters "F"}
          default {puts stderr ":list :shifters - unknown option $v"}
        }
      }
      :inactive {
        switch $v {
          T {$win.entry configure -state disabled}
          F {$win.entry configure -state normal}
          default {puts stderr ":text :inactive - unknown option $v"}
        }
      }
      :bold {
        switch $v {
          T    {set bold 1}
          F    {set bold 0}
          default {puts stderr ":text :bold - unknown option $v"}
        }
      }
      :size {
        switch $v {
          N    -
          S    -
          L    {set size $v}
          default {puts stderr ":text :size - unknown option $v"}
        }
      }
      :help {
        switch $v {
          T    {set help "T"}
          F    {set help "F"}
          default {:help $helpitem $v}
        }
      }
      :helpitem {set helpitem $v}
      :double-click {}
      :command {global _commandname; set _commandname($name) "$v"}
      :after  {set _pack " -after  [:path $v] "}
      :before {set _pack " -before [:path $v] "}

      default {puts stderr ":text - unknown arg $p"}
    }
    set i [expr $i + 2]
  }

  if {$xw != 0} then {$win.entry configure -width $xw}
  if {$xw == 0} then {set xw [string length $vl]}

  if {$shifters == "T"} then {
     button $win.lefter  -image lefter  -takefocus 0
     button $win.righter -image righter -takefocus 0
     bind $win.lefter  <ButtonRelease-1>  ":text-adjust-entry $win $xw 1"
     bind $win.righter <ButtonRelease-1>  ":text-adjust-entry $win $xw -1"
     global $win.adjust
     set $win.adjust 0
  }

  if {$_autoreturn == "T"} then {
     bind  $win.entry <FocusIn>  ":text-focus In $win $name"
     bind  $win.entry <FocusOut> ":text-focus Out $win $name"
  }

  $win.entry insert 0 "$vl"
  global $win.string
  set $win.string "$vl"
  $win.label configure -text "$ll"

  if {$deaf == "T"} then {:_deaf $win.entry Entry}
  if {$help == "T"} then {:_help $helpitem $win.entry Entry}

  :_set-font $win.label label $bold $size
  :_set-font $win.entry entry $bold $size

  :_set-color $win frame
  :_set-color $win.label label
  :_set-color $win.entry entry

  if {$vx == 0} then {
    pack $win.label  -side left  -fill y
    if {$shifters == "T"} then {
       pack $win.lefter -side left
       pack $win.righter -side right
    }
    pack $win.entry  -side right -fill y
  } else {
    if {[string match *c $vx] == 0} then {
       set lw [expr $vx/$_charw]
    } else {
       set lw [string trim $vx c]
    }
    $win.label configure -anchor ne -width $lw
    pack $win.label  -side left  -anchor ne -fill y
    if {$shifters == "T"} then {
       pack $win.lefter -side left
       pack $win.righter -side right
    }
    pack $win.entry  -side right -anchor nw -fill y
  }

  :_position-widget $placed $win $xp $yp

  :_post-updateposn $placed $win $xp $yp

}

proc :text-adjust-entry {path width inc} {
    upvar #0 $path.adjust adjust
    incr adjust $inc
    set l [expr [string length [$path.entry get]] - $width + 4]
    if {$adjust > $l} then {set adjust $l}
    if {$adjust < 0}  then {set adjust 0}
    $path.entry xview $adjust
}

set __EntryData ""
set __EntryWin ""

proc :text-event {win name} {
  global frame _commandname __EntryData
  set data "[$win.entry get]"
  set __EntryData "$data"
  if {[info exists _commandname($name)]} then {
     eval $_commandname($name) [list [string toupper $name] "$data"]
  } else {
     click [list $frame "" [string toupper $name] "$data"]
  }
}

proc :text-focus {event win name} {
  global __EntryData __EntryWin
  switch $event {
     In  {
#        puts stdout "focus $event $win $name"
        set __EntryData "[$win.entry get]"
        set __EntryWin $win
     }
     Out {
#        puts stdout "focus $event $win $name"
        set data "[$win.entry get]"
        set __EntryWin ""
        if {"$data" != "$__EntryData"} then {
#           puts stdout "entry data has changed (\"$data\" != \"$__EntryData\") - calling :text-event"
           :text-event $win $name
        }
     }
     default {}
  }
}


proc :modify_text {win name args} {
  global debug __EntryWin __EntryData

#    options -   :value ? :label ? :inactive T|F

  set args [lindex $args 0]
  if {$debug} then {insert-debug ":modify_text called with  $win $name $args"}

  set i 0
  while {$i < [llength $args]} {
    set p [lindex $args $i]
    set vl [lindex $args [expr $i + 1]]
    switch $p {

      :w   {$win.entry configure -width $vl}
      :read-only -
      :deaf {}
      :inactive  {
        switch $vl {
          T {$win.entry configure -state disabled}
          F {$win.entry configure -state normal}
          default {puts stderr ":modify_text  :inactive  - unknown option $v"}
        }
      }
      :value  {
        $win.entry delete 0 end
        $win.entry insert 0 "$vl"
        global $win.string
        set $win.string "$vl"
        if {$__EntryWin == $win} then {set __EntryData "$vl"}
      }
      :label  {
        $win.label configure -text "$vl"
      }
      default {puts stderr ":modify_text - unknown arg $p"}
    }
    set i [expr $i + 2]
  }

}

proc :query_text {win name args} {
  global debug

#    settings -    :w :label :value

  set args [lindex $args 0]
  if {$debug} then {insert-debug ":query_text called with  $win $name $args"}

  switch [set p [lindex $args 0]] {
    :w {return [$win.entry cget -width]}
    :label {return [$win.label cget -text]}
    :value {return [$win.entry get]}

    default {puts stderr ":query_text $win - unknown arg $p"}
  }
}


proc :list args {
##########
##########
  global debug _frame _panel _container _layout _xposn _yposn _placed _pack
  global  _listch _listcw

#    :list name :x ? :y ? :w ? :h ? :rows ? :list-width ? :strings ?
#                 double-click T  deaf T  exclusive T|F

  set placed $_placed
  set xp $_xposn; set yp $_yposn
  set xw 0; set xlw 0; set xh 0; set xrows 0; set xcols 0; set abs 0
  set bold 0; set deaf "F"; set help "T"; set scroll "V"; set font "F"
  set see 0

  set name [string tolower [lindex $args 0]]
  set helpitem [lindex $args 0]

  if {$_container == ""} then {
     if {$_panel == ""} then {
        set win $_frame.$name
     } else {
        set win $_frame.$_panel.$name
     }
  } else {
     if {$_panel == ""} then {
        set win $_frame.$_container.$name
     } else {
        set win $_frame.$_container.$_panel.$name
     }
  }

  global _classname _winname
  set _classname($name) "list"
  set _winname($name)  $win

  :_pre-updateposn;  set xp $_xposn; set yp $_yposn

  frame $win
  if {$debug} then {$win configure -relief raised -bd 1}

  listbox   $win.list  -setgrid true -relief sunken
  bind      $win.list   <ButtonRelease-1> ":list-event $win $name"

  set i 1
  while {$i < [llength $args]} {
    set p [lindex $args $i]
    set v [lindex $args [expr $i + 1]]
    switch $p {

      :x  {set xp $v; set placed 1}
      :y  {set yp $v; set placed 1}
      :w  {set xw $v; set abs 1}
      :h  {set xh $v; set abs 1}
      :rows {set xrows $v}
      :cols {set xcols $v}
      :list-width {set xlw $v; set abs 1}
      :exclusive -
      :browsing {
        switch $v {
          T    {$win.list configure -selectmode browse}
          F    {$win.list configure -selectmode multiple}
          default {puts stderr ":list :browsing - unknown option $v"}
        }
      }
      :select {
        switch $v {
          B    {$win.list configure -selectmode browse}
          E    {$win.list configure -selectmode extended}
          M    {$win.list configure -selectmode multiple}
          default {puts stderr ":list :select - unknown option $v"}
        }
      }
      :font {
        switch $v {
          fixed       {set font "F"}
          variable    {set font "V"}
          default {puts stderr ":list :font - unknown option $v"}
        }
      }
      :scroll {
        switch $v {
          V    {set scroll "V"}
          H    {set scroll "H"}
          B    {set scroll "B"}
          N    {set scroll ""}
          default {puts stderr ":list :scroll - unknown option $v"}
        }
      }
      :show {}
      :see {set see $v}
      :deaf {
        switch $v {
          T    {set deaf "T"}
          F    {set deaf "F"}
          default {puts stderr ":list :deaf - unknown option $v"}
        }
      }
      :help {
        switch $v {
          T    {set help "T"}
          F    {set help "F"}
          default {:help $helpitem $v}
        }
      }
      :helpitem {set helpitem $v}
      :bold {
        switch $v {
          T    {set bold 1}
          F    {set bold 0}
          default {puts stderr ":list :bold - unknown option $v"}
        }
      }
      :double-click {
        switch $v {
          T {bind $win.list <Double-ButtonRelease-1> ":list-event-double $win $name"}
          F {}
          default {puts stderr ":list :double-click - unknown option $v"}
        }
      }
      :strings {
        foreach st $v {$win.list insert end "$st"}
      }
      :command {global _commandname; set _commandname($name) "$v"}
      :after  {set _pack " -after  [:path $v] "}
      :before {set _pack " -before [:path $v] "}

      default {puts stderr ":list - unknown arg $p"}
    }
    set i [expr $i + 2]
  }

  if {$abs == 1} then {
    if {$xw == 0}  then {set xw [expr $xlw + 35]}
    if {$xrows != 0} then {set xh  [expr $xrows * $_listch]}
    if {$xcols != 0} then {set xlw [expr $xcols * $_listcw]}
    if {$xlw == 0} then {set xlw [expr $xw - 35]}
    $win configure -width $xw  -height $xh
  } else {
    $win.list configure -width $xcols -height $xrows
  }

  if {$scroll == "V" | $scroll == "B"} then {
     scrollbar $win.yscroll -command "$win.list yview" -orient vertical
     $win.list  config -yscroll "$win.yscroll set"
  }
  if {$scroll == "H" | $scroll == "B"} then {
     scrollbar $win.xscroll -command "$win.list xview" -orient horizontal
     $win.list  config -xscroll "$win.xscroll set"
  }

  if {$deaf == "T"} then {:_deaf $win.list Listbox}
  if {$help == "T"} then {:_help $helpitem $win.list Listbox}

#    save option for list window
  bind $win.list <Control-KeyPress-S>  "midas-browser W midas-save-window %W"
  bind $win.list <Control-KeyPress-s>  "midas-browser W midas-save-window %W"
#    print option for list window
  bind $win.list <Control-KeyPress-P>  "midas-print-window %W"
  bind $win.list <Control-KeyPress-p>  "midas-print-window %W"

  $win.list see $see

  :_set-font $win.list [set font]listbox $bold N

  :_set-color $win frame
  :_set-color $win.list   listbox
  if {$scroll == "V" | $scroll == "B"} then {
     :_set-color $win.yscroll scrollbar
  }
  if {$scroll == "H" | $scroll == "B"} then {
     :_set-color $win.xscroll scrollbar
  }

  if {$abs == 1} then {
     place $win.list    -height $xh -width $xlw -x 0 -y 0
       if {$scroll == "V" | $scroll == "B"} then {
          place $win.yscroll  -height $xh -x $xlw -y 0
       }
       if {$scroll == "H" | $scroll == "B"} then {
          place $win.xscroll  -height $xlw -x 0 -y 0
       }
  } else {
     if {$scroll == "V" | $scroll == "B"} then {
        pack $win.yscroll -side right -fill y
     }
     if {$scroll == "H" | $scroll == "B"} then {
        pack $win.xscroll -side bottom -fill x
     }
     pack $win.list   -expand yes -fill both
  }

  :_position-widget $placed $win $xp $yp

  :_post-updateposn $placed $win $xp $yp

}

set __listEventA ""
set __listEventB ""

proc :list-event {win name} {
  global frame __listEventA __listEventB
  global _commandname
#insert-debug "event  [$win.list curselection] "
  set s [$win.list curselection]
  if {$s != ""} then {
    set __listEventA [selection get]
    set __listEventB $s
    if {[info exists _commandname($name)]} then {
      eval $_commandname($name) [list [string toupper $name] $__listEventA $__listEventB]
    } else {
      click [list $frame :list-select [string toupper $name] $__listEventA $__listEventB]
    }
  } else {
    if {[info exists _commandname($name)]} then {
      eval $_commandname($name) [list [string toupper $name] {} {}]
    } else {
      click [list $frame :list-select [string toupper $name] {} {}]
    }
  }
}

proc :list-event-double {win name} {
  global frame __listEventA __listEventB
#insert-debug "doubleevent  [$win.list curselection]"
  if {$__listEventB == ""} then {:list-event $win $name}
  double-click [list $frame :list-select [string toupper $name] $__listEventA $__listEventB ]
}

proc :modify_list {win name args} {
  global debug

#    options -   :append T|F :empty T|F :show T|F :strings ?? :select ? :deselect ? :inactive T|F :see ?

  set args [lindex $args 0]
  if {$debug} then {insert-debug ":modify_list called with  $win $name $args"}

  set i 0
  while {$i < [llength $args]} {
    set p [lindex $args $i]
    set v [lindex $args [expr $i + 1]]
    switch $p {

      :strings {
        foreach st $v {$win.list insert end "$st"}
        $win.list yview end
      }
      :rows {$win.list configure -height $v}
      :cols {$win.list configure -width  $v}
      :select {
        if {$v < 0} then {
          $win.list selection clear 0 end
        } else {
          eval $win.list selection set $v
        }
      }
      :deselect {
        if {$v < 0} then {
          $win.list selection clear 0 end
        } else {
          $win.list selection clear $v $v
        }
      }
      :empty {
        switch $v {
          T {$win.list delete 0 end}
          F {}
          default {puts stderr ":modify_list  :empty  - unknown option $v"}
        }
      }
      :append {
        switch $v {
          T {}
          F {$win.list delete 0 end}
          default {puts stderr ":modify_list  :append  - unknown option $v"}
        }
      }
      :show {
        switch $v {
          T {}
          F {}
          default {puts stderr ":modify_list  :show  - unknown option $v"}
        }
      }
      :see {$win.list see $v}

      default {puts stderr ":modify_list - unknown arg $p"}
    }
    set i [expr $i + 2]
  }

}

proc :query_list {win name args} {
  global debug

#    settings -    :strings (all indices) :selected (selected indices) :selstrings (selected strings)

  set args [lindex $args 0]
  if {$debug} then {insert-debug ":query_list called with  $win $name $args"}

  switch [set p [lindex $args 0]] {
    :strings  {return [$win.list get 0 end]}
    :selected {
      set sz [$win.list size]
      set l {}
      for {set i 0} {$i<$sz} {incr i} {
        if {[$win.list selection includes $i]} {lappend l $i}
      }
      return $l
    }
    :selstrings {
      set sz [$win.list size]
      set l {}
      for {set i 0} {$i<$sz} {incr i} {
        if {[$win.list selection includes $i]} {
          set s ""; foreach m [$win.list get $i $i] {append s "$m "}
          lappend l [string trim $s]
        }
      }
      return $l
    }
    default {puts stderr ":query_list $win - unknown arg $p"}
  }
}


proc :galley args {
############
############
  global debug _frame _panel _container _layout _xposn _yposn _placed _pack _expand

#    :galley name :x ? :y ? :rows ? :cols ? :strings ?

  set placed $_placed
  set xp $_xposn; set yp $_yposn
  set xrows 0; set xcols 0
  set bold 0;set size N;set see 0; set deaf "F";set help T; set scroll "V"; set font "V"

  set name [string tolower [lindex $args 0]]
  set helpitem [lindex $args 0]

  if {$_container == ""} then {
     if {$_panel == ""} then {
        set win $_frame.$name
     } else {
        set win $_frame.$_panel.$name
     }
  } else {
     if {$_panel == ""} then {
        set win $_frame.$_container.$name
     } else {
        set win $_frame.$_container.$_panel.$name
     }
  }

  global _classname _winname
  set _classname($name) "galley"
  set _winname($name)  $win

  :_pre-updateposn;  set xp $_xposn; set yp $_yposn

  frame $win
  if {$debug} then {$win configure -relief raised -bd 1}

  text   $win.text  -wrap word -relief sunken

  if {$debug} then {insert-debug ":galley created $win $win.scroll and $win.text"}

  set i 1
  while {$i < [llength $args]} {
    set p [lindex $args $i]
    set v [lindex $args [expr $i + 1]]
    switch $p {

      :x  {set xp $v; set placed 1}
      :y  {set yp $v; set placed 1}
      :rows {set xrows $v}
      :cols {set xcols $v}
      :show {}
      :see  {set see $v}
      :font {
        switch $v {
          fixed       {set font "F"}
          variable    {set font "V"}
          default {puts stderr ":list :font - unknown option $v"}
        }
      }
      :scroll {
        switch $v {
          V    {set scroll "V"}
          H    {set scroll "H"}
          B    {set scroll "B"}
          N    {set scroll ""}
          default {puts stderr ":list :scroll - unknown option $v"}
        }
      }
      :inactive  {
        switch $v {
          T {$win.text configure -state disabled}
          F {$win.text configure -state normal}
          default {puts stderr ":galley  :inactive  - unknown option $v"}
        }
      }
      :deaf {
        switch $v {
          T    {set deaf "T"}
          F    {set deaf "F"}
          default {puts stderr ":galley :deaf - unknown option $v"}
        }
      }
      :bold {
        switch $v {
          T    {set bold 1}
          F    {set bold 0}
          default {puts stderr ":galley :bold - unknown option $v"}
        }
      }
      :size {
        switch $v {
          N    -
          S    -
          L    {set size $v}
          default {puts stderr ":text :size - unknown option $v"}
        }
      }
      :help {
        switch $v {
          T    {set help "T"}
          F    {set help "F"}
          default {:help $helpitem $v}
        }
      }
      :helpitem {set helpitem $v}
      :strings {$win.text insert end "$v"}
      :after  {set _pack " -after  [:path $v] "}
      :before {set _pack " -before [:path $v] "}

      default {puts stderr ":galley - unknown arg $p"}
    }
    set i [expr $i + 2]
  }

  if {$xcols != 0} then {$win.text configure -width  $xcols}
  if {$xrows != 0} then {$win.text configure -height $xrows}

  if {$scroll == "V" | $scroll == "B"} then {
     scrollbar $win.yscroll -command "$win.text yview" -orient vertical
     $win.text  config -yscroll "$win.yscroll set"
  }
  if {$scroll == "H" | $scroll == "B"} then {
     scrollbar $win.xscroll -command "$win.text xview" -orient horizontal
     $win.text  config -xscroll "$win.xscroll set"
  }

  if {$deaf == "T"} then {:_deaf $win.text Text}
  if {$help == "T"} then {:_help $helpitem $win Text}

  $win.text yview $see

#    save option for text window
  bind $win.text <Control-KeyPress-S>  "midas-browser W midas-save-window %W"
  bind $win.text <Control-KeyPress-s>  "midas-browser W midas-save-window %W"
#    print option for text window
  bind $win.text <Control-KeyPress-P>  "midas-print-window %W"
  bind $win.text <Control-KeyPress-p>  "midas-print-window %W"

  :_set-font $win.text [set font]text $bold $size

  :_set-color $win frame
  :_set-color $win.text   text
  if {$scroll == "V" | $scroll == "B"} then {
     :_set-color $win.yscroll scrollbar
  }
  if {$scroll == "H" | $scroll == "B"} then {
     :_set-color $win.xscroll scrollbar
  }

  if {$scroll == "V" | $scroll == "B"} then {
     pack $win.yscroll -side right -fill y
  }
  if {$scroll == "H" | $scroll == "B"} then {
     pack $win.xscroll -side bottom -fill x
  }
  pack $win.text    -expand $_expand -fill both

  :_position-widget $placed $win $xp $yp

  :_post-updateposn $placed $win $xp $yp

}


proc :modify_galley {win name args} {
  global debug

#    options -   :append T|F :empty T|F :show T|F :strings ?? :inactive T|F

  set tag ""
  set args [lindex $args 0]
  if {$debug} then {insert-debug ":modify_galley called with  $win $name $args"}

  set i 0
  while {$i < [llength $args]} {
    set p [lindex $args $i]
    set v [lindex $args [expr $i + 1]]
    switch $p {

      :strings {
        $win.text insert end "$v" $tag
        $win.text yview end
      }
      :rows {$win.text configure -height $v}
      :cols {$win.text configure -width  $v}
      :usingtag  {set tag $v}
      :createtag {eval $win.text tag configure $v}
      :deletetag {eval $win.text tag delete $v}
      :empty {
        switch $v {
          T {$win.text delete 1.0 end}
          F {}
          default {puts stderr ":modify_galley  :empty  - unknown option $v"}
        }
      }
      :append {
        switch $v {
          T {}
          F {$win.text delete 1.0 end}
          default {puts stderr ":modify_galley  :append  - unknown option $v"}
        }
      }
      :show {
        switch $v {
          T {}
          F {}
          default {puts stderr ":modify_galley  :show  - unknown option $v"}
        }
      }
      :see  {$win.text yview $v}
      :deaf {}
      :inactive  {
        switch $v {
          T {
            $win.text configure -state disabled
          }
          F {
            $win.text configure -state normal
          }
          default {puts stderr ":modify_galley  :inactive  - unknown option $v"}
        }
      }
      default {puts stderr ":modify_galley - unknown arg $p"}
    }
    set i [expr $i + 2]
  }

}

proc :query_galley {win name args} {
  global debug

#    settings -    :strings

  set args [lindex $args 0]
  if {$debug} then {insert-debug ":query_galley called with  $win $name $args"}

  switch [set p [lindex $args 0]] {
    :strings  {return [$win.text get 0 end]}

    default {puts stderr ":query_galley $win - unknown arg $p"}
  }
}


proc :message args {
#############
#############
  global debug _frame _panel _container _layout _placed _anchor _pack _charw _charh
  global _xposn _yposn _colgap _rowgap

#    :message name :x ? :y ? :w ? :valuex ? :label ? :bold T/F

  set placed $_placed
  set vx 0
  set font "message"; set colour "message"; set anchor "ne"
  set bold 0

  set name [string tolower [lindex $args 0]]

  if {$_container == ""} then {
     if {$_panel == ""} then {
        set win $_frame.$name
     } else {
        set win $_frame.$_panel.$name
     }
  } else {
     if {$_panel == ""} then {
        set win $_frame.$_container.$name
     } else {
        set win $_frame.$_container.$_panel.$name
     }
  }

  global _classname _winname
  set _classname($name) "message"
  set _winname($name)  $win

  :_pre-updateposn;  set xp $_xposn; set yp $_yposn

  frame $win -relief flat
  if {$debug} then {$win configure -relief raised -bd 1}

  message $win.msg -aspect 10000
  if {$debug} then {insert-debug ":message created $win and $win.msg"}

  set i 1
  while {$i < [llength $args]} {
    set p [lindex $args $i]
    set v [lindex $args [expr $i + 1]]
    switch $p {

      :x  {set xp $v; set placed 1}
      :y  {set yp $v; set placed 1}
      :w  {set vx [set v]c}
      :valuex {set vx $v}
      :label  {$win.msg configure -text "$v"}
      :justify {$win.msg configure -justify $v}
      :font   {set font "$v"}
      :colour {set colour "$v"}
      :anchor {set anchor "$v"}
      :show {}
      :bold  {
        switch $v {
          T {set bold 1}
          F {set bold 0}
          default {puts stderr ":message :bold - unknown option $v"}
        }
      }
      :after  {set _pack " -after  [:path $v] "}
      :before {set _pack " -before [:path $v] "}

      default {puts stderr ":message - unknown arg $p"}
    }
    set i [expr $i + 2]
  }

  :_set-font $win.msg $font $bold N

  :_set-color $win frame
  :_set-color $win.msg $colour
  
  if {$vx == 0} then {
    pack $win.msg  -side left  -fill y
  } else {
    if {[string match *c $vx] == 0} then {
       set mw $vx
    } else {
       set mw [expr [string trim $vx c] * $_charw]
    }
    $win configure -width $mw -height $_charh
    if {$anchor == "ne"} then {
       place $win.msg -x $mw -y 0 -anchor $anchor
    } else {
       $win.msg configure -width $mw
       place $win.msg -x 0 -y 0 -anchor $anchor
    }
  }

  :_position-widget $placed $win $xp $yp

  :_post-updateposn $placed $win $xp $yp

}

proc :modify_message {win name args} {
  global debug

#    options -   :label ?  :bold ?

  set args [lindex $args 0]
  if {$debug} then {insert-debug ":modify_message called with  $win $name $args"}

  set i 0
  while {$i < [llength $args]} {
    set p [lindex $args $i]
    set v [lindex $args [expr $i + 1]]
    switch $p {

      :label  {$win.msg configure -text "$v"}
      :font   {:_set-font $win.msg "$v" -1 N}
      :colour {:_set-color $win.msg "$v"}
      :bold  {
        switch $v {
          T  {:_set-font $win.msg message 1 N}
          F  {:_set-font $win.msg message 0 N}
        }
      }

      default {puts stderr ":modify_message - unknown arg $p"}
    }
    set i [expr $i + 2]
  }

}

proc :query_message {win name args} {
  global debug

#    settings -    :label

  set args [lindex $args 0]
  if {$debug} then {insert-debug ":query_message called with  $win $name $args"}

  switch [set p [lindex $args 0]] {
    :label {return [$win.msg cget -text]}

    default {puts stderr ":query_message $win - unknown arg $p"}
  }
}


proc :checkbox args {
##############
##############
  global debug env _frame _panel _container _layout _placed _pack
  global  _xposn _yposn _colgap _rowgap _charh _charw

#    :checkbox name :x ? :y ? :valuex ? :label ? :value ? :deaf T/F :show T|F

  set placed $_placed
  set xp $_xposn; set yp $_yposn; set vx 0; set vl 0; set xh 0; set xw 0
  set bold 0; set deaf "F"; set help "T"; set colours ""; set blayout "right"; set llayout "left"

  set name [string tolower [lindex $args 0]]
  set helpitem [lindex $args 0]
  set ll [string toupper [string range $name 0 0]]
  append ll [string range $name 1 end]

  if {$_container == ""} then {
     if {$_panel == ""} then {
        set win $_frame.$name
     } else {
        set win $_frame.$_panel.$name
     }
  } else {
     if {$_panel == ""} then {
        set win $_frame.$_container.$name
     } else {
        set win $_frame.$_container.$_panel.$name
     }
  }

  global _classname _winname
  set _classname($name) "checkbox"
  set _winname($name)  $win

  :_pre-updateposn;  set xp $_xposn; set yp $_yposn

  frame $win -relief flat
  if {$debug} then {$win configure -relief raised -bd 1}

  label $win.label
  global $name.value
  checkbutton  $win.button -variable $name.value  \
        -command ":checkbox-event $win $name"
  $win.button configure -indicatoron false

  if {$debug} then {insert-debug ":checkbox created $win $win.label and $win.button"}

  set i 1
  while {$i < [llength $args]} {
    set p [lindex $args $i]
    set v [lindex $args [expr $i + 1]]
    switch $p {
      :x  {set xp $v; set placed 1}
      :y  {set yp $v; set placed 1}
      :w  {set xw $v}
      :h  {set xh $v}
      :value  {set vl $v}
      :valuex {set vx $v}
      :label  {set ll "$v"}
      :deaf {
        switch $v {
          T    {set deaf "T"}
          F    {set deaf "F"}
          default {puts stderr ":choice :checkbox - unknown option $v"}
        }
      }
      :inactive {
        switch $v {
          T {$win.button configure -state disabled}
          F {$win.button configure -state normal}
          default {puts stderr ":checkbox :inactive - unknown option $v"}
        }
      }
      :bold {
        switch $v {
          T    {set bold 1}
          F    {set bold 0}
          default {puts stderr ":checkbox :bold - unknown option $v"}
        }
      }
      :layout {
        switch $v {
          L    {set blayout "left";  set llayout "right"}
          R    {set blayout "right"; set llayout "left"}
          default {puts stderr ":checkbox :layout - unknown option $v"}
        }
      }
      :help {
        switch $v {
          T    {set help "T"}
          F    {set help "F"}
          default {:help $helpitem $v}
        }
      }
      :helpitem {set helpitem $v}
      :double-click {}
      :show  {
        switch $v {
          T {}
          F {}
          default {puts stderr ":checkbox :show - unknown option $v"}
        }
      }
      :colours {set colours $v}
      :after  {set _pack " -after  [:path $v] "}
      :before {set _pack " -before [:path $v] "}
      :command {global _commandname; set _commandname($name) "$v"}

      default {puts stderr ":checkbox - unknown arg $p"}
    }
    set i [expr $i + 2]
  }

  if {$xw != 0} then {$win.button configure -width $xw}
  if {$xh != 0} then {$win.button configure -height $xh}
  $win.label configure -text "$ll"

  :_set-font $win.label label $bold N

  if {$vl == 0} then {
    $win.button deselect
    $win.button configure -image tickoff
  } else {
    $win.button select
    $win.button configure -image tickon
  }

  if {$vx == 0} then {
    pack $win.button -side $blayout -fill y
    pack $win.label  -side $llayout  -fill y
  } else {
    if {[string match *c $vx] == 0} then {
       set lw [expr $vx/$_charw]
    } else {
       set lw [string trim $vx c]
    }
    $win.label configure -anchor ne -width $lw
    pack $win.button  -side $blayout -anchor nw -fill y
    pack $win.label   -side $llayout  -anchor ne -fill y
  }

  if {$deaf == "T"} then {:_deaf $win.button Checkbutton}
  if {$help == "T"} then {:_help $helpitem $win Checkbutton}

  :_set-color $win frame
  :_set-color $win.button checkbutton
  if {$colours != ""} then {
     if {[first $colours] != ""} then {$win.button configure -background [first $colours]}
     if {[second $colours] != ""} then {$win.button configure -selectcolor [second $colours]}
  }
  :_set-color $win.label  label

  :_position-widget $placed $win $xp $yp

  :_post-updateposn $placed $win $xp $yp

}


proc :checkbox-event {win name} {
  global frame _commandname $name.value

  set vl [set $name.value]
  if {$vl == 0} then {
    $win.button configure -image tickoff
  } else {
    $win.button configure -image tickon
  }

  if {[info exists _commandname($name)]} then {
     eval $_commandname($name) [list [string toupper $name] $vl]
  } else {
     click [list $frame "" [string toupper $name] [set $name.value]]
  }
}


proc :modify_checkbox {win name args} {
  global debug

#    options -   :value ? :inactive F|T

  set args [lindex $args 0]
  if {$debug} then {insert-debug ":modify_checkbox called with  $win $name $args"}

  set i 0
  while {$i < [llength $args]} {
    set p [lindex $args $i]
    set vl [lindex $args [expr $i + 1]]
    switch $p {

      :deaf {}

      :colours {
        if {[first $vl] != ""} then {$win.button configure -background [first $vl]}
        if {[second $vl] != ""} then {$win.button configure -selectcolor [second $vl]}
      }
      :inactive  {
        switch $vl {
          T {$win.button configure -state disabled}
          F {$win.button configure -state normal}
          default {puts stderr ":modify_checkbox  :inactive  - unknown option $v"}
        }
      }
      :value  {
        if {$vl == 0} then {
          $win.button deselect
          $win.button configure -image tickoff
        } else {
          $win.button select
          $win.button configure -image tickon
        }
      }
      default {puts stderr ":modify_checkbox - unknown arg $p"}
    }
    set i [expr $i + 2]
  }

}

proc :query_checkbox {win name args} {
  global debug $name.value

#    settings -    :label :value

  set args [lindex $args 0]
  if {$debug} then {insert-debug ":query_checkbox called with  $win $name $args"}

  switch [set p [lindex $args 0]] {
    :label {return [$win.label cget -text]}
    :value {return [if {[$win.button cget -image] == "tickoff"} {set b 0} {set b 1}]}
    default {puts stderr ":query_checkbox $win - unknown arg $p"}
  }
}


proc :number args {
############
############
  global debug _frame _panel _container _layout _xposn _yposn _placed _pack _colgap _rowgap _charh _charw

#    :number name :x ? :y ? :w ? :h ? :valuex ? :label ? :value ? :min ? :max ? :inc ? :deaf T/F

  set placed $_placed
  set width 0
  set xp $_xposn; set yp $_yposn
  set xw 0; set xh 0; set vx 0; set vl 0
  set bold 0; set deaf "F"; set help "T"

  set name [string tolower [lindex $args 0]]
  set helpitem [lindex $args 0]
  set ll [string toupper [string range $name 0 0]]
  append ll [string range $name 1 end]

  if {$_container == ""} then {
     if {$_panel == ""} then {
        set win $_frame.$name
     } else {
        set win $_frame.$_panel.$name
     }
  } else {
     if {$_panel == ""} then {
        set win $_frame.$_container.$name
     } else {
        set win $_frame.$_container.$_panel.$name
     }
  }

  global _classname _winname
  set _classname($name) "number"
  set _winname($name)  $win

  :_pre-updateposn;  set xp $_xposn; set yp $_yposn

  frame $win -relief flat
  if {$debug} then {$win configure -relief raised -bd 1}

  global $win.max $win.min
  set $win.max 100000; set $win.min -100000

  label $win.label
  entry $win.entry -relief sunken

  bind  $win.entry <Return>   ":number-event-entry $win $name"


  global _default_autoreturn
  set _autoreturn $_default_autoreturn

  global $win.value

  checkbutton $win.button-up -variable $win.button  \
    -command ":number-event-button $win $name +1"
  checkbutton $win.button-dn -variable $win.button  \
    -command ":number-event-button $win $name -1"

  $win.button-up configure -image inc  -selectimage inc -indicatoron 0
  $win.button-dn configure -image dec  -selectimage dec -indicatoron 0
  $win.button-up deselect
  $win.button-dn deselect

  if {$debug} then {insert-debug \
     ":number created $win $win.label $win.entry $win.button-up $win.button-dn"}

  set i 1
  while {$i < [llength $args]} {
    set p [lindex $args $i]
    set v [lindex $args [expr $i + 1]]
    switch $p {

      :x  {set xp $v; set placed 1}
      :y  {set yp $v; set placed 1}
      :h  {set xh $v}
      :w  {set xw $v}
      :valuex {set vx $v}
      :label  {set ll "$v"}
      :value  {set vl $v}
      :min {set $win.min $v}
      :max {set $win.max $v}
      :inc {$win.button-up configure -command ":number-event-button $win $name +$v"
            $win.button-dn configure -command ":number-event-button $win $name -$v"}
      :show {}
      :deaf {
        switch $v {
          T    {set deaf "T"}
          F    {set deaf "F"}
          default {puts stderr ":number :deaf - unknown option $v"}
        }
      }
      :auto {
        switch $v {
          T    {set _autoreturn "T"}
          F    {set _autoreturn "F"}
          default {puts stderr ":number :auto - unknown option $v"}
        }
      }
      :help {
        switch $v {
          T    {set help "T"}
          F    {set help "F"}
          default {:help $helpitem $v}
        }
      }
      :helpitem {set helpitem $v}
      :inactive {
        switch $v {
          T {
            $win.entry configure -state disabled
            $win.button-up configure -state disabled
            $win.button-dn configure -state disabled
          }
          F {
            $win.entry configure -state normal
            $win.button-up configure -state normal
            $win.button-dn configure -state normal
          }
          default {puts stderr ":number :inactive - unknown option $v"}
        }
      }
      :bold {
        switch $v {
          T    {set bold 1}
          F    {set bold 0}
          default {puts stderr ":number :bold - unknown option $v"}
        }
      }
      :double-click {}
      :after  {set _pack " -after  [:path $v] "}
      :before {set _pack " -before [:path $v] "}

      :command {global _commandname; set _commandname($name) "$v"}


      default {puts stderr ":number - unknown arg $p"}
    }
    set i [expr $i + 2]
  }

  if {$vl < [set $win.min]} then {set vl [set $win.min]}

  if {$_autoreturn == "T"} then {
    bind  $win.entry <FocusIn>  ":number-focus In $win $name"
    bind  $win.entry <FocusOut> ":number-focus Out $win $name"
  }

  $win.entry configure -width $xw
  $win.entry delete 0 end
  $win.entry insert 0 "$vl"
  set $win.value $vl
  $win.label configure -text "$ll"

  if {$deaf == "T"} then {:_deaf $win.entry Entry}
  if {$deaf == "T"} then {:_deaf $win.button-up Checkbutton}
  if {$deaf == "T"} then {:_deaf $win.button-dn Checkbutton}

  :_set-font $win.label label $bold N
  :_set-font $win.entry entry $bold N

  :_set-color $win frame
  :_set-color $win.label  label
  :_set-color $win.entry  entry
  :_set-color $win.button-up checkbutton
  :_set-color $win.button-dn checkbutton

  if {$vx == 0} then {
    pack $win.button-dn $win.button-up -side right -fill y
    pack $win.entry -side right -fill y
    pack $win.label -side left  -fill y
  } else {
    if {[string match *c $vx] == 0} then {
       set lw [expr $vx/$_charw]
    } else {
       set lw [string trim $vx c]
    }
    $win.label configure -anchor ne -width $lw
    pack $win.button-dn $win.button-up -side right -fill y
    pack $win.entry  -side right -anchor nw -fill y
    pack $win.label  -side left  -anchor ne -fill y
  }

  if {$help == "T"} then {:_help $helpitem $win Entry}

  :_position-widget $placed $win $xp $yp

  :_post-updateposn $placed $win $xp $yp

}

proc :number-event-button  {win name inc} {
  global frame $win.value $win.min $win.max

  set value [expr [set $win.value] $inc]
  if {$value > [set $win.max]} {return}
  if {$value < [set $win.min]} {return}
  set $win.value $value
  $win.entry delete 0 end
  $win.entry insert 0 $value
  $win.button-up deselect
  $win.button-dn deselect
  global _commandname
  if {[info exists _commandname($name)]} then {eval $_commandname($name) [list [string toupper $name] $value]} else \
  {click [list $frame "" [string toupper $name] $value]}
}

proc :number-event-entry {win name} {
  global frame $win.value $win.min $win.max __EntryData

  set value [$win.entry get]
  set __EntryData "$value"

  if {$value > [set $win.max]} {return}
  if {$value < [set $win.min]} {return}
  set $win.value $value
  global _commandname
  if {[info exists _commandname($name)]} then {eval $_commandname($name) [list [string toupper $name] $value]} else \
  {click [list $frame "" [string toupper $name] $value]}
}

proc :number-focus {event win name} {
  global __EntryData __EntryWin
  switch $event {
     In  {
#        puts stdout "focus $event $win $name"
        set __EntryData "[$win.entry get]"
        set __EntryWin $win
     }
     Out {
#        puts stdout "focus $event $win $name"
        set __EntryWin ""
        set data "[$win.entry get]"
        if {"$data" != "$__EntryData"} then {
#           puts stdout "entry data has changed (\"$data\" != \"$__EntryData\") - calling :number-event-entry"
           :number-event-entry $win $name
        }
     }
     default {}
  }
}

proc :modify_number {win name args} {
  global debug __EntryData __EntryWin
  global $win.min $win.max

#    options -   :value ? :inactive F|T

  set args [lindex $args 0]
  if {$debug} then {insert-debug ":modify_number called with  $win $name $args"}

  set i 0
  while {$i < [llength $args]} {
    set p [lindex $args $i]
    set vl [lindex $args [expr $i + 1]]
    switch $p {

      :deaf {}
      :inactive  {
        switch $vl {
          T {
            $win.entry configure -state disabled
            $win.button-up configure -state disabled
            $win.button-dn configure -state disabled
          }
          F {
            $win.entry configure -state normal
            $win.button-up configure -state normal
            $win.button-dn configure -state normal
          }
          default {puts stderr ":modify_number  :inactive  - unknown option $v"}
        }
      }
      :min   {set $win.min $vl}
      :max   {set $win.max $vl}
      :label {$win.label configure -text "$vl"}
      :value  {
        $win.entry delete 0 end
        $win.entry insert 0  "[expr $vl]"
        global $win.value; set $win.value $vl
        if {$__EntryWin == $win} then {set __EntryData $vl}
      }
      default {puts stderr ":modify_number - unknown arg $p"}
    }
    set i [expr $i + 2]
  }

}

proc :query_number {win name args} {
  global debug $win.min $win.max $win.value

#    settings -    :label :min :max :value

  set args [lindex $args 0]
  if {$debug} then {insert-debug ":query_number called with  $win $name $args"}

  switch [set p [lindex $args 0]] {
    :label {return [$win.label cget -text]}
    :min   {return [set $win.min]}
    :max   {return [set $win.max]}
    :value {return [set $win.value]}

    default {puts stderr ":query_number $win - unknown arg $p"}
  }
}


proc :gauge args {
###########
###########
  global frame
  global debug _frame _panel _container _layout _xposn _yposn _placed _pack _colgap _rowgap
  global _charh _charw

#    :gauge name :x ? :y ? :w ? :h ? :valuex ? :label ? :min ? :max ? :value ?
#          :ticks ? ":colour ?

  set placed $_placed
  set width 0
  set bold 0; set gw 80; set gh 0
  set xp $_xposn; set yp $_yposn
  set xw 0; set xh 0; set vx 0; set min 0; set max 0; set vl "0"
  set inactive "F"; set deaf "F"; set help "T"
  set maxlabel ""; set minlabel ""

  set name [string tolower [lindex $args 0]]
  set helpitem [lindex $args 0]
  set ll [string toupper [string range $name 0 0]]
  append ll [string range $name 1 end]

  if {$_container == ""} then {
     if {$_panel == ""} then {
        set win $_frame.$name
     } else {
        set win $_frame.$_panel.$name
     }
  } else {
     if {$_panel == ""} then {
        set win $_frame.$_container.$name
     } else {
        set win $_frame.$_container.$_panel.$name
     }
  }

  global _classname _winname
  set _classname($name) "gauge"
  set _winname($name)  $win

  :_pre-updateposn;  set xp $_xposn; set yp $_yposn

  frame $win -relief flat
  if {$debug} then {$win configure -relief raised -bd 1}

  set i 1
  while {$i < [llength $args]} {
    set p [lindex $args $i]
    set v [lindex $args [expr $i + 1]]
    switch $p {

      :x  {set xp $v; set placed 1}
      :y  {set yp $v; set placed 1}
      :h  {set xh $v}
      :w  {set xw $v}
      :valuex {set vx $v}
      :label  {set ll "$v"}
      :value  {set vl $v}
      :min {
          set min $v
          if {$minlabel == ""} then {set minlabel "$min"}
      }
      :max {
          set max $v
          if {$maxlabel == ""} then {set maxlabel "$max"}
      }
      :max-tick-label {set maxlabel "$v"}
      :min-tick-label {set minlabel "$v"}
      :gauge-width {set gw $v}
      :gauge-height {set gh $v}
      :ticks {}
      :show {}
      :deaf {
        switch $v {
          T    {set deaf "T"}
          F    {set deaf "F"}
          default {puts stderr ":gauge :deaf - unknown option $v"}
        }
      }
      :help {
        switch $v {
          T    {set help "T"}
          F    {set help "F"}
          default {:help $helpitem $v}
        }
      }
      :helpitem {set helpitem $v}
      :inactive {
        switch $v {
          T    {set inactive "T"}
          F    {set inactive "F"}
          default {puts stderr ":gauge :inactive - unknown option $v"}
        }
      }
      :bold {
        switch $v {
          T    {set bold 1}
          F    {set bold 0}
          default {puts stderr ":gauge :bold - unknown option $v"}
        }
      }
      :after  {set _pack " -after  [:path $v] "}
      :before {set _pack " -before [:path $v] "}

      default {puts stderr ":gauge - unknown arg $p"}
    }
    set i [expr $i + 2]
  }

  if {$ll != ""} then {
    label $win.label -text "$ll"

    if {$vx == 0} then {
       pack $win.label  -side left  -fill y
    } else {
       if {[string match *c $vx] == 0} then {
          set lw [expr $vx/$_charw]
       } else {
          set lw [string trim $vx c]
       }
       $win.label configure -anchor ne -width $lw
       pack $win.label  -side left  -anchor ne -fill y
    }

    :_set-font $win.label label $bold N

    :_set-color $win.label label
  }

  if {$gh == 0} then {set gh [expr $gw / 10]}

  if {$minlabel != ""} then {
     label $win.min -text "$minlabel"
     :_set-font $win.min label $bold N
  }

  global $name.value
  scale $win.scale -variable $name.value
  $win.scale configure -relief flat -orient horizontal -showvalue 0 \
     -sliderlength 5 -length $gw -width $gh -highlightthickness 0
  if {$inactive == "T"} then {$win.scale configure -state disabled}
  $win.scale configure -from $min -to $max
  set $name.value $vl

  if {$deaf == "T"} then {:_deaf $win.scale Scale}
  if {$help == "T"} then {:_help $helpitem $win Scale}

  if {$maxlabel != ""} then {
     label $win.max -text "$maxlabel"
     :_set-font $win.max label $bold N
  }

  pack $win.scale -side top -fill x
  if {$minlabel != ""} then {pack $win.min   -side left}
  if {$maxlabel != ""} then {pack $win.max   -side right}

  if {$debug} then {insert-debug ":gauge created $win.scale $win.min $win.max"}

  :_set-color $win frame
  if {$minlabel != ""} then {:_set-color $win.min label}
  if {$maxlabel != ""} then {:_set-color $win.max label}
  :_set-color $win.scale scale

  :_position-widget $placed $win $xp $yp

  :_post-updateposn $placed $win $xp $yp

}


proc :modify_gauge {win name args} {
  global debug

#    options -   :value ? :min ? :max ? :inactive T|F

  set args [lindex $args 0]
  if {$debug} then {insert-debug ":modify_gauge called with  $win $name $args"}

  set i 0
  while {$i < [llength $args]} {
    set p [lindex $args $i]
    set vl [lindex $args [expr $i + 1]]
    switch $p {

      :deaf {}
      :inactive  {
        switch $vl {
          T {$win.scale configure -state disabled}
          F {$win.scale configure -state normal}
          default {puts stderr ":modify_gauge  :inactive  - unknown option $vl"}
        }
      }
      :label {
         $win.label configure -text "$vl"
      }
      :min {
         catch {$win.min configure -text "$vl"}
         $win.scale configure -from $vl
      }
      :max {
         catch {$win.max configure -text "$vl"}
         $win.scale configure -to $vl
      }
      :max-tick-label {
         $win.max configure -text "$vl"
      }
      :value  {
        global $name.value
        set $name.value [expr $vl]
      }
      default {puts stderr ":modify_gauge - unknown arg $p"}
    }
    set i [expr $i + 2]
  }

}

proc :query_gauge {win name args} {
  global debug $name.value

#    settings -    :label :min :max :min-tick-label :max-tick-label :value

  set args [lindex $args 0]
  if {$debug} then {insert-debug ":query_gauge called with  $win $name $args"}

  switch [set p [lindex $args 0]] {
    :label {return [$win.label cget -text]}
    :min   {return [$win.scale cget -from]}
    :max   {return [$win.scale cget -to]}
    :min-tick-label   {return [$win.min cget -text]}
    :max-tick-label   {return [$win.max cget -text]}
    :value {return [set $name.value]}


    default {puts stderr ":query_gauge $win - unknown arg $p"}
  }
}

set _default_resolution 1

proc :slider args {
############
############
  global frame
  global debug _frame _panel _container _layout _xposn _yposn _placed _pack _colgap _rowgap _charh _charw

#    :slider name :x ? :y ? :w ? :h ? :valuex ? :label ? :min ? :max ? :value ?

  set placed $_placed
  set width 0
  set xp $_xposn; set yp $_yposn
  set xw 0; set xh 0; set vx 0; set min 0; set max 0; set vl "0"; set ticks F
  set bold 0; set inactive "F"; set deaf "F"; set help "T"
  set orient "horizontal"; set sl 100; set sw 15; set slw 10; set ew 6; set lmw 0
  set steps 0

  global _default_resolution
  set resolution $_default_resolution

  set name [string tolower [lindex $args 0]]
  set helpitem [lindex $args 0]
  set ll [string toupper [string range $name 0 0]]
  append ll [string range $name 1 end]

  if {$_container == ""} then {
     if {$_panel == ""} then {
        set win $_frame.$name
     } else {
        set win $_frame.$_panel.$name
     }
  } else {
     if {$_panel == ""} then {
        set win $_frame.$_container.$name
     } else {
        set win $_frame.$_container.$_panel.$name
     }
  }

  global _classname _winname
  set _classname($name) "slider"
  set _winname($name)  $win

  :_pre-updateposn;  set xp $_xposn; set yp $_yposn

  frame $win -relief flat
  if {$debug} then {$win configure -relief raised -bd 1}

  global _default_autoreturn
  set _autoreturn $_default_autoreturn

  set i 1
  while {$i < [llength $args]} {
    set p [lindex $args $i]
    set v [lindex $args [expr $i + 1]]
    switch $p {

      :x  {set xp $v; set placed 1}
      :y  {set yp $v; set placed 1}
      :h  {set xh $v}
      :w  {set xw $v}
      :valuex {set vx $v}
      :label  {set ll $v}
      :value  {set vl $v}
      :min {set min $v}
      :max {set max $v}
      :resolution {set resolution $v}
      :steps {set steps $v}
      :limit-width {set lmw $v}
      :entry-width {set ew $v}
      :scale-length {set sl $v}
      :scale-width {set sw $v}
      :slider-width {set slw $v}
      :layout {
        switch $v {
          V    {set orient vertical}
          H    {set orient horizontal}
          default {puts stderr ":slider :layout - unknown option $v"}
        }
      }
      :ticks {
        switch $v {
          T    {set ticks $v}
          F    {set ticks $v}
          default {puts stderr ":slider :ticks - unknown option $v"}
        }
      }
      :show {}
      :deaf {
        switch $v {
          T    {set deaf "T"}
          F    {set deaf "F"}
          default {puts stderr ":slider :deaf - unknown option $v"}
        }
      }
      :inactive {
        switch $v {
          T    {set inactive "T"}
          F    {set inactive "F"}
          default {puts stderr ":slider :inactive - unknown option $v"}
        }
      }
      :auto {
        switch $v {
          T    {set _autoreturn "T"}
          F    {set _autoreturn "F"}
          default {puts stderr ":slider :auto - unknown option $v"}
        }
      }
      :bold {
        switch $v {
          T    {set bold 1}
          F    {set bold 0}
          default {puts stderr ":slider :bold - unknown option $v"}
        }
      }
      :help {
        switch $v {
          T    {set help "T"}
          F    {set help "F"}
          default {:help $helpitem $v}
        }
      }
      :helpitem {set helpitem $v}
      :command {global _commandname; set _commandname($name) "$v"}
      :after  {set _pack " -after  [:path $v] "}
      :before {set _pack " -before [:path $v] "}

      default {puts stderr ":slider - unknown arg $p"}
    }
    set i [expr $i + 2]
  }

  label $win.label -text "$ll"
  if {$vx == 0} then {
#    pack $win.label  -side left  -anchor ne -fill y
  } else {
    if {[string match *c $vx] == 0} then {
       set lw [expr $vx/$_charw]
    } else {
       set lw [string trim $vx c]
    }
    $win.label configure -anchor ne -width $lw
#    pack $win.label  -side left  -anchor ne -fill y
  }

  if {$orient == "horizontal"} then {
    pack $win.label  -side left  -anchor ne -fill y
  } else {
    pack $win.label  -side bottom  -anchor se -fill x
  }

  :_set-font $win.label label $bold N

  entry $win.entry -relief sunken -width $ew
  if {$inactive == "T"} then {$win.entry configure -state disabled}
  $win.entry insert 0  "$vl"
  bind  $win.entry <Return> ":slider-event-entry $win $name"

  if {$_autoreturn == "T"} then {
    bind  $win.entry <FocusIn>  ":slider-focus In $win $name"
    bind  $win.entry <FocusOut> ":slider-focus Out $win $name"
  }

  :_set-font $win.entry entry $bold N

  label $win.min -text "$min"
  if {$lmw != 0} then {$win.min configure -width $lmw}

  :_set-font $win.min label $bold N

  global _resolution _steps _min _max
  set _resolution($name) $resolution
  set _steps($name) $steps
  set _min($name) $min
  set _max($name) $max

  if {$resolution == 0} then {
#    no resolution supplied for the scale
     if {$steps == 0} then {set steps 256}
#    calculate resolution given number of required steps
        set resolution [expr [expr $max - $min + 1.0] / $steps]
        if {$resolution < 0} then {set resolution -$resolution}
  }

  global $name.value
  scale $win.scale -relief sunken -orient $orient -showvalue 0  \
      -bigincrement 0 -resolution $resolution  \
      -variable $name.value -command ":slider-inc-scale $win"
  if {$inactive == "T"} then {$win.scale configure -state disabled}
  $win.scale configure -length $sl -width $sw -sliderlength $slw
  if {$orient == "horizontal"} then {
     $win.scale configure -from $min -to $max
  } else {
     $win.scale configure -from $max -to $min
  }

  if {$ticks == "T"} then {
     set vl [expr [expr $max - $min] / 10]
     $win.scale configure -tickinterval $vl
  }

  set $name.value $vl
  bind $win.scale <ButtonRelease-1> ":slider-event-scale $win $name"

  label $win.max -text "$max"
  if {$lmw != 0} then {$win.max configure -width $lmw}

  :_set-font $win.max label $bold N

  if {$orient == "horizontal"} then {
     pack $win.max  $win.scale  -side right -fill y
     pack  $win.entry $win.min  -side left -fill y
  } else {
     pack  $win.max    -side top -fill x
     pack  $win.scale  -side top -fill none
     pack  $win.entry $win.min  -side bottom -fill x
  }

  if {$debug} then {insert-debug ":slider created $win.label and $win.scale"}

  if {$deaf == "T"} then {:_deaf $win.entry Entry}
  if {$deaf == "T"} then {:_deaf $win.scale Scale}
  if {$help == "T"} then {:_help $helpitem $win Scale}

  :_set-color $win frame
  :_set-color $win.entry entry
  :_set-color $win.scale scale
  :_set-color $win.label label
  :_set-color $win.min   label
  :_set-color $win.max   label

  if {$ticks == "T"} then {$win.scale configure -foreground black}

  :_position-widget $placed $win $xp $yp

  :_post-updateposn $placed $win $xp $yp

}


proc :slider-inc-scale {win value} {
  $win.entry delete 0 end
  $win.entry insert 0 $value
}

proc :slider-event-entry {win name} {
  global frame $name.value __EntryData
  set data "[$win.entry get]"
  set __EntryData "$data"
  set $name.value "$data"
  global _commandname
  if {[info exists _commandname($name)]} then {
     eval $_commandname($name) [list [string toupper $name]  [set $name.value]]
  } else {
     click [list $frame "" [string toupper $name]  [set $name.value]]
  }
}

proc :slider-event-scale {win name} {
  global frame $name.value
  global _commandname
  if {[info exists _commandname($name)]} then {
     eval $_commandname($name) [list [string toupper $name]  [set $name.value]]
  } else {
     click [list $frame "" [string toupper $name]  [set $name.value]]
  }
}

proc :slider-focus {event win name} {
  global __EntryData __EntryWin
  switch $event {
     In  {
#        puts stdout "focus $event $win $name"
        set __EntryData "[$win.entry get]"
        set __EntryWin $win
     }
     Out {
#        puts stdout "focus $event $win $name"
        set __EntryWin ""
        set data "[$win.entry get]"
        if {"$data" != "$__EntryData"} then {
#           puts stdout "entry data has changed (\"$data\" != \"$__EntryData\") - calling :slider-event-entry"
           :slider-event-entry $win $name
        }
     }
     default {}
  }
}


proc :modify_slider {win name args} {
  global debug __EntryData __EntryWin

#    options -   :min ? :max ? :value ? :inactive T|F

  set args [lindex $args 0]
  if {$debug} then {insert-debug ":modify_slider called with  $win $name $args"}

  global _resolution _steps _min _max
  set resolution $_resolution($name)
  set steps $_steps($name)
  set min $_min($name)
  set max $_max($name)

  set i 0
  while {$i < [llength $args]} {
    set p [lindex $args $i]
    set vl [lindex $args [expr $i + 1]]
    switch $p {

      :deaf {}
      :inactive  {
        switch $vl {
          T {
            $win.scale configure -state disabled
            $win.entry configure -state disabled
          }
          F {
            $win.scale configure -state normal
            $win.entry configure -state normal
          }
          default {puts stderr ":modify_slider  :inactive  - unknown option $v"}
        }
      }
      :label {
         $win.label configure -text "$vl"
      }
      :min {
         set _min($name) $vl
         set min $vl
         if {$resolution == 0} then {
#    no resolution supplied for the scale
            if {$steps == 0} then {set steps 256}
#    calculate resolution given number of required steps
            set resolution [expr [expr $max - $min + 0.0] / $steps]
            if {$resolution < 0} then {set resolution -$resolution}
         }
         $win.min configure -text "$vl"
         $win.scale configure -from $vl -resolution $resolution
      }
      :max {
         set _max($name) $vl
         set max $vl
         if {$resolution == 0} then {
#    no resolution supplied for the scale
            if {$steps == 0} then {set steps 256}
#    calculate resolution given number of required steps
            set resolution [expr [expr $max - $min + 0.0] / $steps]
            if {$resolution < 0} then {set resolution -$resolution}
         }
         $win.max configure -text "$vl"
         $win.scale configure -to $vl -resolution $resolution
      }
      :value  {
        $win.entry delete 0 end
        $win.entry insert 0 "[expr $vl]"
        global $name.value
        set $name.value [expr $vl]
        if {$__EntryWin == $win} then {set __EntryData $vl}
      }
      default {puts stderr ":modify_slider - unknown arg $p"}
    }
    set i [expr $i + 2]
  }

}

proc :query_slider {win name args} {
  global debug $name.value _min _max

#    settings -    :label :min :max :value

  set args [lindex $args 0]
  if {$debug} then {insert-debug ":query_slider called with  $win $name $args"}

  switch [set p [lindex $args 0]] {
    :label {return [$win.label cget -text]}
    :min   {return $_min($name)}
    :max   {return $_max($name)}
    :value {return [set $name.value]}

    default {puts stderr ":query_slider $win - unknown arg $p"}
  }
}


proc :menubutton args {
################
################
  global frame
  global debug _frame _panel _container _layout _xposn _yposn _placed _pack _colgap _rowgap _charh _rows

#    :menubutton name :x ? :y ? :label ? :strings ? :value ?

  set placed $_placed
  set width 0; set menu 0
  set vl 0; set xw 0; set xh 0
  set bold 0; set deaf "F"; set help "T"
  set hlt 0; set pady 4
  set __rows 10

  set name [string tolower [lindex $args 0]]
  set helpitem [lindex $args 0]
  set ll [string toupper [string range $name 0 0]]
  append ll [string range $name 1 end]

  if {$_container == ""} then {
     if {$_panel == ""} then {
        set win $_frame.$name
     } else {
        set win $_frame.$_panel.$name
     }
  } else {
     if {$_panel == ""} then {
        set win $_frame.$_container.$name
     } else {
        set win $_frame.$_container.$_panel.$name
     }
  }

  global _classname _winname
  set _classname($name) "menubutton"
  set _winname($name)  $win

  :_pre-updateposn;  set xp $_xposn; set yp $_yposn

  menubutton $win -height 1 -relief raised -menu $win.menu -indicatoron true

  if {$debug} then {insert-debug ":menubutton created $win and $win.menu"}

  set i 1
  while {$i < [llength $args]} {
    set p [lindex $args $i]
    set v [lindex $args [expr $i + 1]]
    switch $p {

      :x  {set xp $v; set placed 1}
      :y  {set yp $v; set placed 1}
      :w  {set xw $v}
      :h  {set xh $v}
      :rows {set __rows $v; set _rows($name) $v}
      :label    {set ll $v}
      :strings  {set st $v; set menu 1}
      :menu     {set st $v; set menu 2}
      :value {set vl $v}
      :show {}
      :deaf {
        switch $v {
          T    {set deaf "T"}
          F    {set deaf "F"}
          default {puts stderr ":menubutton :deaf - unknown option $v"}
        }
      }
      :help {
        switch $v {
          T    {set help "T"}
          F    {set help "F"}
          default {:help $helpitem $v}
        }
      }
      :helpitem {set helpitem $v}
      :inactive {
        switch $v {
          T {$win configure -state disabled}
          F {$win configure -state normal}
          default {puts stderr ":menubutton :inactive - unknown option $v"}
        }
      }
      :bold {
        switch $v {
          T    {set bold 1}
          F    {set bold 0}
          default {puts stderr ":menubutton :bold - unknown option $v"}
        }
      }
      :double-click {}
      :after  {set _pack " -after  [:path $v] "}
      :before {set _pack " -before [:path $v] "}
      :command {global _commandname; set _commandname($name) "$v"}

      default {puts stderr ":menubutton - unknown arg $p"}
    }
    set i [expr $i + 2]
  }

  $win configure -width $xw -height $xh -highlightthickness $hlt -pady $pady -text "$ll"

  if {$deaf == "T"} then {:_deaf $win Menubutton}
  if {$help == "T"} then {:_help $helpitem $win Menubutton}

  if {$menu == 0} then {
     menu $win.menu -tearoff false
     :_set-font $win.menu menu $bold N
  }

  if {$menu == 1} then {
     menu $win.menu -tearoff false
     :_set-font $win.menu menu $bold N
        set ii 0
        set jj 0
        foreach s $st {
          if {$jj == $__rows} then {
             $win.menu add command -label "$s" -columnbreak 1 \
                -command ":menubutton-event $win $name \"$s\" $ii"
             set jj 0
          } else {
             $win.menu add command -label "$s"  \
                -command ":menubutton-event $win $name \"$s\" $ii"
          }
          lappend _$name.menu "$s"
          incr ii
          incr jj
        }
  }

  if {$menu == 2} then {
     menu $win.menu -tearoff true
     :_set-font $win.menu menu $bold N

       set ii 0
       set jj 0
       set kk 0
       foreach s $st {
          if {[llength $s] <= 1} then {
            $win.menu add command -label "[first $s]" \
               -command ":menubutton-event $win $name \"[first $s]\" $kk"
            incr kk
          } else {
            incr ii
            $win.menu add cascade -label "[first $s]" -menu $win.menu.c$ii
            set s [lreplace $s 0 0]
            catch {menu $win.menu.c$ii -tearoff true}
            :_set-font $win.menu.c$ii menu 0 N
            $win.menu.c$ii delete 0 end
            foreach ss $s {
               if {[llength $ss] <= 1} then {
                   $win.menu.c$ii add command -label "[first $ss]" \
                      -command ":menubutton-event $win $name \"[first $ss]\" $kk"
                   incr kk
               } else {
                   incr jj
                   $win.menu.c$ii add cascade -label "[first $ss]" -menu $win.menu.c$ii.c$jj
                   set ss [lreplace $ss 0 0]
                   catch {menu $win.menu.c$ii.c$jj -tearoff true}
                   :_set-font $win.menu.c$ii.c$jj menu 0 N
                   $win.menu.c$ii.c$jj delete 0 end
                   foreach sss $ss {
                      $win.menu.c$ii.c$jj add command -label "[first $sss]" \
                          -command ":menubutton-event $win $name \"[first $sss]\" $kk"
                      incr kk
                   }
               }
            }
         }
      }
  }

  :_set-font $win menubutton $bold N

  if {$menu > 0} then {
     set cmd [list "[lindex $st $vl]"]
     bind $win <ButtonRelease-3> ":menubutton-event $win $name $cmd $vl"
  }

  :_set-color $win menubutton

  :_position-widget $placed $win $xp $yp

  :_post-updateposn $placed $win $xp $yp

}

proc :menubutton-event {win name item index} {
  global frame _commandname
  if {[info exists _commandname($name)]} then {
     eval $_commandname($name) [list [string toupper $name] "$item" $index]
  } else {
     click [list $frame "" [string toupper $name] "$item" $index]
  }
}

proc :modify_menubutton {win name args} {
  global debug _rows
  set __rows 10

#    options -   :strings ?

  set args [lindex $args 0]
  if {$debug} then {insert-debug ":modify_menubutton called with  $win $name $args"}

  if ([info exists _rows($name)])  {set __rows $_rows($name)}

  set i 0
  while {$i < [llength $args]} {
    set p [lindex $args $i]
    set st [lindex $args [expr $i + 1]]
    switch $p {

      :rows {set __rows $p; set _rows($name) $p}
      :deaf {}
      :inactive  {
        switch $st {
          T {$win configure -state disabled}
          F {$win configure -state normal}
          default {puts stderr ":modify_menubutton  :inactive  - unknown option $v"}
        }
      }
      :label {$win configure -text "$st"}
      :strings  {
        $win.menu delete 0 end
        set ii 0
        set jj 0
        foreach s $st {
          if {$jj == $__rows} then {
             $win.menu add command -label "$s" -columnbreak 1 \
                -command ":menubutton-event $win $name \"$s\" $ii"
             set jj 0
          } else {
             $win.menu add command -label "$s"  \
                -command ":menubutton-event $win $name \"$s\" $ii"
          }
          lappend _$name.menu "$s"
          incr ii
          incr jj
        }
      }
      :menu {
        $win.menu delete 0 end
        set ii 0
        set jj 0
        set kk 0
        foreach s $st {
          if {[llength $s] <= 1} then {
            $win.menu add command -label "$s" \
               -command ":menubutton-event $win $name \"$s\" $kk"
            incr kk
          } else {
            incr ii
            set c [first $s]
            $win.menu add cascade -label "$c" -menu $win.menu.c$ii
            set s [lreplace $s 0 0]
            catch {menu $win.menu.c$ii -tearoff true}
            :_set-font $win.menu.c$ii menu 0 N
            $win.menu.c$ii delete 0 end
            foreach ss $s {
               if {[llength $ss] <= 1} then {
                   $win.menu.c$ii add command -label "$ss" \
                      -command ":menubutton-event $win $name \"$ss\" $kk"
                   incr kk
               } else {
                   incr jj
                   set cc [first $ss]
                   $win.menu.c$ii add cascade -label "$cc" -menu $win.menu.c$ii.c$jj
                   set ss [lreplace $ss 0 0]
                   catch {menu $win.menu.c$ii.c$jj -tearoff true}
                   :_set-font $win.menu.c$ii.c$jj menu 0 N
                   $win.menu.c$ii.c$jj delete 0 end
                   foreach sss $ss {
                      $win.menu.c$ii.c$jj add command -label "$sss" \
                          -command ":menubutton-event $win $name \"$sss\" $kk"
                      incr kk
                   }
               }
            }
         }
       }
      }
      default {puts stderr ":modify_menubutton - unknown arg $p"}
    }
    set i [expr $i + 2]
  }

}

proc :query_menubutton {win name args} {
  global debug _rows

#    settings -    :label :rows :menu :strings

  set args [lindex $args 0]
  if {$debug} then {insert-debug ":query_menubutton called with  $win $name $args"}

  switch [set p [lindex $args 0]] {
    :label {return [$win cget -text]}
    :rows  {return [if ([info exists _rows($name)])  {set __rows $_rows($name)} {set __rows 10}]}
    :menu -
    :strings {set l {};for {set i 0} {$i <= [$win.menu index last]} {incr i} {
lappend l "[$win.menu entrycget $i -label]"};return $l}

    default {puts stderr ":query_menubutton $win - unknown arg $p"}
  }
}


proc :optionbutton args {
################
################
  global frame
  global debug _frame _panel _container _layout _xposn _yposn _placed _pack _colgap _rowgap _charh

#    :optionbutton name :x ? :y ? :label ? :strings ? :value ?

  set placed $_placed
  set width 0
  set vl 0; set xw 0; set xh 0; set st ""
  set bold 0; set deaf "F"; set help "T"
  set hlt 0; set pady 3

  set name [string tolower [lindex $args 0]]
  set helpitem [lindex $args 0]
  set ll [string toupper [string range $name 0 0]]
  append ll [string range $name 1 end]

  if {$_container == ""} then {
     if {$_panel == ""} then {
        set win $_frame.$name
     } else {
        set win $_frame.$_panel.$name
     }
  } else {
     if {$_panel == ""} then {
        set win $_frame.$_container.$name
     } else {
        set win $_frame.$_container.$_panel.$name
     }
  }

  global _classname _winname
  set _classname($name) "optionbutton"
  set _winname($name)  $win

  :_pre-updateposn;  set xp $_xposn; set yp $_yposn

  button $win -relief sunken -command ":optionbutton-event $win $name"

  if {$debug} then {insert-debug ":optionbutton created $win"}

  set i 1
  while {$i < [llength $args]} {
    set p [lindex $args $i]
    set v [lindex $args [expr $i + 1]]
    switch $p {

      :x  {set xp $v; set placed 1}
      :y  {set yp $v; set placed 1}
      :w  {set xw $v}
      :h  {set xh $v}
      :strings  -
      :menu     {set st $v}
      :value {set vl $v}
      :show {}
      :deaf {
        switch $v {
          T    {set deaf "T"}
          F    {set deaf "F"}
          default {puts stderr ":optionbutton :deaf - unknown option $v"}
        }
      }
      :inactive {
        switch $v {
          T {$win configure -state disabled}
          F {$win configure -state normal}
          default {puts stderr ":optionbutton :inactive - unknown option $v"}
        }
      }
      :bold {
        switch $v {
          T    {set bold 1}
          F    {set bold 0}
          default {puts stderr ":optionbutton :bold - unknown option $v"}
        }
      }
      :help {
        switch $v {
          T    {set help "T"}
          F    {set help "F"}
          default {:help $helpitem $v}
        }
      }
      :helpitem {set helpitem $v}
      :double-click {}
      :after  {set _pack " -after  [:path $v] "}
      :before {set _pack " -before [:path $v] "}
      :command {global _commandname; set _commandname($name) "$v"}

      default {puts stderr ":optionbutton - unknown arg $p"}
    }
    set i [expr $i + 2]
  }

  $win configure -width $xw -height $xh -highlightthickness $hlt -pady $pady

  global _options _index
  set _options($name) $st
  set _index($name) $vl

  $win configure  -text "[lindex $st $vl]"

  if {$deaf == "T"} then {:_deaf $win Button}
  if {$help == "T"} then {:_help $helpitem $win Button}

  :_set-font $win button $bold N

  :_set-color $win button

  :_position-widget $placed $win $xp $yp

  :_post-updateposn $placed $win $xp $yp

}

proc :optionbutton-event {win name} {
  global _options _index
  incr _index($name)
  if {$_index($name) >= [llength $_options($name)]} then {set _index($name) 0}
  set item [lindex $_options($name) $_index($name)]
  $win configure  -text "$item"
  global frame _commandname
  if {[info exists _commandname($name)]} then {$_commandname($name) $_index($name) "$item"} else \
  {click [list $frame "" [string toupper $name] $_index($name) "$item"]}
}


proc :modify_optionbutton {win name args} {
  global debug

#    options -   :strings ?

  set args [lindex $args 0]
  if {$debug} then {insert-debug ":modify_menubutton called with  $win $name $args"}

  set i 0
  while {$i < [llength $args]} {
    set p [lindex $args $i]
    set st [lindex $args [expr $i + 1]]
    switch $p {

      :deaf {}
      :inactive  {
        switch $st {
          T {$win configure -state disabled}
          F {$win configure -state normal}
          default {puts stderr ":modify_menubutton  :inactive  - unknown option $v"}
        }
      }
      :value {
         global _options _index
         set _index($name) $st
         
         $win configure -text "[lindex $_options($name) $_index($name)]"
      }
      :menu -
      :strings  {
         global _options _index
         set _options($name) $st

         $win configure  -text "[lindex $_options($name) $_index($name)]"
      }
      default {puts stderr ":modify_menubutton - unknown arg $p"}
    }
    set i [expr $i + 2]
  }

}

proc :query_optionbutton {win name args} {
  global debug _options _index

#    settings -    :string/:label (current label string) :value :menu/:strings (all label strings)

  set args [lindex $args 0]
  if {$debug} then {insert-debug ":query_optionbutton called with  $win $name $args"}

  switch [set p [lindex $args 0]] {
    :string -
    :label {return [lindex $_options($name) $_index($name)]}
    :value {return $_index($name)}
    :menu -
    :strings {return $_options($name)}

    default {puts stderr ":query_optionbutton $win - unknown arg $p"}
  }

}


proc :choice args {
############
############
  global debug _frame _panel _container _layout _placed _pack
  global  _xposn _yposn _colgap _rowgap _charh _charw _chcesize

#    :choice name :x ? :y ? :valuex ? :w ? :label ? :strings ? :value ? :exclusive T/F
#          :deaf T|F :layout V|H

  set placed $_placed
  set xp $_xposn; set yp $_yposn; set ww 0; set vx 0; set st ""; set ll ""; set vl 0; set pad 0
  set excl "T"; set lay "H"; set inactive "F"; set deaf "F"; set help "T"
  set bold 0

  set name [string tolower [lindex $args 0]]
  set helpitem [lindex $args 0]

  if {$_container == ""} then {
     if {$_panel == ""} then {
        set win $_frame.$name
     } else {
        set win $_frame.$_panel.$name
     }
  } else {
     if {$_panel == ""} then {
        set win $_frame.$_container.$name
     } else {
        set win $_frame.$_container.$_panel.$name
     }
  }

  global _classname _winname
  set _classname($name) "choice"
  set _winname($name)  $win

  :_pre-updateposn;  set xp $_xposn; set yp $_yposn

  frame $win -relief flat
  if {$debug} then {$win configure -relief raised -bd 1}
  $win configure -relief raised -bd 1
  if {$debug} then {insert-debug ":  created $win"}

  :_set-color $win frame

  set i 1
  while {$i < [llength $args]} {
    set p [lindex $args $i]
    set v [lindex $args [expr $i + 1]]
    switch $p {
      :x  {set xp $v; set placed 1}
      :y  {set yp $v; set placed 1}
      :w  {set ww $v}
      :valuex {
        set vx $v
        if {$ll == ""} then {set ll " "}
      }
      :value {set vl $v}
      :bold {
        switch $v {
          T    {set bold 1}
          F    {set bold 0}
          default {puts stderr ":choice :bold - unknown option $v"}
        }
      }
      :inactive {
        switch $v {
          T    {set inactive "T"}
          F    {set inactive "F"}
          default {puts stderr ":choice :inactive - unknown option $v"}
        }
      }
      :deaf {
        switch $v {
          T    {set deaf "T"}
          F    {set deaf "F"}
          default {puts stderr ":choice :deaf - unknown option $v"}
        }
      }
      :help {
        switch $v {
          T    {set help "T"}
          F    {set help "F"}
          default {:help $helpitem $v}
        }
      }
      :helpitem {set helpitem $v}
      :label {set ll "$v"}
      :strings  {set st "$v"}
      :layout  {
        switch $v {
          V -
          H {set lay $v}
          default {puts stderr ":choice :layout - unknown option $v"}
        }
      }
      :exclusive  {
        switch $v {
          T -
          F {set excl $v}
          default {puts stderr ":choice :exclusive - unknown option $v"}
        }
      }
      :pad  {set pad $v}
      :show {}
      :double-click {}
      :command {global _commandname; set _commandname($name) "$v"}
      :after  {set _pack " -after  [:path $v] "}
      :before {set _pack " -before [:path $v] "}

      default {puts stderr ":choice - unknown arg $p"}
    }
    set i [expr $i + 2]
  }

  if {$ll != ""} then {
    label $win.label -text "$ll"
    :_set-font $win.label label $bold N
    :_set-color $win.label label

     if {$vx == 0} then {
        if {$lay == "H"} then {pack $win.label -side left -anchor ne -fill y} else \
            {pack $win.label -side top}
     } else {
       if {[string match *c $vx] == 0} then {
          set lw [expr $vx/$_charw]
       } else {
          set lw [string trim $vx c]
       }
       $win.label configure -anchor ne -width $lw
        if {$lay == "H"} then {pack $win.label -side left -anchor ne -fill y} else \
            {pack $win.label -side top}
     }
  }

  global _choiceopt
  set _choiceopt($name) $excl

  if {$st != ""} then {
    set i 0
    foreach s $st {
      global $name.b$i
      checkbutton  $win.button$i  -variable $name.b$i -text $s \
        -command ":choice-event $win $name $i" -indicatoron false
      if {$inactive == "T"} then {$win.button$i configure -state disabled}
      if {$deaf == "T"} then {:_deaf $win.button$i Checkbutton}

      :_set-font $win.button$i checkbutton $bold N

      :_set-color $win.button$i checkbutton

      if {$ww != 0} then {$win.button$i configure -width $ww}

      if {$lay == "H"} then {
         pack $win.button$i -side left -padx $pad
      } else {
         pack $win.button$i -side top  -pady $pad
      }
      incr i
    }
  }

  if {$help == "T"} then {:_help $helpitem $win Choice}

  :modify $name :value $vl

  :_position-widget $placed $win $xp $yp

  :_post-updateposn $placed $win $xp $yp

}


proc :choice-event {win name i} {
  global frame $name.b$i _choiceopt  _commandname

  if {$_choiceopt($name) == "T"} then {

#    option is exclusive T

    set vl [set $name.b$i]

    if {$vl == 1} then {
      for {set j 0} {$j < 32} {incr j} {
        if {$j != $i} then {
          global $name.b$j
          if [info exists $name.b$j] then {set $name.b$j 0} \
          else {set j 32}
        }
      }
    } else {set $name.b$i 1}

    if {[info exists _commandname($name)]} then {
       eval $_commandname($name) [list [string toupper $name]  $i]
    } else {
       click [list $frame "" [string toupper $name] $i]
    }
  } else {

#    option is exclusive F

    set vl 0
    for {set j 0} {$j < 32} {incr j} {
      global $name.b$j
      if [info exists $name.b$j] then {
        set vl [expr [expr [set $name.b$j] << $j] + $vl]
      } else {set j 32}
    }

    if {[info exists _commandname($name)]} then {
       eval $_commandname($name) [list [string toupper $name]  $vl]
    } else {
       click [list $frame "" [string toupper $name] $vl]
    }
  }

}

proc :modify_choice {win name args} {
  global debug

#    options -   :value ? :inactive T|F

  set args [lindex $args 0]
  if {$debug} then {insert-debug ":modify_choice called with  $win $name $args"}

  set i 0
  while {$i < [llength $args]} {
    set p [lindex $args $i]
    set vl [lindex $args [expr $i + 1]]
    switch $p {

      ;deaf {}
      :inactive  {
        switch $vl {
          T {
            for {set j 0} {$j < 32} {incr j} {
               global $name.b$j
               if [info exists $name.b$j] then {$win.button$j configure -state disabled}  \
               else {set j 32}
            }
          }
          F {
            for {set j 0} {$j < 32} {incr j} {
               global $name.b$j
               if [info exists $name.b$j] then {$win.button$j configure -state normal}  \
               else {set j 32}
            }
          }
          default {puts stderr ":modify_choice  :inactive  - unknown option $v"}
        }
      }
      :value  {
        global _choiceopt
# puts stderr ":modify :choice $name $vl $_choiceopt($name)"
        if {$_choiceopt($name) == "T"} then {

#    option is exclusive T

          for {set j 0} {$j < 32} {incr j} {
          global $name.b$j
            if {$j != $vl} then {
              if [info exists $name.b$j] then {set $name.b$j 0} \
              else {set j 32}
            } else {set $name.b$j 1}
          }
        } else {

#    option is exclusive F

          for {set j 0} {$j < 32} {incr j} {
            global $name.b$j
            if [info exists $name.b$j] then {
              set $name.b$j [expr $vl & 1]
              set vl [expr $vl >> 1]
            } else {set j 32}
          }
        }

      }
      default {puts stderr ":modify_choice - unknown arg $p"}
    }
    set i [expr $i + 2]
  }

}

proc :query_choice {win name args} {
  global debug _choiceopt

#    settings -    :label :string/:strings :value

  set args [lindex $args 0]
  if {$debug} then {insert-debug ":query_choice called with  $win $name $args"}

  switch [set p [lindex $args 0]] {
    :label {if {[info commands $win.label]!=""} {return [$win.label cget -text]}}
    :strings -
    :string {
      if {$_choiceopt($name) == "T"} then {

#    option is exclusive T
        set vl -1
        for {set j 0} {$j < 32} {incr j} {
          global $name.b$j
          if [info exists $name.b$j] then {
            if {[set $name.b$j] != 0} then {set vl $j; set j 32}
          } else {set j 32}
        }
        if {[info commands $win.button$vl]!=""} {return [$win.button$vl cget -text]} {return}
      } else {

#    option is exclusive F

        set l {}
        for {set j 0} {$j < 32} {incr j} {
          global $name.b$j
          if [info exists $name.b$j] then {
            if {[set $name.b$j] != 0} then {
              lappend l [$win.button$j cget -text]
            }
          } else {set j 32}
        }
        return $l
      }
    }

    :value {
      if {$_choiceopt($name) == "T"} then {

#    option is exclusive T

        set vl -1
        for {set j 0} {$j < 32} {incr j} {
          global $name.b$j
          if [info exists $name.b$j] then {
            if {[set $name.b$j] != 0} then {set vl $j; set j 32}
          } else {set j 32}
        }
      } else {

#    option is exclusive F

        set vl 0
        for {set j 0} {$j < 32} {incr j} {
          global $name.b$j
          if [info exists $name.b$j] then {
            set vl [expr [expr [set $name.b$j] << $j] + $vl]
          } else {set j 32}
        }
      }
      if {$vl != -1} {return $vl} {return}
    }
    default {puts stderr ":query_choice $win - unknown arg $p"}
  }
}


proc :choice-stack args {
##################
##################
  global frame
  global debug _frame _panel _container _layout _xposn _yposn _placed _pack _colgap _rowgap _charh _charw _rows


#    :choice-stack name :x ? :y ? :label ? :menu ? :strings ? :valuex ? :value ? :exclusive T/F

  set placed $_placed
  set width 0; set menu 0
  set ll ""; set st ""; set vx 0; set vl 0; set ww 0
  set bold 0; set deaf "F"; set help "T"
  set __rows 10

  set name [string tolower [lindex $args 0]]
  set helpitem [lindex $args 0]

  if {$_container == ""} then {
     if {$_panel == ""} then {
        set win $_frame.$name
     } else {
        set win $_frame.$_panel.$name
     }
  } else {
     if {$_panel == ""} then {
        set win $_frame.$_container.$name
     } else {
        set win $_frame.$_container.$_panel.$name
     }
  }

  global _classname _winname
  set _classname($name) "choice-stack"
  set _winname($name)  $win

  :_pre-updateposn;  set xp $_xposn; set yp $_yposn

  frame $win -relief flat
  if {$debug} then {$win configure -relief raised -bd 1}
  $win configure -relief raised -bd 1
  menubutton $win.menu -relief raised -menu $win.menu.menu -indicatoron true
  message $win.message -aspect 10000

  if {$debug} then {insert-debug ":choice-stack created $win, $win.menu and $win.message"}

  set i 1
  while {$i < [llength $args]} {
    set p [lindex $args $i]
    set v [lindex $args [expr $i + 1]]
    switch $p {

      :x  {set xp $v; set placed 1}
      :y  {set yp $v; set placed 1}
      :rows {set __rows $v; set _rows($name) $v}
      :value {set vl $v}
      :exclusive {}
      :valuex {
        set vx $v
        if {$ll == ""} then {set ll " "}
      }
      :label    {set ll $v}
      :menu     {set st $v; set menu 2}
      :strings  {set st $v; set menu 1}
      :show {}
      :deaf {
        switch $v {
          T    {set deaf "T"}
          F    {set deaf "F"}
          default {puts stderr ":choice-stack :deaf - unknown option $v"}
        }
      }
      :inactive {
        switch $v {
          T {$win.menu configure -state disabled}
          F {$win.menu configure -state normal}
          default {puts stderr ":choice-stack :inactive - unknown option $v"}
        }
      }
      :bold {
        switch $v {
          T    {set bold 1}
          F    {set bold 0}
          default {puts stderr ":choice-stack :bold - unknown option $v"}
        }
      }
      :help {
        switch $v {
          T    {set help "T"}
          F    {set help "F"}
          default {:help $helpitem $v}
        }
      }
      :helpitem {set helpitem $v}
      :double-click {}
      :after  {set _pack " -after  [:path $v] "}
      :before {set _pack " -before [:path $v] "}
      :command {global _commandname; set _commandname($name) "$v"}

      default {puts stderr ":choice-stack - unknown arg $p"}
    }
    set i [expr $i + 2]
  }

  $win.message configure -text "[lindex $st $vl]"

  if {$deaf == "T"} then {:_deaf $win.menu Menubutton}
  if {$help == "T"} then {:_help $helpitem $win Choice}

  :_set-font $win.message message $bold N
  :_set-font $win.menu menubutton $bold N

  :_set-color $win frame
  :_set-color $win.menu    menubutton
  :_set-color $win.message message

  if {$ll != ""} then {
    label $win.label -text "$ll"

    :_set-font $win.label label $bold N
    :_set-color $win.label label

     if {$vx == 0} then {
        pack $win.label -side left -anchor ne -fill y
     } else {
       if {[string match *c $vx] == 0} then {
          set lw [expr $vx/$_charw]
       } else {
          set lw [string trim $vx c]
       }
       $win.label configure -anchor ne -width $lw
       pack $win.label  -side left  -anchor ne -fill y
     }
     pack $win.menu  -side left
     pack $win.message -side left -anchor nw
  } else {
     pack $win.menu $win.message -side left
  }

  global _$name.menu
  set _$name.menu {}

  if {$menu == 0} then {
     menu $win.menu.menu -tearoff false
     :_set-font $win.menu.menu menu $bold N
  }

  if {$menu == 1} then {
     menu $win.menu.menu -tearoff false
     :_set-font $win.menu.menu menu $bold N
        set ii 0
        set jj 0
        foreach s $st {
          if {$jj == $__rows} then {
             $win.menu.menu add command -label "$s" -columnbreak 1 \
                -command ":choice-stack-event $win $name $ii"
             set jj 0
          } else {
             $win.menu.menu add command -label "$s"  \
                -command ":choice-stack-event $win $name $ii"
          }
          lappend _$name.menu "$s"
          incr ii
          incr jj
        }
  }

  if {$menu == 2} then {
     menu $win.menu.menu -tearoff false
     :_set-font $win.menu.menu menu $bold N
     set i 0
     foreach s $st {
       if {[llength $s] <= 1} then {
          $win.menu.menu add command -label "$s" \
              -command ":choice-stack-event $win $name $i"
          lappend _$name.menu "$s"
          incr i
       } else {
          set c [first $s]
          set j $i
          $win.menu.menu add cascade -label "$c" -menu $win.menu.menu.c$j
          set s [lreplace $s 0 0]
          menu $win.menu.menu.c$j -tearoff true
          :_set-font $win.menu.menu.c$j menu $bold N
          foreach ss $s {
             $win.menu.menu.c$j add command -label "$ss" \
                -command ":choice-stack-event $win $name $i"
          lappend _$name.menu "$ss"
          incr i
          }
       }
     }
  }

  :_position-widget $placed $win $xp $yp

  :_post-updateposn $placed $win $xp $yp

}

proc :choice-stack-event {win name item} {
  global frame _commandname
  global _$name.menu
  set s [lindex [set _$name.menu] $item]
  $win.message configure -text "$s"

  if {[info exists _commandname($name)]} then {
     eval $_commandname($name) [list [string toupper $name] "$item" "$s"]
  } else {
     click [list $frame "" [string toupper $name] "$item" "$s"]
  }
}

proc :modify_choice-stack {win name args} {
  global debug _rows
  set __rows 10

#    options -   :value ?  :strings ? :inactive T|F

  set args [lindex $args 0]
  if {$debug} then {insert-debug ":modify_choice-stack called with  $win $name $args"}

  if ([info exists _rows($name)])  {set __rows $_rows($name)}

  set i 0
  while {$i < [llength $args]} {
    set p [lindex $args $i]
    set vl [lindex $args [expr $i + 1]]
    switch $p {

      :rows {set __rows $p; set _rows($name) $p}
      :deaf {}
      :inactive  {
        switch $vl {
          T {$win.menu configure -state disabled}
          F {$win.menu configure -state normal}
          default {puts stderr ":modify_choice-stack  :inactive  - unknown option $vl"}
        }
      }
      :value  {
        global _$name.menu
        set s [lindex [set _$name.menu] $vl]
        $win.message configure -text "$s"
      }
      :strings  {
        global _$name.menu
        set _$name.menu {}
        $win.menu.menu delete 0 end
        set ii 0
        set jj 0
        foreach s $vl {
          if {$jj == $__rows} then {
             $win.menu.menu add command -label "$s" -columnbreak 1 \
                -command ":choice-stack-event $win $name $ii"
             set jj 0
          } else {
             $win.menu.menu add command -label "$s"  \
                -command ":choice-stack-event $win $name $ii"
          }
          lappend _$name.menu "$s"
          incr ii
          incr jj
        }
      }
      :menu {
        global _$name.menu
        set _$name.menu {}
        $win.menu.menu delete 0 end
        set ii 0
        foreach s $vl {
          if {[llength $s] <= 1} then {
            $win.menu.menu add command -label "$s" \
               -command ":choice-stack-event $win $name $ii"
            lappend _$name.menu "$s"
            incr ii
          } else {
            set c [first $s]
            set jj $ii
            $win.menu.menu add cascade -label "$c" -menu $win.menu.menu.c$jj
            set s [lreplace $s 0 0]
            catch {menu $win.menu.menu.c$jj -tearoff true}
            :_set-font $win.menu.menu.c$jj menu 0 N
            $win.menu.menu.c$jj delete 0 end
            foreach ss $s {
               $win.menu.menu.c$jj add command -label "$ss" \
                  -command ":choice-stack-event $win $name $ii"
               lappend _$name.menu "$ss"
               incr ii
            }
         }
       }
      }
      default {puts stderr ":modify_choice-stack - unknown arg $p"}
    }
    set i [expr $i + 2]
  }

}

proc :query_choice-stack {win name args} {
  global debug _$name.menu _rows

#    settings -    :label :rows :string :value :menu/strings

  set args [lindex $args 0]
  if {$debug} then {insert-debug ":query_choice-stack called with  $win $name $args"}

  switch [set p [lindex $args 0]] {
    :label {if {[info commands $win.label]!=""} {return [$win.label cget -text]}}
    :rows  {return [if ([info exists _rows($name)])  {set __rows $_rows($name)} {set __rows 10}]}
    :string  {return [$win.message cget -text]}
    :value  {return [lsearch [set _$name.menu] [$win.message cget -text]]}
    :menu -
    :strings  {return [set _$name.menu]}

    default {puts stderr ":query_choice-stack $win - unknown arg $p"}
  }
}


proc :canvas args {
############
############
  global debug _frame _panel _container _canvas _layout _xposn _yposn _placed _pack _colour

#    :canvas name :x ? :y ? :w ? :h ?

  set placed $_placed
  set xw 0; set xh 0

  set name [string tolower [lindex $args 0]]

  if {$_container == ""} then {
     if {$_panel == ""} then {
        set win $_frame.$name
     } else {
        set win $_frame.$_panel.$name
     }
  } else {
     if {$_panel == ""} then {
        set win $_frame.$_container.$name
     } else {
        set win $_frame.$_container.$_panel.$name
     }
  }

  global _classname _winname
  set _classname($name) "canvas"
  set _winname($name)  $win

  :_pre-updateposn;  set xp $_xposn; set yp $_yposn

  canvas $win -relief raised -borderwidth 1
  if {$debug} then {insert-debug ":canvas created $win"}
  set _canvas $win

  set i 1
  while {$i < [llength $args]} {
    set p [lindex $args $i]
    set v [lindex $args [expr $i + 1]]
    switch $p {

      :x  {set xp $v; set placed 1}
      :y  {set yp $v; set placed 1}
      :w  {set xw $v}
      :h  {set xh $v}

      default {puts stderr ":canvas - unknown arg $p"}
    }
    incr i 2
  }

  $win configure -width $xw -height $xh

  :_set-color $win canvas

  :_position-widget $placed $win $xp $yp

  :_post-updateposn $placed $win $xp $yp

}

proc :modify_canvas {win name args} {
  global debug

#    options -    :delete T|F :w ? :h ?

  set args [lindex $args 0]
  if {$debug} then {insert-debug ":modify_canvas called with  $win $name $args"}

  set i 0
  while {$i < [llength $args]} {
    set p [lindex $args $i]
    set v [lindex $args [expr $i + 1]]
    switch $p {

      :w {$win configure -width $v}
      :h {$win configure -height $v}
      :delete {
        switch $v {
          T {destroy $win}
          F {}
          default {puts stderr ":modify  :delete  - unknown option $v"}
        }
      }
      default {puts stderr ":modify_canvas - unknown arg $p"}
    }
    set i [expr $i + 2]
  }

}

proc :query_canvas {win name args} {
  global debug

#    settings -    :w :h

  set args [lindex $args 0]
  if {$debug} then {insert-debug ":query_canvas called with  $win $name $args"}

  switch [set p [lindex $args 0]] {
    :w {return [$win cget -width]}
    :h {return [$win cget -height]}

    default {puts stderr ":query_canvas $win - unknown arg $p"}
  }
}


proc :draw args {
  global debug _canvas

#    :draw object :colour ? :value ? :arrow ? :tag ? :delete T

  set colour ""; set arrow ""; set text ""

  set object [string tolower [lindex $args 0]]
  set win $_canvas

  set tag "$object"
  set cmd "$win create $object"
  set i 1
  while {$i < [llength $args]} {
    set p [lindex $args $i]
    set v [lindex $args [expr $i + 1]]
    switch $p {

      :colour {set colour $v}
      :arrow  {set arrow $v}
      :text   {set text "$v"}
      :tag    {set tag "$v"}
      :delete {$win delete $tag}
      :value  {
        append cmd " " $v
        incr i 2
        while {$i < [llength $args]} {
          append cmd " " [lindex $args $i]
          incr i
        }
      }

      default {puts stderr ":draw - unknown arg $p"}
    }
    incr i 2
  }

  if {$object == "modify"} then {return}

  set i [eval $cmd]
  $win itemconfigure $i -tags $tag
  if {$colour != ""} then {$win itemconfigure $i -fill $colour}
  if {$arrow != ""}  then {$win itemconfigure $i -arrow $arrow}
  if {$text != ""}   then {$win itemconfigure $i -text "$text" -anchor nw}

}

set spectrumTrace 0

proc :spectrum {gallery class action args} {
##################
##################
  global debug spectrumTrace _frame _panel _container  _xposn _yposn _placed
  global env

  set placed $_placed
  set xw 0; set xh 0

  set bold 1; set help "T"

#    :spectrum widget_name class action args

  if {$spectrumTrace} then {insert-debug "calling :spectrum $gallery $class $action $args"}


#    values for class
#        spectrum
#         create
#            clear
#            add      [name]
#            remove
#            sqeeeze
#            overlap  [name]
#            removeoverlaps
#            current    [|m|ptr]
#            title         [|n]
#            name      [|m]
#            save       [n]
#         get
#            overlay-dataset  [i j]
#            data      [base range counts]
#            linestyle   [0|1|2]
#            refresh
#            info
#            add-dataset   [i j]
#            size      [i j]|[i j k l]
#            calibrate [x | y] string

#        refresh
#            tags
#            calibrations


#        configure
#            -colourmap
#            -overlapcolourmap
#            -style
#            -select

#        cget

#        view
#            shift  [n]
#            scale  [n]
#            set    n m
#            reset
#            undo
#            redo
#            get
#            current

#        get
#            dimension
#            channel    [x y]  (x & y are coordinates)
#            count      [x y]
#            calibration   [x y]
#            channelx      [x y]
#            channely      [x y]
#            calibrationx   [x y]
#            calibrationy   [x y]
#            decalibrationx   [l]
#            decalibrationy   [l]
#            box


#        gate
#            pop  [m]
#            push [l m]
#            get  [m]
#            set  [x1,y1 x2,y2]
#            vertex hit [x y]
#            vertex add
#            vertex remove
#            vertex set
#            get2d
#            select
#            set2d

#        pointer
#            pop
#            get
#            push
#            set
#            size

#        tag
#            near x y
#            set  x y
#            get    n
#            push   n
#            pop    n
#            get2d  n
#            push2d x y
#            move

#        cut  x|y  chan  maxtrixptr  cutptr

  set name [string tolower $gallery]
  set helpitem $gallery

  switch $class {
    spectrum {

      switch $action {
        create {

  if {$_container == ""} then {
     if {$_panel == ""} then {
        set win $_frame.$name
        set master $_frame
     } else {
        set win $_frame.$_panel.$name
        set master $_frame.$_panel
     }
  } else {
     if {$_panel == ""} then {
        set win $_frame.$_container.$name
        set master $_frame.$_container
     } else {
        set win $_frame.$_container.$_panel.$name
        set master $_frame.$_container.$_panel
     }
  }

           global _classname _winname
           set _classname($name) "spectrum"
           set _winname($name)  $win

          :_pre-updateposn;  set xp $_xposn; set yp $_yposn

          spectrum $win

          if {$spectrumTrace} then {insert-debug ":spectrum created $win"}

          set i 0
          while {$i < [llength $args]} {
            set p [lindex $args $i]
            set v [lindex $args [expr $i + 1]]
            switch $p {

              :x  {set xp $v; set placed 1}
              :y  {set yp $v; set placed 1}
              :w  {set xw $v}
              :h  {set xh $v}
              :help {
                 switch $v {
                    T    {set help "T"}
                    F    {set help "F"}
                    default {:help $helpitem $v}
                 }
              }
              :helpitem {set helpitem $v}

              default {puts stderr ":spectrum create - unknown arg $p"}
            }
            incr i 2
          }

          $win configure -width $xw -height $xh

          if {$help == "T"} then {:_help $helpitem $win Spectrum}
          :_set-color $win spectrum

          :_set-font $win spectrum 0 N

          if {[info exists env(tkspectrumspeccolour)] == 1} then {
             set map [fourth [$win configure -overlapcolourmap]]
             $win configure -overlapcolourmap [lreplace $map 0 0 $env(tkspectrumspeccolour)]
          }

          :_position-widget $placed $win $xp $yp
          pack propagate $master no

          :_post-updateposn $placed $win $xp $yp

        }
        overlap -
        removeoverlaps -
        removelastoverlap -
        removealloverlaps -
        add -
        remove -
        squeeze -
        current -
        title -
        size -
        name -
        save -
        get -
        overlap-dataset -
        data -
        linestyle -
        refresh -
        info  -
        add-dataset  -
        calibrate  -
        clear {

          global _winname
          set win $_winname($name)
          set reply [eval $win $class $action $args]
          if {$spectrumTrace} then {insert-debug "$win $class $action $args (reply $reply)"}
          return $reply
        }

        default {insert-debug ":spectrum  - unknown action $action for class spectrum"}
      }
    }
    configure -
    view -
    get -
    refresh -
    tag -
    gate -
    pointer -
    cut -
    cget {

       global _winname
       set win $_winname($name)
       set reply [eval $win $class $action $args]
       if {$spectrumTrace} then {insert-debug "$win $class $action $args (reply $reply)"}
       return $reply
    }

    default {insert-debug ":spectrum  - unknown class $class"}
  }
}


:initialise

#:frame [self frame-name] :label "default frame for [Get_My_Name]" :transient T

:load-help

